'****************************************************************
'*  Name    : SM_IntHandler01.inc                               *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2017 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 05/06/2017                                        *
'*  Version : 1.0                                               *
'*  Notes   : Interrupt Handler for State Machine Mulitasking   *
'*          :                                                   *
'****************************************************************


$ifndef _SM_SM_IntHandlerInc_
$define _SM_SM_IntHandlerInc_
'-------------------------------------------------------------------------------

'*******************************************************************************
'* Name    : High Interrupt_Handler                                            *
'* Purpose : Generate a 1mS interrupt                                          *
'* Notes   : Using the Timer2 & PR2 it is more easy.                           *
'*******************************************************************************     
    On_Hardware_Interrupt GoTo Interrupt_Handler
'-------------------------------------------------------------------------------
    Declare Reminders = OFF
Interrupt_Handler:
    Context Save 
    '---------------------------------------------------------------------------
    If PIR1Bits_TMR2IF = 1 Then     ' Check if this is a timer2 interrupt?
        PIR1Bits_TMR2IF = 0         ' Clear the Timer2 interrupt flag for next time.
        '-----------------------------------------------------------------------
        $ifdef _PinsTest_
        Toggle bTest1
        $endif
        $ifdef _SynchronizeMSFunction_
        F_Timer1mS = 1             ' Set a Flag every 1mS
        $endif
        '-----------------------------------------------------------------------
        ' Configurable generic Timer to read the ADC channel.
        $if TaskADC_Interval > 0
        Inc TaskADC_Timer           ' Increment every 1 mS.
        If TaskADC_Timer = TaskADC_Interval Then
            TaskADC_Timer = 0
            If F_DisableADCInt = 0 Then
                ADCON0bits_GO_DONE = 1  ' Start an AD conversion.
            EndIf
            $ifdef _PinsTest_
            Set bTest2              ' To check the duration of the ADC + averages + Maths
            $endif        
        EndIf
        $endif
        '-----------------------------------------------------------------------
        ' Start the PWM LCD Backlight, every 10mS.
        $if TaskBackLight_Interval > 0
        Inc TaskBackLight_Timer       ' Increment every 1 mS.
        If TaskBackLight_Timer = TaskBackLight_Interval Then
            TaskBackLight_Timer = 0
            High bLCD_BackLight
            Set F_PWMBackLight
            PWMBLCounter = PWMBLCounterValue
        EndIf
        ' Count down the PWM BackLight.
        ' Do every mS.
        If F_PWMBackLight = 1 Then
            Dec PWMBLCounter           ' Decrements the DelayCounter every 1mS.
            If PWMBLCounter = 0 Then   ' Check if delay counter reaches 0.
                F_PWMBackLight = 0     ' Delay End, clear the delay FLAG.
                Low bLCD_BackLight
            EndIf        
        EndIf
        $endif       
        '-----------------------------------------------------------------------
        ' Toggle a flag for Blinking LEds. Generic Timer.
        $if TaskToggleFlag1_Interval > 0
        Inc TaskToggleFlag1_Timer       ' Increment every 1 mS.
        If TaskToggleFlag1_Timer = TaskToggleFlag1_Interval Then
            TaskToggleFlag1_Timer = 0
            Toggle F_ToggleSignal1        ' 1 Hz Timer, to toggle some leds.
        EndIf
        $endif
        '-----------------------------------------------------------------------
        ' 1 Second flag to read the clock/calendar
        $if Task1S_Interval > 0
        Inc Task1S_Timer       ' Increment every 1 mS.
        If Task1S_Timer = Task1S_Interval Then
            Task1S_Timer = 0
            F_TaskTimer1S = 1           ' For Main state05
        EndIf
        $endif
        '-----------------------------------------------------------------------
        $ifdef _EnableSM1VDelay_
        ' Virtual Delay 1mS for the SM1 States.
        If F_SM1VirtualDelay = 1 Then   ' The VDelay is started.
            Dec SM1VDelayCounter         ' Decrements the SM1VDelayCounter every 1mS.
            If SM1VDelayCounter = 0 Then ' Check if delay counter reaches 0.
                F_SM1VirtualDelay = 0   ' Delay END, clear the Vdelay FLAG.
            EndIf
        EndIf
        $endif  
        '-----------------------------------------------------------------------
        $ifdef _EnableSM2VDelay_
        ' Virtual Delay 1mS for the SM2 States.
        If F_SM2VirtualDelay = 1 Then     ' The VDelay is started.
            Dec SM2VDelayCounter          ' Decrements the SM1VDelayCounter every 1mS.
            If SM2VDelayCounter = 0 Then  ' Check if delay counter reaches 0.
                F_SM2VirtualDelay = 0     ' Delay END, clear the Vdelay FLAG.
                SM2Task[SM2DefVDelayTask] = SM2VDelayStateIndex  ' Loading the new Task.
            EndIf
        EndIf
        $endif  
        '-----------------------------------------------------------------------
        ' Delay counter for the Delayed task.
        $ifdef _EnableSM2DelayedState_
        If F_SM2DelayedTask = 1 Then        ' The Delay is started.
            Dec SM2TaskDelayCounter         ' Decrements the DelayCounter every 1mS.
            If SM2TaskDelayCounter = 0 Then ' Check if delay counter reaches 0.
                F_SM2DelayedTask = 0          ' Delay End, clear the delay FLAG.
                SM2Task[SM2DefDelayedTask] = SM2DelayedState    ' Loading the Delayed Task.
            EndIf
        EndIf
        $endif
        '-----------------------------------------------------------------------
        ' Delay counter for the StartPulse(PortPin,Time) command.
        $ifdef _EnablePulsePin_
        ' Count every 1mS.
        If F_PulseDelay = 1 Then       ' The Delay is started.
            Dec PulseCounter           ' Decrements the DelayCounter every 1mS.
            If PulseCounter = 0 Then   ' Check if delay counter reaches 0.
                F_PulseDelay = 0       ' Delay End, clear the delay FLAG.
            EndIf
        EndIf
        $endif  
        '-----------------------------------------------------------------------
        ' Delay for time out system.
        $ifdef _EnableTimeOutSystem_
        ' Count every 1mS.
        If F_TimeOutDelay = 1 Then       ' The Delay is started.
            Dec TimeOutCounter           ' Decrements the DelayCounter every 1mS.
            If TimeOutCounter = 0 Then   ' Check if delay counter reaches 0.
                F_TimeOutDelay = 0       ' Delay End, clear the delay FLAG.
            EndIf
        EndIf
        $endif          
        '----------------------------------------------------------------------- 
        ' Generic User Delay1
        $ifdef _EnableUserDelay1_
        ' Count every 1mS.
        If F_Delay1 = 1 Then            ' The Delay is started.
            Dec Delay1Counter           ' Decrements the DelayCounter every 1mS.
            If Delay1Counter = 0 Then   ' Check if delay counter reaches 0.
                F_Delay1 = 0            ' Delay End, clear the delay FLAG.
            EndIf
        EndIf
        $endif  
        '-----------------------------------------------------------------------
        ' Generic User Delay2
        $ifdef _EnableUserDelay2_
        ' Count every 1mS.
        If F_Delay2 = 1 Then            ' The Delay is started.
            Dec Delay2Counter           ' Decrements the DelayCounter every 1mS.
            If Delay2Counter = 0 Then   ' Check if delay counter reaches 0.
                F_Delay2 = 0            ' Delay End, clear the delay FLAG.
            EndIf
        EndIf
        $endif  
        '-----------------------------------------------------------------------
        ' Generic User Delay3
        $ifdef _EnableUserDelay3_
        ' Count every 1mS.
        If F_Delay3 = 1 Then            ' The Delay is started.
            Dec Delay3Counter           ' Decrements the DelayCounter every 1mS.
            If Delay3Counter = 0 Then   ' Check if delay counter reaches 0.
                F_Delay3 = 0            ' Delay End, clear the delay FLAG.
            EndIf
        EndIf
        $endif  
        '-----------------------------------------------------------------------
    EndIf    
'-------------------------------------------------------------------------------
    ' Waiting for the ADC interrupt reduces more than 10uS the time of the interrupt handler.
    $if TaskADC_Interval > 0
    If PIR1Bits_ADIF = 1 Then
        PIR1Bits_ADIF = 0
        ' Now the ADC reading is available in ADRESL/ADRESH.
        ' Load the ADC average task into the buffer in the predefined position.
        LoadTask(SM2_ADCaverage)
        ' Increment a value to syncronize with the ADC task.
        $ifdef _CheckADCSystem_
        Inc ADCCounter
        $endif
    EndIf
    $endif
'-------------------------------------------------------------------------------
    ' Signal 1Hz interrupt from the DS1307 clock calendar chip.
    $ifdef _EnableSMTimer0_
    ' PORTA.4 TMR0 Interrupt
    ' The Timer0 interrupt is used at High to Low edge interrupt.
    If INTCONBits_TMR0IF = 1 Then
        T0CONBits_TMR0ON = 0    ' Stop the Timer0.
        Nop
        TMR0L = 255             ' Reload the Timer0 for next interrupt.
        T0CONBits_TMR0ON = 1    ' Restart the Timer0.
        INTCONBits_TMR0IF = 0
        F_DS1307_1S = 1         ' External flag for DS1307 interrupt.
    EndIf
    $endif
'-------------------------------------------------------------------------------
' Here the another user interrupt handler code....
' .......
' .......
' .......
'-------------------------------------------------------------------------------
END_INT:
'    $ifdef _PinsTest_
'    Clear bTest1
'    $endif
'-------------------------------------------------------------------------------
    Context Restore          ' Exit the interrupt handler and restore variables
    Declare Reminders = On
'*******************************************************************************
$endif
'===============================================================================

