'****************************************************************
'*  Name    : SM_PIC18F25K20.inc                                *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2017 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 23/05/2017                                        *
'*  Version : 1.0                                               *
'*  Notes   : Declare all the parameters of the PIC used.       *
'*          : PIC Definitions CPU = PIC18F25K20                 *
'*          : with Xtal 16Mhz, with 4xPLL                       *
'****************************************************************
' STATE MACHINE WITH MULTITASKS SYSTEM.
'===============================================================================
    Device = 18F25K20
'-------------------------------------------------------------------------------
    $ifdef _AMicus18_80Mhz_
    ' Settings CPU Amicus18 Board at (20Mhz) + PLL (with settings fuses)
    Declare Xtal = 80
    $else
        $ifdef _ProteusDebug_
        ' Settings CPU Proteus at 4Mhz + PLL (with settings fuses)
        Declare Xtal = 16
        $else
        ' Settings CPU Amicus18 Board at (16Mhz ) + PLL (with settings fuses)
        Declare Xtal = 64
        $endif    
    $endif
'*******************************************************************************
'* Name    : Compiler Declare                                                  *
'* Purpose : Configuration of the compiler                                     *
'* Notes   :                                                                   *
'******************************************************************************* 
    $if _defined(_AGV_LSM_BootloaderV41_) And _defined(_AmicusBootloader_)
        $error "CHOOSE ONE BOOTLOADER ONLY"
    $endif
    '---------------------------------------------------------------------------
    ' More security using the Bootloader in the Low Side of ROM memory (only for real piece).
    $ifndef _ProteusDebug_
    $ifdef _AGV_LSM_BootloaderV41_
    Declare PROTON_START_ADDRESS = 2048
    $endif
    $endif
    '---------------------------------------------------------------------------
    Declare Eeprom_Address = $F00000
    Declare Optimiser_Level = 2
    Declare Dead_Code_Remove = On
    $ifdef _ProteusDebug_
    Declare Create_Coff = On
    $endif
    Declare Watchdog = OFF
'    Declare Float_Display_Type = Fast
'    Declare Float_Rounding = Off
    Declare Bootloader = OFF
    Declare Show_System_Variables = On
    Declare MemWrite_Int_Control = 1
    $if 65537 < _code
    Declare Access_Upper_64K = On
    $endif
'    Declare Keypad_Port PORTB

    $ifndef True
        $define True 1
    $endif
    $ifndef False
        $define False 0
    $endif
'*******************************************************************************
'* Name    : Config Fuses                                                   
'* Purpose :                                                                  
'* Notes   : Version for Proteus/Amicus18 & another version for AGV LSM Bootloader
'******************************************************************************* 
$if _defined(_ProteusDebug_) Or _defined(_AmicusBootloader_)
'-------------------------------------------------------------------------------
Config_Start
    FOSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC1)
    FCMEN = OFF	    ;Fail-Safe Clock Monitor disabled
    IESO = OFF	 ;Oscillator Switchover mode disabled
    PWRT = On	  ;PWRT enabled
    BOREN = On	 ;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 22	  ;VBOR set to 2.2 V nominal
    WDTEN = OFF	    ;WDT is controlled by SWDTEN bit of the WDTCON register
    WDTPS = 128	    ;1:128
    CCP2MX = PORTC	;CCP2 input/output is multiplexed with RC1
    PBADEN = On	    ;PORTB<4:0> pins are configured as analog input channels on Reset
    LPT1OSC = OFF	;Timer1 configured for higher power operation
    HFOFST = OFF	;The system clock is held off until the HFINTOSC is stable.
    MCLRE = On	     ;MCLR pin enabled; RE3 input pin disabled
    STVREN = On	    ;Stack full/underflow will cause Reset
    LVP = OFF	  ;Single-Supply ICSP disabled
    XINST = OFF	    ;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    Debug = OFF	    ;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
    Cp0 = OFF	;Block 0 (000800-001FFFh) not code-protected
    CP1 = OFF	;Block 1 (002000-003FFFh) not code-protected
    CP2 = OFF	;Block 2 (004000-005FFFh) not code-protected
    CP3 = OFF	;Block 3 (006000-007FFFh) not code-protected
    CPB = OFF	;Boot block (000000-0007FFh) not code-protected
    CPD = OFF	;Data EEPROM not code-protected
    WRT0 = OFF	;Block 0 (000800-001FFFh) not write-protected
    WRT1 = OFF	;Block 1 (002000-003FFFh) not write-protected
    WRT2 = OFF	;Block 2 (004000-005FFFh) not write-protected
    WRT3 = OFF	;Block 3 (006000-007FFFh) not write-protected
    WRTC = On	;Configuration registers (300000-3000FFh) write-protected
    WRTB = OFF	;Boot Block (000000-0007FFh) not write-protected
    WRTD = OFF	;Data EEPROM not write-protected
    EBTR0 = OFF	;Block 0 (000800-001FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (002000-003FFFh) not protected from table reads executed in other blocks
    EBTR2 = OFF	;Block 2 (004000-005FFFh) not protected from table reads executed in other blocks
    EBTR3 = OFF	;Block 3 (006000-007FFFh) not protected from table reads executed in other blocks
    EBTRB = OFF	;Boot Block (000000-0007FFh) not protected from table reads executed in other blocks
Config_End
'-------------------------------------------------------------------------------
$else
'-------------------------------------------------------------------------------
' Full Boot Block protected for the AGV Low Side ROM Bootloader.
Config_Start
    FOSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC1)
    FCMEN = OFF	;Fail-Safe Clock Monitor disabled
    IESO = OFF	;Oscillator Switchover mode disabled
    PWRT = On	;PWRT enabled
    BOREN = On	;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 22	;VBOR set to 2.2 V nominal
    WDTEN = OFF	;WDT is controlled by SWDTEN bit of the WDTCON register
    WDTPS = 128	;1:128
    CCP2MX = PORTC	;CCP2 input/output is multiplexed with RC1
    PBADEN = On	    ;PORTB<4:0> pins are configured as analog input channels on Reset
    LPT1OSC = OFF	;Timer1 configured for higher power operation
    HFOFST = OFF	;The system clock is held off until the HFINTOSC is stable.
    MCLRE = On	;MCLR pin enabled; RE3 input pin disabled
    STVREN = On	;Stack full/underflow will cause Reset
    LVP = OFF	;Single-Supply ICSP disabled
    XINST = OFF	;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    Debug = OFF	;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
    Cp0 = OFF	;Block 0 (000800-001FFFh) not code-protected
    CP1 = OFF	;Block 1 (002000-003FFFh) not code-protected
    CP2 = OFF	;Block 2 (004000-005FFFh) not code-protected
    CP3 = OFF	;Block 3 (006000-007FFFh) not code-protected
    CPB = On	;Boot block (000000-0007FFh) code-protected
    CPD = On	;Data EEPROM code-protected
    WRT0 = OFF	;Block 0 (000800-001FFFh) not write-protected
    WRT1 = OFF	;Block 1 (002000-003FFFh) not write-protected
    WRT2 = OFF	;Block 2 (004000-005FFFh) not write-protected
    WRT3 = OFF	;Block 3 (006000-007FFFh) not write-protected
    WRTC = On	;Configuration registers (300000-3000FFh) write-protected
    WRTB = On	;Boot Block (000000-0007FFh) write-protected
    WRTD = OFF	;Data EEPROM not write-protected
    EBTR0 = OFF	;Block 0 (000800-001FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (002000-003FFFh) not protected from table reads executed in other blocks
    EBTR2 = OFF	;Block 2 (004000-005FFFh) not protected from table reads executed in other blocks
    EBTR3 = OFF	;Block 3 (006000-007FFFh) not protected from table reads executed in other blocks
    EBTRB = On	;Boot Block (000000-0007FFh) protected from table reads executed in other blocks
Config_End
'-------------------------------------------------------------------------------
$endif
'===============================================================================
(*
' AGV LSM Bootloader V4.1:
Config_Start
    FOSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC1)
    FCMEN = OFF	;Fail-Safe Clock Monitor disabled
    IESO = OFF	;Oscillator Switchover mode disabled
    PWRT = On	;PWRT enabled
    BOREN = On	;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 22	;VBOR set to 2.2 V nominal
    WDTEN = OFF	;WDT is controlled by SWDTEN bit of the WDTCON register
    WDTPS = 128	;1:128
    CCP2MX = PORTC	;CCP2 input/output is multiplexed with RC1
    PBADEN = On	;PORTB<4:0> pins are configured as analog input channels on Reset
    LPT1OSC = OFF	;Timer1 configured for higher power operation
    HFOFST = OFF	;The system clock is held off until the HFINTOSC is stable.
    MCLRE = On	;MCLR pin enabled; RE3 input pin disabled
    STVREN = On	;Stack full/underflow will cause Reset
    LVP = OFF	;Single-Supply ICSP disabled
    XINST = OFF	;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    Debug = OFF	;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
    Cp0 = OFF	;Block 0 (000800-001FFFh) not code-protected
    CP1 = OFF	;Block 1 (002000-003FFFh) not code-protected
    CP2 = OFF	;Block 2 (004000-005FFFh) not code-protected
    CP3 = OFF	;Block 3 (006000-007FFFh) not code-protected
    CPB = On	;Boot block (000000-0007FFh) code-protected
    CPD = On	;Data EEPROM code-protected
    WRT0 = OFF	;Block 0 (000800-001FFFh) not write-protected
    WRT1 = OFF	;Block 1 (002000-003FFFh) not write-protected
    WRT2 = OFF	;Block 2 (004000-005FFFh) not write-protected
    WRT3 = OFF	;Block 3 (006000-007FFFh) not write-protected
    WRTC = On	;Configuration registers (300000-3000FFh) write-protected
    WRTB = On	;Boot Block (000000-0007FFh) write-protected
    WRTD = OFF	;Data EEPROM not write-protected
    EBTR0 = OFF	;Block 0 (000800-001FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (002000-003FFFh) not protected from table reads executed in other blocks
    EBTR2 = OFF	;Block 2 (004000-005FFFh) not protected from table reads executed in other blocks
    EBTR3 = OFF	;Block 3 (006000-007FFFh) not protected from table reads executed in other blocks
    EBTRB = On	;Boot Block (000000-0007FFh) protected from table reads executed in other blocks
Config_End
*)
'===============================================================================
' My Bits definition of the PIC18F25K20.
    Include "D:\MyIncFiles\SYS_18FXXK20.inc"
'    Include "INC\SYS_18FXXK20.inc"
'*******************************************************************************
PIC_INIT:
    Dim SM_Errors As Byte
    Dim F_Reset As SM_Errors.0   
    Dim F_Watchdog As SM_Errors.1
    Dim F_BrownOut As SM_Errors.2    
    Dim F_StackFull As SM_Errors.3
    Dim F_StackUnderflow As SM_Errors.4
'===============================================================================
    SM_Errors = 0
' Copy this code in every SM PIC files.
' At Power up control the state of the PIC.
    ' Check the Reset Flag bit
    If RCONBits_RI = 0 Then
        RCONBits_RI = 1
        HRSOut "RESET",CR,LF
        F_Reset = 1
    EndIf
    ' Check the Watchdog Time-out Flag bit
    If RCONBits_TO = 0 Then
        HRSOut "WATCHDOG",CR,LF
        F_Watchdog = 1
    EndIf
    ' Check the Brown-out Reset Status bit
    If RCONBits_POR = 1 Then
        If RCONBits_BOR = 0 Then
            RCONBits_BOR = 1
            HRSOut "BROWN-OUT",CR,LF
            F_BrownOut = 1
        EndIf
    EndIf
    ' Check the Power-on Reset Status bit
    If RCONBits_POR = 0 Then
        RCONBits_POR = 1
        HRSOut "POWER-ON",CR,LF
    EndIf 
    ' Check the Stack pointer register
    If STKPTRBits_STKFUL = 1 Then
        STKPTRBits_STKFUL = 0
        HRSOut "STACK FULL",CR,LF
        F_StackFull = 1
    EndIf
    If STKPTRBits_STKUNF = 1 Then
        STKPTRBits_STKUNF = 0
        HRSOut "STACK UNDERFLOW",CR,LF
        F_StackUnderflow = 1
    EndIf
'===============================================================================
    Clrwdt
    WDTCON.0 = 0            ' Disable the Watchdog for the moment.
    INTCON = 0              ' Disable All interrupts
    TBLPTRU = 0
    INTCON2Bits_RBPU = 1    ' Disable internal PullUp
    WPUB = 0                ' Pullups disabled in PORTB
    T0CON = 0
    '---------------------------------------------------------------------------
' Init the PORTS to digital
'    ANSEL = 0
'    ANSELH = 0
'    CM1CON0 = 0     ' COmparator 1 disabled.
'    CM2CON0 = 0     ' COmparator 2 disabled.
'*******************************************************************************

'===============================================================================
PIC_INITEXIT:
'===============================================================================

