'****************************************************************
'*  Name    : TC74-H_beta.inc                                   *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) July 2013 [AFreixanet 2013]         *
'*          : All Rights Reserved                               *
'*  Date    : 17/02/2016                                        *
'*  Version : 1.0                                               *
'*  Notes   : Driver for the I2C TC74 Temperature Sensor        *
'*          : using the hardware I2C of PIC(r). MSSP MASTER MODE*
'* Compiler : Proton Basic and Amicus18                         *
'*          : Code especially wrote for PIC18F25K20/PIC18F25K22 *
'****************************************************************
'
' Version 1.1 26/02/2016
' Option to disable the GIE_Save() & GIE_Restore()code.
' Fixed error interrupt with NACK I2C bus.
' New security for Definition of pins of the I2C Bus.
'
' Version 1.2 13/03/2016
' Fixed error in the OrgTemperature variable to save the original value.
' Added configuration of the I2C Bus pins to digital for PIC18F2xK22 series.
' Added the TC74_Present() command.
'
' Version 1.3 09/05/2017
' The TC74_NormalMode(SensorAddress) cannot have a delay for State Machine System
'
'-------------------------------------------------------------------------------
'===============================================================================
' This program is free software, you can redistribute it and/or modify.
' The code is offered as-is and the author take any responsibility for its use.
' This program is distributed in the hope that it will be useful, but WITHOUT ANY
' WARRANTY. However, a mention of the author and compiler would be appreciated.
'
' Este programa es libre, puede redistribuirlo y/o modificarlo.
' Se entrega este cdigo tal como esta y el autor NO se hace responsable de
' su utilizacin. Este programa se distribuye con la esperanza de que sea til,
' pero SIN NINGUNA GARANTA. No obstante citar el autor y el compilador
' sera muy apreciado.

' Ce programme est un logiciel libre, vous pouvez le redistribuer et/ou modifier.
' Ce logiciel est offert tel qu'il est, et son auteur NE prend AUCUNE responsabilit
' pour son utilization. Ce programme est distribu dans l'espoir qu'il sera util,
' mais SANS AUCUNE GARANTIE. Cependant mentionner l'auteur et le compileur serait
' aprci.
'===============================================================================
'===============================================================================
' Tiny Serial Digital Thermal Sensor
' (r) 2002 Microchip Technology Inc. DS21462
'
' The TC74 is a serially accessible, digital temperature sensor particularly suited
' for low cost and small formfactor applications. Temperature data is converted from
' the onboard thermal sensing element and made available as an 8-bit digital word.
' Communication with the TC74 is accomplished via a 2-wire SMBus/I2C compatible
' serial port. This bus also can be used to implement multi-drop/multi-zone monitoring.
' The SHDN bit in the CONFIG register can be used to activate the low power Standby mode.
' Temperature resolution is 1C. Conversion rate is a nominal 8 samples/sec.
' During normal operation, the quiescent current is 200 uA (typ).
' During standby operation, the quiescent current is 5 uA (typ).
' Small size, low installed cost and ease of use make the TC74 an ideal choice
' for implementing thermal management in a variety of systems.
'
'Features
'    Digital Temperature Sensing in SOT-23-5 or TO-220 Packages
'    Outputs Temperature as an 8-Bit Digital Word
'    Simple SMBus/I2C Serial Port Interface
'    Solid-State Temperature Sensing:
'       - 2C (max.) Accuracy from +25C to +85C
'       - 3C (max.) Accuracy from 0C to +125C
'    Supply Voltage of 2.7V to 5.5V
'    Low Power:
'       - 200 uA (typ.) Operating Current
'       - 5 A (typ.) Standby Mode Current
'
' Applications
'    Thermal Protection for Hard Disk Drives and other PC Peripherals.
'    PC Card Devices for Notebook Computers
'    Low Cost Thermostat Controls
'    Power Supplies
'    Thermistor Replacement
'
'===============================================================================
' LIBRARY COMMANDS:
' This is a libary using the Hardware I2C commands of the Amicus18 compiler.
' Some new commands are added to make secure the sensor readings.
' All routines are compatible with the PIC18F25K20 and PIC18F25K22 for the Amicus18
' board. For others PICs please check datatsheet.
'===============================================================================
'-------------------------------------------------------------------------------       
'*******************************************************************************
$ifndef _TC74H_INC_
    $define _TC74H_INC_
'-------------------------------------------------------------------------------
    GoTo OverDriverTC74
'*******************************************************************************    
'-------------------------------------------------------------------------------
' Uncomment the next line to disable the NACK function.
'$define _DisableNACK_

' Uncomment the next line to enable the interrupt protection.
'$define _EnableGIE_Save_

'-------------------------------------------------------------------------------
' Function Name: Definicin de las direcciones de los diferentes modelos de sensor.
' Input        : 
' Output       : 
' Notes        :
'
    Dim TC74_I2C_Flags As Byte System
    Dim TC74_ACK As TC74_I2C_Flags.0
    Dim ModeNormalSent As TC74_I2C_Flags.1
    Dim TC74_FirstValidAddress As TC74_I2C_Flags.2
    
    Dim xTC74_Result As Byte System
    Dim xTC74_Address As Byte System
    Dim xTC74_AddressWrite As Byte System 
    Dim xTC74_AddressRead As Byte System 
    Dim xACKReturn As Byte System
    Dim xTCSign As Byte System
    Dim OrgTemperature As Byte System
'-------------------------------------------------------------------------------
' Addresses of diferents standard temperature sensors. 1001 101  xTC74_AddressRead
    $define TC74A0 %01001000
    $define TC74A1 %01001001
    $define TC74A2 %01001010
    $define TC74A3 %01001011
    $define TC74A4 %01001100
    $define TC74A5 %01001101
    $define TC74A6 %01001110
    $define TC74A7 %01001111

    $define TC74A0_ADRRead %10010001
    $define TC74A1_ADRRead %10010011
    $define TC74A2_ADRRead %10010101    
    $define TC74A3_ADRRead %10010111
    $define TC74A4_ADRRead %10011001
    $define TC74A5_ADRRead %10011011
    $define TC74A6_ADRRead %10011101
    $define TC74A7_ADRRead %10011111

    $define TC74A0_ADRWrite %10010000
    $define TC74A1_ADRWrite %10010010
    $define TC74A2_ADRWrite %10010100    
    $define TC74A3_ADRWrite %10010110
    $define TC74A4_ADRWrite %10011000
    $define TC74A5_ADRWrite %10011010
    $define TC74A6_ADRWrite %10011100
    $define TC74A7_ADRWrite %10011110

    $define TC74_RTR     $00    ' Command: Read Temperature (TEMP REG)
    $define TC74_RWCR    $01    ' Command: Read/Write Configuration (CONFIG REG)
    $define TC74_STANDBY $80    ' Setting: Standby value 
    $define TC74_NORMAL  $00    ' Setting: Normal value   
'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
' Function Name: Parameters Definition of the I2C Bus.
' Input        : 
' Output       : 
' Notes        : 

#ifnSym __SDA_Port
    Declare SDA_Pin PORTC.4           ' SDA connected to this pin
#endIfSym
#ifnSym __SCL_Port
    Declare SCL_Pin PORTC.3           ' SCL connected to this pin
    Declare Hbus_Bitrate 100          ' Bus frequency 100kHz for the TC74 sensor 
    Declare Slow_Bus On   
#endIfSym

'-------------------------------------------------------------------------------
'*******************************************************************************
' Enable this line in the main program before this library to enable interrupt save option.
'    $define _EnableGIE_Save_

'-------------------------------------------------------------------------------
$ifdef _EnableGIE_Save_
'-------------------------------------------------------------------------------
' Declare a variable for INTCON copy.
    $ifndef _xINTCON_Save_Var_
    $define _xINTCON_Save_Var_
    Dim xINTCON_Save As Byte System
    $endif



' This new command save and clear the GIE global interrupt.
$ifndef GIE_Save
    $define GIE_Save()          '              
    #ifdef __INTERRUPTS_ENABLED '
    xINTCON_Save = INTCON       '
    INTCONbits_GIE = 0          '
    #endif                      '

$endif

'-------------------------------------------------------------------------------    
' This new command restore the GIE global interrupt.
$ifndef GIE_Restore
    $define GIE_Restore()       '
    #ifdef __INTERRUPTS_ENABLED '
    If xINTCON_Save.7 = 1 Then  '
        INTCONbits_GIE = 1      '
    EndIf                       '
    #endif                      '
    
$endif
'-------------------------------------------------------------------------------
$endif
'*******************************************************************************
'-------------------------------------------------------------------------------
' Send command Read Mode. Write Byte Format
' Input : none
' Output: Return Var (optional)
' Syntax: TC74_WriteNormalMode(pACKReturn) 
' Notes : 
'
$define TC74_NormalMode(pValue, pACKReturn) _mTC74_WriteNMode pValue, pACKReturn

_mTC74_WriteNMode Macro- pValue, pACKReturn
    #if (Prm_Count > 2)
	   #error "_mTC74_WriteNMode - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 1)
            #error "_mTC74_WriteNMode - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mTC74_WriteNMode - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if(Prm_1 == Num8)
                    Num_Byte pValue, xTC74_Address                    
                #endif
                #if(Prm_1 == Byte)
                    Byte_Byte pValue, xTC74_Address                    
                #endif
                GoSub _mTC74_WriteNMode_Sub
                $ifdef _EnableGIE_Save_
                GIE_Restore()
                $endif
                $ifndef _DisableNACK_
                #if (Prm_Count == 2)
                    #if (Prm_2 != Byte)
                        #error "_mTC74_WriteNMode - (Param2) Must be a Byte!"
                        Exitm
                    #else
                        Byte_Byte xACKReturn, pACKReturn
                    #endif
                #endif    
                $endif
            #endif                
        #endif
    #endif                
Endm

#ifMacro- _mTC74_WriteNMode, _mTC74_SecureReadTemp, _mTC74_Present
_mTC74_WriteNMode_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    $ifndef _DisableNACK_
    xACKReturn = 0                  ' Set the NACK flag error 
    $endif
    GoSub SetTC74Addresses_Sub
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    HBStart
    HBusOut xTC74_AddressWrite      ' Address and Write  
    $ifndef _DisableNACK_
    $ifdef _SSPCON2
    If SSPCON2.6 = 1 Then           ' (ACKSTAT) Check ACK from the sensor
        HBStop
        xACKReturn = 1              ' Set A NACK error
        Return  
    EndIf
    $endif
    $ifdef _SSP1CON2    
    If SSP1CON2.6 = 1 Then          ' (ACKSTAT) Check ACK from the sensor
        HBStop
        xACKReturn = 1              ' Set A NACK error
        Return 
    EndIf
    $endif    
    $endif
    HBusOut TC74_RWCR               ' Point to the Config Register.
    HBusOut TC74_NORMAL             ' Send Value "Normal"
    HBStop
    '---------------------------------------------------------------------------
    ' NO delay for State Machine System.
    $ifndef _STMachine_inc_
    DelayMS 255                     ' Delay needed to setup the sensor.
    $endif
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Send command Standby Mode. Write Byte Format
' Input : 
' Output: Return Var (optional)
' Syntax: TC74_WriteStandByMode(TC74A4, pACKReturn)
' Notes : 
'
$define TC74_StandbyMode(pValue, pACKReturn) _mTC74_WriteStandbyMode pValue, pACKReturn

_mTC74_WriteStandbyMode Macro- pValue, pACKReturn
    #if (Prm_Count > 2)
	   #error "_mTC74_WriteStandByMode - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 1)
            #error "_mTC74_WriteStandByMode - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mTC74_WriteStandByMode - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if(Prm_1 == Num8)
                    Num_Byte pValue, xTC74_Address                    
                #endif
                #if(Prm_1 == Byte)
                    Byte_Byte pValue, xTC74_Address                    
                #endif
                GoSub _mTC74_WriteStandByMode_Sub
                $ifndef _DisableNACK_
                #if (Prm_Count == 2)
                    #if (Prm_2 != Byte)
                        #error "_mTC74_WriteStandbyMode - (Param2) Must be a Byte!"
                        Exitm
                    #else
                        Byte_Byte xACKReturn, pACKReturn
                    #endif
                #endif
                $endif               
            #endif                
        #endif
    #endif                  
Endm

#ifMacro- _mTC74_WriteStandbyMode
_mTC74_WriteStandByMode_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    $ifndef _DisableNACK_
    xACKReturn = 0              ' Clear the NACK flag error 
    $endif
    GoSub SetTC74Addresses_Sub
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    HBStart
    HBusOut xTC74_AddressWrite  ' TC74 Address and Write
    $ifndef _DisableNACK_
    $ifdef _SSPCON2
    If SSPCON2.6 = 1 Then       ' (ACKSTAT) Check ACK from the sensor
        HBStop
        xACKReturn = 1          ' Set A NACK error
        GoTo _mTC74_WriteSBModeExit  
    EndIf
    $endif
    $ifdef _SSP1CON2     
    If SSP1CON2.6 = 1 Then      ' (ACKSTAT) Check ACK from the sensor
        HBStop
        xACKReturn = 1          ' Set A NACK error
        GoTo _mTC74_WriteSBModeExit
    EndIf
    $endif  
    $endif
    HBusOut TC74_RWCR           ' Config Register Address
    HBusOut TC74_STANDBY        ' Loading the value to the Config Register.
    HBStop
_mTC74_WriteSBModeExit:
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name     : Read the Status of the sensor : Write Byte Format
' Input Variables   : 
' Output Variables  : Return Var
' Syntax            : Var = TC74_ReadStatus(TC74A4)
' Notes             : If sensor is not connected pACKReturn = 1
'
$define TC74_ReadStatus(pValue) _mTC74_ReadStatus pValue

_mTC74_ReadStatus Macro- pValue
    #if (Prm_Count > 1)
	   #error "_mTC74_ReadStatus - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 1)
            #error "_mTC74_ReadStatus - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mTC74_ReadStatus - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if(Prm_1 == Num8)
                    Num_Byte pValue, xTC74_Address                    
                #endif
                #if(Prm_1 == Byte)
                    Byte_Byte pValue, xTC74_Address                    
                #endif
                GoSub _mTC74_ReadStatus_Sub

                #if (_mTC74_ReadStatus_RETURN != 1)
                    #error "_mTC74_ReadStatus - Mandatory return parameter misssing"
                    Exitm
                #else
                    #if(Return_Type != Byte)
                        #error "_mTC74_ReadStatus - Return variable should be a Byte variable"
                        Exitm
                    #else
                        Byte_Byte xACKReturn, Return_Var
                    #endif
                #endif          
            #endif                
        #endif
    #endif
Endm

#ifMacro- _mTC74_ReadStatus, _mTC74_Present
_mTC74_ReadStatus_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    $ifndef _DisableNACK_
    xACKReturn = 0                  ' Clear the NACK error.
    $endif
    GoSub SetTC74Addresses_Sub
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    HBStart
    HBusOut xTC74_AddressWrite      ' TC74 Address and Write
    ' Check the ACK of the device.
    $ifndef _DisableNACK_
    $ifdef _SSPCON2
    If SSPCON2.6 = 1 Then       ' (ACKSTAT) Check ACK from the sensor
        HBStop
        xACKReturn = 1          ' Set A NACK error
        GoTo _mTC74_ReadStatusExit  
    EndIf
    $endif
    $ifdef _SSP1CON2    
    If SSP1CON2.6 = 1 Then      ' (ACKSTAT) Check ACK from the sensor
        HBStop
        xACKReturn = 1          ' Set A NACK error
        GoTo _mTC74_ReadStatusExit
    EndIf
    $endif   
    $endif
    HBusOut TC74_RWCR               ' Register to read : config register
    HBReStart
    HBusOut xTC74_AddressRead       ' TC74 Address and Read
    HBusIn  xACKReturn              ' Loading the result of the config register
    HBusNack                        ' Send a NACK to the sensor because it is a byte input.
    HBStop
_mTC74_ReadStatusExit:
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Read Byte Format. Read directly from the temperature register.
' Input : 
' Output: Return Var
' Syntax: Var = TC74_ReadTempReg(TC74A4, pSign, pACKReturn)
' Notes : If the sensor is not connected the ACKReturn = 1. 
'
$ifndef _TempOrgFormat_
$define TC74_ReadSRTemp(pValue, pSign, pACKReturn) _mTC74_ReadSRTemp pValue, pSign, pACKReturn

_mTC74_ReadSRTemp Macro- pValue, pSign, pACKReturn
    #if (Prm_Count > 3)
	   #error "_mTC74_ReadSRTemp - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 2)
            #error "_mTC74_ReadSRTemp - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mTC74_ReadSRTemp - (Param1) Should be a Byte or Num8"
                Exitm
            #endif
            #if (Prm_2 != Byte)
                #error "_mTC74_ReadSRTemp - (Param2) Should be a Byte"
                Exitm
            #endif
            #if(Prm_1 == Num8)
                Num_Byte pValue, xTC74_Address                    
            #endif
            #if(Prm_1 == Byte)
                Byte_Byte pValue, xTC74_Address                    
            #endif
            GoSub _mTC74_ReadSRTemp_Sub
            #if(Prm_2 == Byte)
                Wreg_Byte pSign
            #endif
            $ifdef _EnableGIE_Save_
            GIE_Restore()
            $endif

            #if (_mTC74_ReadSRTemp_RETURN != 1)
                #error "_mTC74_ReadSRTemp - Mandatory return parameter misssing"
                Exitm
            #else
                #if (Return_Type != Byte) && (Return_Type != Word)
                    #error "_mTC74_ReadSRTemp - Return variable should be a Byte or Word variable"
                    Exitm
                #else
                    #if (Return_Type == Byte)
                        Byte_Byte xTC74_Result, Return_Var
                    #endif
                    #if (Return_Type == Word)
                        Byte_Word xTC74_Result, Return_Var
                    #endif
                #endif
            #endif
            $ifndef _DisableNACK_
            #if (Prm_Count == 3)
                #if (Prm_3 != Byte)
                    #error "_mTC74_ReadSRTemp - (Param3) Must be a Byte!"
                    Exitm
                #else
                    Byte_Byte xACKReturn, pACKReturn
                #endif
            #endif
            $endif
        #endif
    #endif
Endm

#ifMacro- _mTC74_ReadSRTemp, _mTC74_SecureReadTemp
_mTC74_ReadSRTemp_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    $ifndef _DisableNACK_
    xACKReturn = 0              ' Clear the NACK error
    $endif
    OrgTemperature = 255        ' False result.
    xTC74_Result = 255
    GoSub SetTC74Addresses_Sub
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    ' Read the Status first.
    HBStart
    HBusOut xTC74_AddressWrite   ' TC74 Address and Write
    $ifndef _DisableNACK_
    $ifdef _SSPCON2
    If SSPCON2.6 = 1 Then       ' (ACKSTAT) Check ACK from the sensor
        HBStop
        xACKReturn = 1          ' Set A NACK error
        WREG = 0                ' The sensor is fault. +Sign and Return. 
        Return  
    EndIf
    $endif
    $ifdef _SSP1CON2     
    If SSP1CON2.6 = 1 Then      ' (ACKSTAT) Check ACK from the sensor
        HBStop
        xACKReturn = 1          ' Set A NACK error
        WREG = 0                ' The sensor is fault. +Sign and Return. 
        Return 
    EndIf
    $endif 
    $endif
    HBusOut TC74_RWCR            ' Point to the config register
    HBReStart
    HBusOut xTC74_AddressRead    ' TC74 Address and Read
    HBusIn  xACKReturn           ' Loading the result of the config register
    HBusNack                     ' Send a NACK to the sensor because it is a byte input.
    HBStop
    If xACKReturn.6 = 0 Then    ' Is the sensor ready?
        WREG = 0                ' NO, the sensor is NOT Ready. +Sign and Return.   
        Return
    EndIf
    '---------------------------------------------------------------------------
    DelayUS 10
    '---------------------------------------------------------------------------
    HBStart
    HBusOut xTC74_AddressWrite
    $ifndef _DisableNACK_
    $ifdef _SSPCON2
    If SSPCON2.6 = 1 Then       ' (ACKSTAT) Check ACK from the sensor
        HBStop
        xACKReturn = 1          ' Set A NACK error
        WREG = 0                ' The sensor is fault. +Sign and Return. 
        Return  
    EndIf
    $endif
    $ifdef _SSP1CON2   
    If SSP1CON2.6 = 1 Then      ' (ACKSTAT) Check ACK from the sensor
        HBStop
        xACKReturn = 1          ' Set A NACK error
        WREG = 0                ' The sensor is fault. +Sign and Return. 
        Return 
    EndIf
    $endif 
    $endif
    HBusOut TC74_RTR            ' Point to TEMP register $00
    HBReStart
    HBusOut xTC74_AddressRead
    HBusIn  OrgTemperature      ' Loading the result.
    HBusNack
    HBStop
    '---------------------------------------------------------------------------
    xTC74_Result = OrgTemperature
    If xTC74_Result.7 = 1 Then          
        xTC74_Result = ~xTC74_Result + 1    
        WREG = 1                            ' It is a negative Result.
    Else                                    
        WREG = 0                            ' It is a positive result.
    EndIf
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
$else
'-------------------------------------------------------------------------------
$define TC74_ReadSRTemp(pValue, pACKReturn) _mTC74_ReadSRTemp pValue, pACKReturn

_mTC74_ReadSRTemp Macro- pValue, pACKReturn
    #if (Prm_Count > 2)
	   #error "_mTC74_ReadSRTemp - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 1)
            #error "_mTC74_ReadSRTemp - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mTC74_ReadSRTemp - (Param1) Should be a Byte or Num8"
                Exitm
            #endif
            #if(Prm_1 == Num8)
                Num_Byte pValue, xTC74_Address                    
            #endif
            #if(Prm_1 == Byte)
                Byte_Byte pValue, xTC74_Address                    
            #endif
            GoSub _mTC74_ReadSRTemp_Sub
            $ifdef _EnableGIE_Save_
            GIE_Restore()
            $endif
            #if (_mTC74_ReadSRTemp_RETURN != 1)
                #error "_mTC74_ReadSRTemp - Mandatory return parameter misssing"
                Exitm
            #else
                #if (Return_Type != Byte) && (Return_Type != Word)
                    #error "_mTC74_ReadSRTemp - Return variable should be a Byte or Word variable"
                    Exitm
                #else
                    #if (Return_Type == Byte)
                        Byte_Byte OrgTemperature, Return_Var
                    #endif
                    #if (Return_Type == Word)
                        Byte_Word OrgTemperature, Return_Var
                    #endif
                #endif
            #endif
            $ifndef _DisableNACK_
            #if (Prm_Count == 2)
                #if (Prm_2 != Byte)
                    #error "_mTC74_ReadSRTemp - (Param2) Must be a Byte!"
                    Exitm
                #else
                    Byte_Byte xACKReturn, pACKReturn
                #endif
            #endif
            $endif
        #endif
    #endif
Endm

#ifMacro- _mTC74_ReadSRTemp, _mTC74_SecureReadTemp
_mTC74_ReadSRTemp_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    $ifndef _DisableNACK_
    xACKReturn = 0              ' Clear the NACK error
    $endif
    OrgTemperature = 255        ' False result.
    GoSub SetTC74Addresses_Sub
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    ' Read the Status first.
    HBStart
    HBusOut xTC74_AddressWrite  ' TC74 Address and Write
    $ifndef _DisableNACK_
    $ifdef _SSPCON2
    If SSPCON2.6 = 1 Then       ' (ACKSTAT) Check ACK from the sensor
        HBStop
        xACKReturn = 1          ' Set A NACK error
        Return  
    EndIf
    $endif
    $ifdef _SSP1CON2     
    If SSP1CON2.6 = 1 Then      ' (ACKSTAT) Check ACK from the sensor
        HBStop
        xACKReturn = 1          ' Set A NACK error
        Return 
    EndIf
    $endif 
    $endif
    HBusOut TC74_RWCR           ' Point to the config register
    HBReStart
    HBusOut xTC74_AddressRead   ' TC74 Address and Read
    HBusIn  xACKReturn          ' Loading the result of the config register
    HBusNack                    ' Send a NACK to the sensor because it is a byte input.
    HBStop
    If xACKReturn.6 = 0 Then    ' Is the sensor ready?
        Return                  ' NO, the sensor is NOT Ready, EXIT the routine.
    EndIf
    '---------------------------------------------------------------------------
    DelayUS 10
    '---------------------------------------------------------------------------
    HBStart
    HBusOut xTC74_AddressWrite
    $ifndef _DisableNACK_
    $ifdef _SSPCON2
    If SSPCON2.6 = 1 Then       ' (ACKSTAT) Check ACK from the sensor
        HBStop
        xACKReturn = 1          ' Set A NACK error
        Return                  ' The sensor is fault. Exit. 
    EndIf
    $endif
    $ifdef _SSP1CON2   
    If SSP1CON2.6 = 1 Then      ' (ACKSTAT) Check ACK from the sensor
        HBStop
        xACKReturn = 1          ' Set A NACK error
        Return                  ' The sensor is fault. Exit.
    EndIf
    $endif 
    $endif
    HBusOut TC74_RTR            ' Point to TEMP register $00
    HBReStart
    HBusOut xTC74_AddressRead
    HBusIn  OrgTemperature      ' Loading the result.
    HBusNack
    HBStop
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
$endif
'-------------------------------------------------------------------------------
' Function Name     : Secure temperature READ. Using Read Byte Format only.
' Routine Number    : TC09  : Version Number 1.0
' Input Variables   : 
' Output Variables  : Return Var
' Syntax            : Temperature = TC74_SecureReadTemp(TC74A4)
' Notes             : If NACK error: TC74_ACK = 1
'
$ifndef _TempOrgFormat_
' Calculate the sign and the temperature value.
$define TC74_ReadSeCTemp(pValue, pSign, pACKReturn) _mTC74_SecureReadTemp pValue, pSign, pACKReturn

_mTC74_SecureReadTemp Macro- pValue, pSign, pACKReturn
    #if (Prm_Count > 3)
	   #error "_mTC74_SecureReadTemp - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 2)
            #error "_mTC74_SecureReadTemp - Too few parameters"
            Exitm
        #else
            #if (Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mTC74_SecureReadTemp - (Param1) Should be a Byte or Num8"
                Exitm
            #endif
            #if (Prm_2 != Byte)
                #error "_mTC74_SecureReadTemp - (Param2) Should be a Byte"
                Exitm
            #endif

            #if(Prm_1 == Num8)
                Num_Byte pValue, xTC74_Address                    
            #endif
            #if(Prm_1 == Byte)
                Byte_Byte pValue, xTC74_Address                    
            #endif
            GoSub _mTC74_SecureReadTemp_Sub

            #if (Prm_2 == Byte)
                Wreg_Byte pSign
            #endif  

            #if (_mTC74_SecureReadTemp_RETURN != 1)
                #error "_mTC74_SecureReadTemp - Mandatory return parameter misssing"
                Exitm
            #else
                #if (Return_Type != Byte) && (Return_Type != Word)
                    #error "_mTC74_SecureReadTemp - Return variable should be a Byte or word variable"
                    Exitm
                #else
                    #if (Return_Type == Byte)
                        Byte_Byte xTC74_Result, Return_Var
                    #endif
                    #if (Return_Type == Word)
                        Byte_Word xTC74_Result, Return_Var
                    #endif
                #endif
            #endif    
            $ifndef _DisableNACK_
            #if (Prm_Count == 3)
                #if (Prm_3 != Byte)
                    #error "_mTC74_SecureReadTemp - (Param3) Must be a Byte!"
                    Exitm
                #else
                    Byte_Byte xACKReturn, pACKReturn
                #endif
            #endif
            $endif
        #endif
    #endif
Endm

#ifMacro- _mTC74_SecureReadTemp
_mTC74_SecureReadTemp_Sub:
    ModeNormalSent = 0                              ' Initialize a flag
    '---------------------------------------------------------------------------
_mTC74_SecureReadTemp11_Sub:
    GoSub _mTC74_ReadSRTemp_Sub                     ' Read the Status & the Temperature.                     
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    ' Check the NOT Ready fault and try to fix it.
    If xACKReturn.0 = 0 Then                        ' If the sensor return an ACK (Sensor OK).
        If xACKReturn.6 = 0 Then                    ' Check if the sensor is NOT ready.
            If ModeNormalSent = 0 Then              ' If is the first time...' The sensor is not ready, try to send a Normal Mode command.
                GoSub _mTC74_WriteNMode_Sub         ' Set the Normal Mode, the delay is included in the routine.
                $ifdef _EnableGIE_Save_
                GIE_Restore()
                $endif
                ModeNormalSent = 1                  ' Don't try again.
                DelayUS 50
                GoTo _mTC74_SecureReadTemp11_Sub    ' Check Status & Read again the Temperature.
            EndIf
            xACKReturn = 1          ' The sensor is not ready and dosen't accept the Normal Mode => Fault.
            xTC74_Result = 255      ' No Temperature value
            WREG = 0                ' +Sign and return.
        EndIf
    EndIf
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
$else
' Reading the original value of the TC74 sensor.
'-------------------------------------------------------------------------------
$define TC74_ReadSeCTemp(pValue, pACKReturn) _mTC74_SecureReadTemp pValue, pACKReturn

_mTC74_SecureReadTemp Macro- pValue, pACKReturn
    #if (Prm_Count > 2)
	   #error "_mTC74_SecureReadTemp - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 1)
            #error "_mTC74_SecureReadTemp - Too few parameters"
            Exitm
        #else
            #if (Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mTC74_SecureReadTemp - (Param1) Should be a Byte or Num8"
                Exitm
            #endif

            #if(Prm_1 == Num8)
                Num_Byte pValue, xTC74_Address                    
            #endif
            #if(Prm_1 == Byte)
                Byte_Byte pValue, xTC74_Address                    
            #endif
            GoSub _mTC74_SecureReadTemp_Sub

            #if (_mTC74_SecureReadTemp_RETURN != 1)
                #error "_mTC74_SecureReadTemp - Mandatory return parameter misssing"
                Exitm
            #else
                #if (Return_Type != Byte) && (Return_Type != Word)
                    #error "_mTC74_SecureReadTemp - Return variable should be a Byte or word variable"
                    Exitm
                #else
                    #if (Return_Type == Byte)
                        Byte_Byte OrgTemperature, Return_Var
                    #endif
                    #if (Return_Type == Word)
                        Byte_Word OrgTemperature, Return_Var
                    #endif
                #endif
            #endif    
            $ifndef _DisableNACK_
            #if (Prm_Count == 2)
                #if (Prm_2 != Byte)
                    #error "_mTC74_SecureReadTemp - (Param2) Must be a Byte!"
                    Exitm
                #else
                    Byte_Byte xACKReturn, pACKReturn
                #endif
            #endif
            $endif
        #endif
    #endif
Endm

#ifMacro- _mTC74_SecureReadTemp
_mTC74_SecureReadTemp_Sub:
    ModeNormalSent = 0                              ' Initialize a flag
    '---------------------------------------------------------------------------
_mTC74_SecureReadTemp11_Sub:
    GoSub _mTC74_ReadSRTemp_Sub                     ' Read the Status & the Temperature.                     
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    ' Check the fault and try to fix it.
    If xACKReturn.0 = 0 Then                        ' If the sensor return an ACK (Sensor OK).
        If xACKReturn.6 = 0 Then                    ' Check if the sensor is NOT ready.
            If ModeNormalSent = 0 Then              ' If is the first time...' The sensor is not ready, try to send a Normal Mode command.
                GoSub _mTC74_WriteNMode_Sub         ' Set the Normal Mode, the delay is included in the routine.
                $ifdef _EnableGIE_Save_
                GIE_Restore()
                $endif
                ModeNormalSent = 1                  ' Don't try again.
                GoTo _mTC74_SecureReadTemp11_Sub    ' Check Status & Read again the Temperature.
            EndIf
            xACKReturn = 1          ' The sensor is not ready/NACK and dosen't accept the Normal Mode.
            OrgTemperature = 255    ' No Temperature value, exit the routine.
        EndIf
    EndIf
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
$endif
'-------------------------------------------------------------------------------
' Detect the TC44 sensor checking the status register.
' Input     : None
' Output    : pACKReturn holds 64 value if the TC74 is available (ready)
' Notes     : This procedure assumes that there is only 1 I2C device attached to the bus
'
'$define TC74_Present(pNumberSensor,pACKReturn) pACKReturn = TC74_ReadStatus(pNumberSensor)


$define TC74_Present(pNumberSensor, pACKReturn) '
    xTC74_Address = pNumberSensor               '
    _mTC74_Present                              '
    pACKReturn = xACKReturn

_mTC74_Present Macro-
    GoSub _mTC74_Present_Sub
Endm

#ifMacro- _mTC74_Present
_mTC74_Present_Sub:
    ModeNormalSent = 0
_mTC74_Present22_Sub:
    GoSub _mTC74_ReadStatus_Sub                    
    If xACKReturn.0 = 0 Then
        If xACKReturn.6 = 0 Then
            If ModeNormalSent = 0 Then
                GoSub _mTC74_WriteNMode_Sub
                ModeNormalSent = 1
                GoTo _mTC74_Present22_Sub
            EndIf
            xACKReturn = 1
        Else
            xACKReturn = 0   
        EndIf
    EndIf
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Calculate the Address write and read of the TC74 sensor.
' Input : 
' Output:
' Notes : 
'
SetTC74Addresses_Sub:
    xTC74_AddressWrite = xTC74_Address << 1    
    xTC74_AddressRead = xTC74_AddressWrite | $01
    Return
'*******************************************************************************
'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
$ifdef _HRSOutVersionFile_inc_
_TC74H_INC_Version: CData "TC74-H.inc Version 1.2 - 13/03/2016",0
$endif
'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
OverDriverTC74:     
'===============================================================================
' Set the I2C pins to digital. (Result = 2 lines)
$if (_device = _18F23K22) Or (_device = _18F24K22) Or (_device = _18F25K22) Or (_device = _18F26K22)
$ifndef _AnselI2CSoftPins_
$define _AnselI2CSoftPins_
#if (__SCL_PORT == PORTA)
    #if (__SCL_PIN == 0)
        $ifdef _ANSEL
        ANSEL.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SCL_PIN == 1)
        $ifdef _ANSEL
        ANSEL.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SCL_PIN == 2)
        $ifdef _ANSEL
        ANSEL.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SCL_PIN == 3)
        $ifdef _ANSEL
        ANSEL.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SCL_PIN == 5)
        $ifdef _ANSEL
        ANSEL.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SCL_PORT == PORTB)
    #if (__SCL_PIN == 0)
        $ifdef _ANSELH
        ANSELH.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SCL_PIN == 1)
        $ifdef _ANSELH
        ANSELH.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SCL_PIN == 2)
        $ifdef _ANSELH
        ANSELH.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SCL_PIN == 3)
        $ifdef _ANSELH
        ANSELH.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SCL_PIN == 4)
        $ifdef _ANSELH
        ANSELH.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SCL_PORT == PORTC)
    $ifdef _ANSELC
    #if (__SCL_PIN == 2)
        ANSELC.2 = 0
    #endif
    #if (__SCL_PIN == 3)
        ANSELC.3 = 0
    #endif
    #if (__SCL_PIN == 4)
        ANSELC.4 = 0
    #endif
    #if (__SCL_PIN == 5)
        ANSELC.5 = 0
    #endif
    #if (__SCL_PIN == 6)
        ANSELC.6 = 0
    #endif
    #if (__SCL_PIN == 7)
        ANSELC.7 = 0
    #endif
    $endif
#endif

$ifdef _PORTD
#if (__SCL_PORT == PORTD)
    $ifdef _ANSELD
    #if (__SCL_PIN == 0)
        ANSELD.0 = 0
    #endif
    #if (__SCL_PIN == 1)
        ANSELD.1 = 0
    #endif
    #if (__SCL_PIN == 2)
        ANSELD.2 = 0
    #endif
    #if (__SCL_PIN == 3)
        ANSELD.3 = 0
    #endif
    #if (__SCL_PIN == 4)
        ANSELD.4 = 0
    #endif
    #if (__SCL_PIN == 5)
        ANSELD.5 = 0
    #endif
    #if (__SCL_PIN == 6)
        ANSELD.6 = 0
    #endif
    #if (__SCL_PIN == 7)
        ANSELD.7 = 0
    #endif
    $endif    
#endif
$endif

$ifdef _PORTE
#if (__SCL_PORT == PORTE)
    #if (__SCL_PIN == 0)
        $ifdef _ANSEL       
        ANSEL.5 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.0 = 0
        $endif
    #endif
    #if (__SCL_PIN == 1)
        $ifdef _ANSEL
        ANSEL.6 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.1 = 0
        $endif
    #endif
    #if (__SCL_PIN == 2)
        $ifdef _ANSEL
        ANSEL.7 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.2 = 0
        $endif
    #endif
#endif
$endif
'-------------------------------------------------------------------------------
#if (__SDA_PORT == PORTA)
    #if (__SDA_PIN == 0)
        $ifdef _ANSEL
        ANSEL.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SDA_PIN == 1)
        $ifdef _ANSEL
        ANSEL.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SDA_PIN == 2)
        $ifdef _ANSEL
        ANSEL.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SDA_PIN == 3)
        $ifdef _ANSEL
        ANSEL.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SDA_PIN == 5)
        $ifdef _ANSEL
        ANSEL.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SDA_PORT == PORTB)
    #if (__SDA_PIN == 0)
        $ifdef _ANSELH
        ANSELH.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SDA_PIN == 1)
        $ifdef _ANSELH
        ANSELH.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SDA_PIN == 2)
        $ifdef _ANSELH
        ANSELH.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SDA_PIN == 3)
        $ifdef _ANSELH
        ANSELH.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SDA_PIN == 4)
        $ifdef _ANSELH
        ANSELH.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SDA_PORT == PORTC)
    $ifdef _ANSELC
    #if (__SDA_PIN == 2)
        ANSELC.2 = 0
    #endif
    #if (__SDA_PIN == 3)
        ANSELC.3 = 0
    #endif
    #if (__SDA_PIN == 4)
        ANSELC.4 = 0
    #endif
    #if (__SDA_PIN == 5)
        ANSELC.5 = 0
    #endif
    #if (__SDA_PIN == 6)
        ANSELC.6 = 0
    #endif
    #if (__SDA_PIN == 7)
        ANSELC.7 = 0
    #endif
    $endif
#endif

$ifdef _PORTD
#if (__SDA_PORT == PORTD)
    $ifdef _ANSELD
    #if (__SDA_PIN == 0)
        ANSELD.0 = 0
    #endif
    #if (__SDA_PIN == 1)
        ANSELD.1 = 0
    #endif
    #if (__SDA_PIN == 2)
        ANSELD.2 = 0
    #endif
    #if (__SDA_PIN == 3)
        ANSELD.3 = 0
    #endif
    #if (__SDA_PIN == 4)
        ANSELD.4 = 0
    #endif
    #if (__SDA_PIN == 5)
        ANSELD.5 = 0
    #endif
    #if (__SDA_PIN == 6)
        ANSELD.6 = 0
    #endif
    #if (__SDA_PIN == 7)
        ANSELD.7 = 0
    #endif
    $endif    
#endif
$endif

$ifdef _PORTE
#if (__SDA_PORT == PORTE)
    #if (__SDA_PIN == 0)
        $ifdef _ANSEL       
        ANSEL.5 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.0 = 0
        $endif
    #endif
    #if (__SDA_PIN == 1)
        $ifdef _ANSEL
        ANSEL.6 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.1 = 0
        $endif
    #endif
    #if (__SDA_PIN == 2)
        $ifdef _ANSEL
        ANSEL.7 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.2 = 0
        $endif
    #endif
#endif
$endif
$endif      ' _AnselI2CSoftPins_
$endif      ' ifdevice
'===============================================================================
$endif      ' $define _TC74H_ 
'-------------------------------------------------------------------------------

