'****************************************************************
'*  Name    : STM07.bas                                         *
'*  Project : Testing the State Machine Template                *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) February 2017 [A. Freixanet 2017]   *
'*          : All Rights Reserved                               *
'*  Date    : 05/06/2017                                        *
'*  Version : 1.0                                               *
'*  Notes   : Using the AMICUS18 Development Board              *
'*            CoolTerm Terminal is Used for bootloader & testing*
'* Powered by PROTON DEVELOPMENT SUITE COMPILER V3.5.9.3        *
'*          Testing the DS1307 I2C clock calendar               *
'*          Testing the TC74 I2C temperature sensor             *
'*          Testing the AMI18 LCD BOARD                         *
'*          Configuration of Date/Time/Contrast by Buttons      *
'* Bootloader: Using AGV Bootloader Version 4.1 Low Side of ROM *
'****************************************************************
' Compile the HEX for the Amicus18 Board for FOSC = 80 Mhz.
'    $define _AMicus18_80Mhz_
'    $define _HSerialBaud_ 230400
'###############################################################################
' Compile the HEX for the Amicus18 Board for FOSC = 64 Mhz.
'    $define _AMicus18_64Mhz_
'###############################################################################
' Compile the HEX for the Proteus test for FOSC = 16 Mhz. (nothing to do)
' Testing the CODE by Proteus
$if !_defined(_AMicus18_80Mhz_) And !_defined(_AMicus18_64Mhz_)
    $define _ProteusDebug_
$endif
'###############################################################################
' Choose the Bootloader you are using for the Amicus18 Board (real hardware).
' CHOOSE ONLY ONE.
'    $define _AmicusBootloader_
    $define _AGV_LSM_BootloaderV41_
'###############################################################################
' All FILES LIST for this PROJECT:
' STM07.bas (main code)
'
' SM_PIC18F25K20
' SYS_18FXXK20.inc
' STMachine02.Inc
' Amicus_ADCbeta.inc
' SM_IntHandler00.Inc
' DS1307-H.Inc
' DS1307-S.Inc
' LCD_ST7036.Inc
' HRsout_K40.Inc
' ORG_SyncBlock_K20.Inc
' TC74-H_beta.inc
' TC74-S_beta.inc
' SM1_Scheduler01.Inc
' SM2_Scheduler01.Inc
' STM07_SM2Code.bas
' STM_Strings07.Inc
'
'###############################################################################
' This source code is free software and provided 'as is', any express or implied 
' warranties are disclaimed. you can redistribute it and/or modify.
' The author is not responsible of the use of this software.
' This program is distributed in the hope that it will be useful, but WITHOUT 
' ANY WARRANTY. However, a mention of the author and compiler would be appreciated.
'*******************************************************************************
' Este cdigo fuente es un software libre, puede redistribuirlo y/o modificarlo.
' El autor no se hace responsable del uso de este software.
' Este programa se distribuye con la esperanza de que sea til,
' pero SIN NINGUNA GARANTA. No obstante citar el autor y el compilador
' sera muy apreciado.
'*******************************************************************************
' Ce code source est un logiciel libre, vous pouvez le redistribuer et/ou modifier.
' L'auteur n'est pas responsable de l'utilisation de ce logiciel.
' Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE.
' Cependant mentionner l'auteur et le compileur serait trs aprci.
'
'###############################################################################
'###############################################################################
'*******************************************************************************
'* Name    : Declare the PIC used.                                             *
'* Purpose : Declare all datas of the used PIC                                 *
'* Notes   : Change the file for a new pIC.                                    *
'******************************************************************************* 
'    Include "D:\MyIncFiles\SM_PIC18F25K20.Inc"
    Include "INC\SM_PIC18F25K20.inc"
'###############################################################################
' STATE MACHINE PROJECT STARTS HERE:
'###############################################################################
'*******************************************************************************
'* Name    : LCD                                                               *
'* Purpose : Declare the LCD pins                                              *
'* Notes   : Configuration of the LCD for the AMI18 Shield.                    *
'******************************************************************************* 
    Declare LCD_DTPin = PORTB.4         ' LCD's Data lines (D4 to D7)
    Declare LCD_ENPin = PORTB.3         ' LCD's EN line
    Declare LCD_RSPin = PORTB.2         ' LCD's RS line
    Declare LCD_Interface = 4           ' 4-bit interface to LCD
    Declare LCD_Lines = 2               ' LCD contains 2 lines
    Declare LCD_DataUs = 50             ' Time to wait after print a data To LCD
    Declare LCD_CommandUs = 4000        ' Time to wait after print a command to LCD
    Declare LCD_Type = Alphanumeric     ' LCD type is alphanumeric
    Symbol __LCD_CharsLenght = 16       ' For the AMI18 LCD Shield: 16 Characters
    Symbol bLCD_BackLight = LATB.1      ' BackLight for the LCD.
'###############################################################################
'*******************************************************************************
'* Name    : State machine numbers                                             *
'* Purpose : Define every State with a name (optional)                         *
'* Notes   : STATE00 to STATE27                                                *
'******************************************************************************* 
    $define SM_Init 0
    $define SM_ProjectInfo 1
    $define SM_DS1307CheckACK 2
    $define SM_Enable1Hz 3
    $define SM_WriteConfig 4
    $define SM_ReadTimeDate 5
    $define SM_DS1307Error_NACK 6
    $define SM_AskForConfig 7
    $define SM_ButtonsOFF 8
    $define SM_ReadButtons 9   
    $define SM_ButtonsOFF2 10
    $define SM_GetWeek 11
    $define SM_GetDay 12
    $define SM_GetMonth 13
    $define SM_GetYear 14
    $define SM_GetHour 15
    $define SM_GetMinute 16
    $define SM_ContrastSetup 17
    $define SM_SetContrast 18   
    $define SM_SaveContrast 19       
    $define SM_AlarmMenu 20
    $define SM_SaveAlarm 21
    $define SM_Alarm 22
    $define SM_SeeAlarm2 23
    $define SM_SeeAlarm1 24
    $define SM_DisableAlarms 25
    $define SM_TC74CheckACK 26
    $define SM_TC74Error_NACK 27
'-------------------------------------------------------------------------------
    ' Update the LABELS when you add a new State Machine 1 Module.
$define SM1_LABELS_LIST '
    BranchL StateIndex,[STATE00,STATE01,STATE02,STATE03,STATE04,STATE05,_     '
    STATE06,STATE07,STATE08,STATE09,STATE10,STATE11,STATE12,STATE13,STATE14,_ '
    STATE15,STATE16,STATE17,STATE18,STATE19,STATE20,STATE21,STATE22,STATE23,_ '
    STATE24,STATE25,STATE26,STATE27]

'*******************************************************************************
'* Name    : State machine2 State numbers                                      *
'* Purpose : Define every State with a name (optional)                         *
'* Notes   : SM2_STATE00 to SM2_STATE11                                        *
'******************************************************************************* 
    $define SM2_ReadTime 0
    $define SM2_PrintLCDLine1 1
    $define SM2_PrintLCDLine2 2
    $define SM2_CheckAlarmState 3
    $define SM2_ADCaverage 4
    $define SM2_TC74SetNormalMode 5
    $define SM2_TC74Read 6
    $define SM2_TC74Close 7
    $define SM2_ReadDate 8
    $define SM2_Data1ToTerminal 9
    $define SM2_Data2ToTerminal 10
    $define SM2_TC74TempToTerminal 11

    $define SM2_StateMax 12             
'-------------------------------------------------------------------------------
    ' Update the LABELS when you add a new State Machine 2 Module.
$define SM2_LABELS_LIST '
    BranchL SM2StateIndex,[SM2_STATE00,SM2_STATE01,SM2_STATE02,SM2_STATE03,_  '
    SM2_STATE04,SM2_STATE05,SM2_STATE06,SM2_STATE07,SM2_STATE08,SM2_STATE09,_ '
    SM2_STATE10,SM2_STATE11] 

'*******************************************************************************
'* Name    : Tasks System                                                      *
'* Purpose : Define the Task priorities                                        *
'* Notes   :                                                                   *
'******************************************************************************* 
' Define the Tasks Buffer lenght           
    $define SM2TasksBufferLenght SM2_StateMax + 2        

' Define the Number of tasks & the corresponding starting state.
' The position of the Task defines the priority.
' All SM2 states must to have a Task numer. (not the SM2_STATE00) 
    '---------------------------------------------------------------------------
    ' User Tasks.
    $define Task0 0     ' SM2_ADCaverage (04)      : (Higher priority)   
    $define Task1 1     ' SM2_ReadTime (0)
    $define Task2 2     ' SM2_PrintLCDLine2 (2)
    $define Task3 3     ' SM2_ReadDate (8)
    $define Task4 4     ' SM2_PrintLCDLine1 (1) 
    $define Task5 5     ' SM2_CheckAlarmState (3)  
    $define Task6 6     ' SM2_Data1ToTerminal (9)
    $define Task7 7     ' SM2_Data2ToTerminal (10)
    $define Task8 8     ' SM2_TC74SetNormalMode (5) 
    $define Task9 9     ' SM2_TC74Read (6)
    $define Task10 10   ' SM2_TC74Close (7)
    $define Task11 11   ' SM2_TC74TempToTerminal (11)
    '---------------------------------------------------------------------------
    ' Reserved State Machine Tasks.
    $define Task12 12   ' SM2 Virtual Delay
    $define Task13 13   ' SM2 Delayed Task           : (Lower priority) 
'-------------------------------------------------------------------------------
' Define the Task priority for several functions of SM2.
    $define SM2DefADCTask Task0        ' SM2_ADCaverage (average) Very High priority.
    $define SM2DefVDelayTask Task12    ' SM2 Virtual Delay
    $define SM2DefDelayedTask Task13   ' SM2 Delayed Task
'===============================================================================
' Copy the number of tasks in this codes.
' Write the Task in this code from State1 to State15.
$define LoadStateTable(pNewState)   '
$if pNewState = 0      '
    SM2Task[Task1] = 0 '
$elseif pNewState = 1  '
    SM2Task[Task4] = 1 '
$elseif pNewState = 2  '
    SM2Task[Task2] = 2 '
$elseif pNewState = 3  '
    SM2Task[Task5] = 3 '
$elseif pNewState = 4  '
    SM2Task[Task0] = 4 '
$elseif pNewState = 5  '
    SM2Task[Task8] = 5 '
$elseif pNewState = 6  '
    SM2Task[Task9] = 6 '
$elseif pNewState = 7  '
    SM2Task[Task10] = 7 '
$elseif pNewState = 8   '
    SM2Task[Task3] = 8  '
$elseif pNewState = 9   '
    SM2Task[Task6] = 9  '
$elseif pNewState = 10  '
    SM2Task[Task7] = 10 '
$elseif pNewState = 11  '
    SM2Task[Task11] = 11 '
$else                    '
$error "NO SUCH STATE IN SM2" '
$endif

'-------------------------------------------------------------------------------
' Write the Task in this code from State1 to State15.
$define DeleteStateTable(pNewState) '
$if pNewState = 0        '
    SM2Task[Task1] = 255 '
$elseif pNewState = 1    '
    SM2Task[Task4] = 255 '
$elseif pNewState = 2    '
    SM2Task[Task2] = 255 '
$elseif pNewState = 3    '
    SM2Task[Task5] = 255 '
$elseif pNewState = 4    '
    SM2Task[Task0] = 255 '
$elseif pNewState = 5    '
    SM2Task[Task8] = 255 '
$elseif pNewState = 6    '
    SM2Task[Task9] = 255 '
$elseif pNewState = 7     '
    SM2Task[Task10] = 255 '
$elseif pNewState = 8     '
    SM2Task[Task3] = 255  '
$elseif pNewState = 9     '
    SM2Task[Task6] = 255  '
$elseif pNewState = 10    '
    SM2Task[Task7] = 255  '
$elseif pNewState = 11    '
    SM2Task[Task11] = 255 '
$else                     '
$error "NO SUCH STATE IN SM2" '
$endif

'-------------------------------------------------------------------------------
'*******************************************************************************
'* Name    : Enable the Library Commands inside the STMachineXX.Inc file       *
'* Purpose : To use the necessary resources only for RAM and code.             *
'* Notes   :                                                                   *
'*******************************************************************************
' Uncomment the followed line to insert the Keypad option.
'    $define _Disable_Keypad_Pullups_
'    $define _4x4KEYPAD_
'    $define _4x3KEYPAD_
'    $define _4x2KEYPAD_
'-------------------------------------------------------------------------------
' Others State Machine libraries.
    $define _HRSE2PStrg_
'    $define _PrintE2PStrg_
'    $define _Manchester_
'    $define _MidPoint_
'    $define _MedianFilter_
'    $define _BubbleSort_
'    $define _GetParity_
'    $define _RotateBits_
'    $define _BinaryTo3BCD_
'    $define _BCD2ToBinary_
'    $define _WordTo5BCD_
'    $define _DwordArray_
'    $define _CountUntil_
'    $define _ArrayROL_
'    $define _ArrayROR_
'    $define _ArrayROLR_
'    $define _ArrayRORR_
'    $define _ArrayBit_
'    $define _CRC16_
'    $define _Multi_DIM_Array_
'*******************************************************************************
'* Name    : HSerial                                                           *
'* Purpose : Declare the HSerial Communication for the STM Library             *
'* Notes   : Uncomment a line to declare a Terminal Baud Rate. (One only)      *
'******************************************************************************* 
' Uncomment a line to define a Terminal Baud Rate, and compile. (One only)
$ifndef _HSerialBaud_
    $ifdef _ProteusDebug_
'    $define _HSerialBaud_ 9600
'    $define _HSerialBaud_ 19200   
'    $define _HSerialBaud_ 38400
    $define _HSerialBaud_ 57600
    $else
'    $define _HSerialBaud_ 9600
'    $define _HSerialBaud_ 19200   
'    $define _HSerialBaud_ 38400
'    $define _HSerialBaud_ 57600
    $define _HSerialBaud_ 115200
'    $define _HSerialBaud_ 230400
    $endif
$endif
'===============================================================================
'    $if _HSerialBaud_ = 9600
'    'Calculated Baudrate = 9598 @ Xtal 64MHz, Error = -0,02%
'    Declare Hserial_RCSTA  = %10010000  ' Enable continuous receive 
'    Declare Hserial_TXSTA  = %00100100  ' Enable transmit, BRGH = 1
'    Declare Hserial_Clear  = On         ' Clear overflow automatically
'    Declare Hserial_SPBRG  = $82        ' Baud Rate Generator Low Byte Value
'    SPBRGH    = $06                     ' Baud Rate Generator High Byte Value
'    BAUDCON.3 = 1                       ' Enable the 16 bit Baud Rate Generator  
'    $endif

    $if _HSerialBaud_ = 9600
        Declare Hserial_Baud = 9600
        Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
        Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
        Declare Hserial_Clear = On      ' Enable Error clearing on received characters
    $endif
    $if _HSerialBaud_ = 19200
        Declare Hserial_Baud = 19200        ' Set baud rate to 19200
        Declare Hserial_RCSTA = %10010000   ' Enable serial port and continuous receive
        Declare Hserial_TXSTA = %00100100   ' Enable transmit and asynchronous mode
        Declare Hserial_Clear = On          ' Enable Error clearing on received characters
    $endif
    $if _HSerialBaud_ = 38400
        Declare Hserial_Baud = 38400        ' Set baud rate to 38400
        Declare Hserial_RCSTA = %10010000   ' Enable serial port and continuous receive
        Declare Hserial_TXSTA = %00100100   ' Enable transmit and asynchronous mode
        Declare Hserial_Clear = On          ' Enable Error clearing on received characters
    $endif
    $if _HSerialBaud_ = 57600
        Declare Hserial_Baud = 57600    ' Set baud rate to 57600
        Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
        Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
        Declare Hserial_Clear = On      ' Enable Error clearing on received characters
    $endif
    $if _HSerialBaud_ = 115200
        Declare Hserial_Baud = 115200
        Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
        Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
        Declare Hserial_Clear = On      ' Enable Error clearing on received characters
    $endif
    $if _HSerialBaud_ = 230400
        Declare Hserial_Baud = 230400
        Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
        Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
        Declare Hserial_Clear = On      ' Enable Error clearing on received characters
    $endif
'-------------------------------------------------------------------------------
(*
'    $if _HSerialBaud_ = 9600
'    'Calculated Baudrate = 9598 @ Xtal 64MHz, Error = -0,02%
'    Declare Hserial2_RCSTA  = %10010000  ' Enable continuous receive 
'    Declare Hserial2_TXSTA  = %00100100  ' Enable transmit, BRGH = 1
'    Declare Hserial2_Clear  = On         ' Clear overflow automatically
'    Declare Hserial2_SPBRG  = $82        ' Baud Rate Generator Low Byte Value
'    SPBRGH    = $06                     ' Baud Rate Generator High Byte Value
'    BAUDCON.3 = 1                       ' Enable the 16 bit Baud Rate Generator  
'    $endif

    $if _HSerialBaud_ = 9600
        Declare Hserial2_Baud = 9600
        Declare Hserial2_TXSTA = 36      ' Enable transmit and asynchronous mode
        Declare Hserial2_RCSTA = 144     ' Enable serial port and continuous receive
        Declare Hserial2_Clear = On      ' Enable Error clearing on received characters
    $endif
    $if _HSerialBaud_ = 19200
        Declare Hserial2_Baud = 19200        ' Set baud rate to 19200
        Declare Hserial2_RCSTA = %10010000   ' Enable serial port and continuous receive
        Declare Hserial2_TXSTA = %00100100   ' Enable transmit and asynchronous mode
        Declare Hserial2_Clear = On          ' Enable Error clearing on received characters
    $endif
    $if _HSerialBaud_ = 38400
        Declare Hserial2_Baud = 38400        ' Set baud rate to 38400
        Declare Hserial2_RCSTA = %10010000   ' Enable serial port and continuous receive
        Declare Hserial2_TXSTA = %00100100   ' Enable transmit and asynchronous mode
        Declare Hserial2_Clear = On          ' Enable Error clearing on received characters
    $endif
    $if _HSerialBaud_ = 57600
        Declare Hserial2_Baud = 57600    ' Set baud rate to 57600
        Declare Hserial2_RCSTA = 144     ' Enable serial port and continuous receive
        Declare Hserial2_TXSTA = 36      ' Enable transmit and asynchronous mode
        Declare Hserial2_Clear = On      ' Enable Error clearing on received characters
    $endif
    $if _HSerialBaud_ = 115200
        Declare Hserial2_Baud = 115200
        Declare Hserial2_RCSTA = 144     ' Enable serial port and continuous receive
        Declare Hserial2_TXSTA = 36      ' Enable transmit and asynchronous mode
        Declare Hserial2_Clear = On      ' Enable Error clearing on received characters
    $endif
*)
'-------------------------------------------------------------------------------
'===============================================================================
' Choose the HSRSout command for the STM Library. (One only)
    $define _HRSOut_ HRSOut
'    $define _HRSOut_ HRSOut1
'    $define _HRSOut_ HRSOut2
'    $define _HRSOut_ HRSOut3
'    $define _HRSOut_ HRSOut4
'    $define _HRSOut_ HRSOut5
'*******************************************************************************
'* Name    : State Machine Commands                                            *
'* Notes   : Uncomment the line to enable a STM Function                       *
'*******************************************************************************
' Uncomment the followed line to disable the TIMER0 option
' for the STM Library only.
'    $define _EnableSMTimer0_

' Uncomment the followed line to disable the terminal function to send strings
' for the STM Library only.
' Enabled = Default configuration.
    $define _EnableTerminal_

' Uncomment the followed line to enable the SM_Return function.
' Disabled = Default configuration.
'    $define _CheckReturnStateM_

' Uncomment the followed line to disable the Carriage Return before every state
' for the STM Library only. 
' Enabled = Default configuration.
    $define _DisableCRBeforeState_

' Uncomment the followed line to enable some Timers
' All Enabled. Default configuration
    $define TaskADC_Timer ADCTimer
    $define TaskBackLight_Timer BackLightTimer
    $define TaskToggleFlag1_Timer ToggleFlag1Timer
    $define Task1S_Timer Timer1S
    
    ' Task ADC interval => 4 to 500 mS.
    $define TaskADC_Interval 4

    ' TaskBackLight_Interval = 10mS
    $define TaskBackLight_Interval 10

    ' TaskToggleFlag1_Interval = 500 mS
    $define TaskToggleFlag1_Interval 500

    ' Task1S_Interval = 1000 mS
    $define Task1S_Interval 1000
    
' Uncomment the followed line to Synchronize the Main code with Timer 1mS.
' All Disabled = Default configuration.
    $define _SynchronizeMSFunction_

' Uncomment the followed line to enable some Delays
' Enabled = Default configuration.
    $define _EnableSM1VDelay_

' Uncomment the followed line to enable some function
' Disabled = Default configuration.
    $define _EnablePulsePin_

' User Delay1. Disabled = Default configuration.
'    $define _EnableUserDelay1_    

' User Delay2. Disabled = Default configuration.
'    $define _EnableUserDelay2_     

' User Delay3. Disabled = Default configuration.
'    $define _EnableUserDelay3_

' SM TimeOut system for Buttons and Keypads menus of other systems.
    $define _EnableTimeOutSystem_

' SM TimeOut system for Buttons and Keypads menus.
    $define _EnableBackLightSystem_

'===============================================================================
' Uncomment the followed line to enable another States Machine2 in Background
' All Disabled = Default configuration.
    $define _StatesMachine2_
    $define _EnableSM2VDelay_
    $define _EnableSM2DelayedState_

'-------------------------------------------------------------------------------
' Uncomment the followed line to read twice every sample with very precise 100mS timing.  65535
' ADC SYSTEM: Choose only one Timing.
    $define _EnableADCSystem_
  
'-------------------------------------------------------------------------------
' The average is done in every ADC sample.
' Choose an Average number.
'    $define NumberADCaverages 2
'    $define NumberADCaverages 4
'    $define NumberADCaverages 8
'    $define NumberADCaverages 16
'    $define NumberADCaverages 32
    $define NumberADCaverages 64
'===============================================================================
' (For DEBUG) Uncomment the followed line to insert a state number in every module.
' Disabled = Default configuration. 
' If _DebugStates_ is set then disable _DisableCRBeforeState_.
' If _DebugStates_ is set then use _ReadADC_Every100mS_.
'    $define _DebugStates_ 
    $define _CheckADCSystem_

'*******************************************************************************
'* Name    : Language                                                          *
'* Purpose : Define one language for this project                              *
'* Notes   : One language must be enabled                                      *
'*******************************************************************************            
    $define _English_
'    $define _Spanish_
'    $define _French_ 
'###############################################################################
    $ifdef _ProteusDebug_
    $define _PinsTest_
    $endif
'-------------------------------------------------------------------------------
    $if (_device = _18F23K20) Or (_device = _18F24K20) Or (_device = _18F25K20) Or (_device = _18F26K20) Or (_device = _18F43K20) Or (_device = _18F44K20) Or (_device = _18F45K20) Or (_device = _18F46K20)
    $define _VDD33V_
    $endif

' Write here all the conditions to enable the Tasks Management Code.
$ifdef _EnableADCSystem_
    $ifdef NumberADCaverages
    $if (NumberADCaverages <> 2) And (NumberADCaverages <> 4) And (NumberADCaverages <> 8) And (NumberADCaverages <> 16) And (NumberADCaverages <> 32) And (NumberADCaverages <> 64)
        $error "NO SUCH NUMBER FOR AVERAGES"
    $endif      
    $endif
$endif
'-------------------------------------------------------------------------------
    $ifndef _EnableTerminal_
    $error "_EnableTerminal_ MUST BE DECLARED BY DEFAULT"
    $endif 
'-------------------------------------------------------------------------------
    $ifndef _EnableTimeOutSystem_
    $error "_EnableTimeOutSystem_ MUST BE DECLARED BY DEFAULT"
    $endif 
'-------------------------------------------------------------------------------
    $ifndef _EnableADCSystem_
    $error "_EnableADCSystem_ MUST BE DECLARED BY DEFAULT"
    $endif     
'-------------------------------------------------------------------------------
    $ifndef _EnableBackLightSystem_
    $error "_EnableBackLightSystem_ MUST BE DECLARED BY DEFAULT"
    $endif
'-------------------------------------------------------------------------------    
    $ifndef _EnablePulsePin_
    $error "_EnablePulsePin_ MUST BE DECLARED BY DEFAULT"
    $endif
'-------------------------------------------------------------------------------   
    $ifndef _StatesMachine2_
    $error "_StatesMachine2_ MUST BE DECLARED BY DEFAULT"
    $endif
            
'###############################################################################
'*******************************************************************************
'* Name    : Variables & Constants                                             *
'* Purpose : Declare the Variables or Constants                                *
'* Notes   : The local variables are declared for this State Machine file only *
'* Rules   : If the Variable is used once then get Local Variable.             *
'*         : If the Variable is used several times in a large loop in STM1     *
'*         : with STM2 or in others modules then get a Global Variable.        *
'******************************************************************************* 
' Definition of local Bytes for Main code only
$ifndef _SMLVBS_Block1_
    $define _SMLVBS_Block1_
    Dim SMLVSByte1 As Byte
    Dim SMLVSByte2 As Byte
    Dim SMLVSByte3 As Byte
    Dim SMLVSByte4 As Byte
$endif
' If you need more local Byte Variables, uncomment this block.
'$ifndef _SMLVBS_Block2_
'    $define _SMLVBS_Block2_
'    Dim SMLVSByte5 As Byte     ' Local Variable Byte1
'    Dim SMLVSByte6 As Byte     ' Local Variable Byte2
'    Dim SMLVSByte7 As Byte     ' Local Variable Byte3
'    Dim SMLVSByte8 As Byte     ' Local Variable Byte4
'$endif

' Definition of local Words For Main Code only
'$ifndef _SMLVWS_Block1_
'    $define _SMLVWS_Block1_
'    Dim SMLVSWord1 As Word     ' Local Variable Word1
'    Dim SMLVSWord2 As Word     ' Local Variable Word2
'$endif
' If you need more local Word Variables, uncomment this block.
'$ifndef _SMLVWS_Block2_
'    $define _SMLVWS_Block2_
'    Dim SMLVSWord3 As Word     ' Local Variable Word1
'    Dim SMLVSWord4 As Word     ' Local Variable Word2
'$endif

' Definition of local DWords For Main Code only
'$ifndef _SMLVDWS_Block1_
'    $define _SMLVDWS_Block1_
'    Dim SMLVSDWord1 As Dword     ' Local Variable DWord1            
'$endif
' If you need more local Dword Variables, uncomment this block.
'$ifndef _SMLVDWS_Block2_
'    $define _SMLVDWS_Block2_
'    Dim SMLVSDWord2 As DWord    ' Local Variable DWord2
'$endif

' Definition of local bits
'$ifndef _LVSBitS_Block1_
'    $define _LVSBitS_Block1_
'    Dim LVSBitsA As Byte     ' Local Variable byte
'$endif
'===============================================================================
    ' Variables for the Buttons
    Dim AllButtons As Byte
    Dim AllButtonsBitsDefects As Byte
    Dim ButtonFail As Byte
    Dim ButtonsOffCounter As Byte
    Dim KEYnOld As Byte
    Dim KEYCounter As Byte
'-------------------------------------------------------------------------------
    Dim LCDContrast As Byte
    Dim RTC_MinuteOld As Byte
'-------------------------------------------------------------------------------
    ' Variables for the Temperature system
    Dim ReturnPolarity As Byte
    Dim Temperature As Byte
'-------------------------------------------------------------------------------
    ' Variables for the ADC system
    Dim ADC_IntegerResult As Byte
    Dim ADC_DecimalResult As Word
    Dim ADCResult As Dword
'-------------------------------------------------------------------------------
    ' Flags
    Dim SM_UserSystemFlags1 As Byte
    Dim F_DS1307_1S    As SM_UserSystemFlags1.0
    Dim F_Buttons10mS  As SM_UserSystemFlags1.1
    Dim F_PWM10mS      As SM_UserSystemFlags1.2
    Dim F_StartADCconv As SM_UserSystemFlags1.3
    Dim F_PWMBackLight As SM_UserSystemFlags1.4
    
    Dim SM_UserSystemFlags2 As Byte
    Dim F_AlarmX        As SM_UserSystemFlags2.0 
    Dim F_Alarm1        As SM_UserSystemFlags2.1
    Dim F_Alarm2        As SM_UserSystemFlags2.2
    Dim F_EnableAlarm1  As SM_UserSystemFlags2.3
    Dim F_Alarm1Avaible As SM_UserSystemFlags2.4
    Dim F_SettingAlarm1 As SM_UserSystemFlags2.5
    Dim F_EnableAlarm2  As SM_UserSystemFlags2.6
    Dim F_Alarm2Avaible As SM_UserSystemFlags2.7

    Dim SM_UserSystemFlags3 As Byte
    Dim F_SettingAlarm2 As SM_UserSystemFlags3.0
    Dim F_SettingAlarmX As SM_UserSystemFlags3.1
    Dim F_MenuContrastFromReset As SM_UserSystemFlags3.2
    Dim F_DisableADCInt As SM_UserSystemFlags3.3
    Dim F_ADC_Error     As SM_UserSystemFlags3.4
    Dim F_ErrorNACK_DS1307 As SM_UserSystemFlags3.5
    Dim F_ErrorNACK_TC74 As SM_UserSystemFlags3.6
     
'    Dim SM_Errors As Byte
'    Dim F_Reset As SM_Errors.0   
'    Dim F_Watchdog As SM_Errors.1
'    Dim F_BrownOut As SM_Errors.2    
'    Dim F_StackFull As SM_Errors.3
'    Dim F_StackUnderflow As SM_Errors.4
                      
'===============================================================================
' DECLARE THE ARRAYS BELOW THIS LINE
'===============================================================================
' Definition of PORT pins for Amicus18. (bits)
    Symbol bButton1 = PORTA.0
    Symbol bButton2 = PORTA.1
    Symbol bButton3 = PORTA.2
    Symbol bButton4 = PORTA.3            
    Symbol bBUZZER  = LATB.0
    Symbol bLEDConfig = LATC.2
'-------------------------------------------------------------------------------
    Symbol MaxYear = 30 
    Symbol ThisYear = 17
'-------------------------------------------------------------------------------
' Definition of the Analog pins
    Symbol Analog1IN = PORTA.5  ' AN4
'-------------------------------------------------------------------------------
' Definition of the Test pins. (bits)
    $ifdef _PinsTest_
    Symbol bTest1 = LATC.0
    Symbol bTest2 = LATC.1 
    Symbol bTest3 = LATC.2 
    $endif
    Symbol bAlarm1Out = LATC.5 
'===============================================================================
' Declare the Arrays bellow this line.
    Dim SMDS1307_Array[8] As Byte
    Dim VSecond    As SMDS1307_Array#0     ' Second (0..59)
    Dim VMinute    As SMDS1307_Array#1     ' Minute (0..59)
    Dim VHour      As SMDS1307_Array#2     ' Hour   (0..11 or 0..23)
    Dim VDayOfWeek As SMDS1307_Array#3     ' Day of the week (1..7)
    Dim VDay       As SMDS1307_Array#4     ' Date   (0..31)
    Dim VMonth     As SMDS1307_Array#5     ' Month  (1..12)
    Dim VYear      As SMDS1307_Array#6     ' Year   (0..99)
'===============================================================================

'###############################################################################
'*******************************************************************************
'* Name    : State Machine Library                                             *
'* Purpose : All Special Commands are written in this library.                 *
'* Notes   :                                                                   *
'******************************************************************************* 
'    Include "D:\MyIncFiles\STMachine02.Inc"
    Include "INC\STMachine02.Inc"    
'*******************************************************************************
'=============================================================================== 
' Goto over the macros.
    GoTo UserFunctionsExit
'*******************************************************************************
'* Name    : USER FUNCTION 1 (Meta Macro and Subroutine)                       *
'* Purpose : Toggle the BUZZER in alarm state                                  *
'* Input   : A flag Timer:                                                     *
'* Output  : bBUZZER                                                           *
'* Notes   : Using the F_UserFunction1 flag                                    *
'*******************************************************************************  
' Use this macro in the main code.
    $define StartToggleBuzzer() F_UserFunction1 = 1

'-------------------------------------------------------------------------------
' Use this macro in the main code.
    $define StopToggleBuzzer() F_UserFunction1 = 0 : bBUZZER = 0  

'-------------------------------------------------------------------------------
' Use this macro in the USER FUNCTIONS SECTION in StartStateMachine1.
    $define ToggleBuzzerCode(pTimingFlag) '
    If F_UserFunction1 = 1 Then           '
        Output bBUZZER                    '
        bBUZZER = pTimingFlag             ' 
    EndIf

'*******************************************************************************
'* Name    : USER FUNCTION 2 (Meta Macro And Subroutine)                       *
'* Purpose : Toggle the LED config in configuration states                     *
'* Input   : None                                                              *
'* Output  : bLEDConfig                                                        *
'* Notes   :                                                                   *
'******************************************************************************* 
' Use this macro in the main code.
    $define StartToggleConfig() F_UserFunction2 = 1

'-------------------------------------------------------------------------------
' Use this macro in the main code.
    $define StopToggleConfig() F_UserFunction2 = 0 : bLEDConfig = 0  

'-------------------------------------------------------------------------------
' Use this macro in the USER FUNCTIONS SECTION in StartStateMachine1.
    $define ToggleConfigCode(pTimingFlag) '
    If F_UserFunction2 = 1 Then           '
        Output bLEDConfig                 '
        bLEDConfig = pTimingFlag          ' 
    EndIf
'-------------------------------------------------------------------------------
' Use this macro in the USER FUNCTIONS SECTION in StartStateMachine1.
'    $define Function2Code() '

    
'*******************************************************************************
'* Name    : USER FUNCTION 3 (Meta Macro And Subroutine)                       *
'* Purpose : Not used                                                          *
'* Input   : None                                                              *
'* Output  : None                                                              *
'* Notes   :                                                                   *
'******************************************************************************* 

'-------------------------------------------------------------------------------
' Use this macro in the USER FUNCTIONS SECTION in StartStateMachine1.
'    $define Function3Code() '


'*******************************************************************************
'* Name    : USER FUNCTION 4 (Meta Macro And Subroutine)                       *
'* Purpose : Not used                                                          *
'* Input   : None                                                              *
'* Output  : None                                                              *
'* Notes   :                                                                   *
'******************************************************************************* 

'-------------------------------------------------------------------------------
' Use this macro in the USER FUNCTIONS SECTION in StartStateMachine1.
'    $define Function4Code() '


'*******************************************************************************
'* Name    : USER FUNCTION 5 (Meta Macro And Subroutine)                       *
'* Purpose : Not used                                                          *
'* Input   : None                                                              *
'* Output  :                                                                   *
'* Notes   :                                                                   *
'******************************************************************************* 

'-------------------------------------------------------------------------------
' Use this macro in the USER FUNCTIONS SECTION in StartStateMachine1.
'    $define Function5Code() '


'*******************************************************************************
'* Name    : USER FUNCTION 6 (Meta Macro And Subroutine)                       *
'* Purpose : Synchronize ADC code with interrupt timing 100mS                  *
'* Input   :                                                                   *
'* Output  :                                                                   *
'* Notes   : Use this macro in the USER FUNCTIONS SECTION in StartStateMachine1*
'******************************************************************************* 
'    $define RunFunction6() F_UserFunction6 = 1
'-------------------------------------------------------------------------------
'    $define StopFunction6() F_UserFunction6 = 0
'-------------------------------------------------------------------------------
' Example of code to execute a code every 100mS in background with SM2.
'$define Function6Code(pState)           '


'*******************************************************************************
'* Name    : USER FUNCTION 7 (Meta Macro And Subroutine)                       *
'* Purpose : Not used                                                          *
'* Input   : None                                                              *
'* Output  :                                                                   *
'* Notes   :                                                                   *
'******************************************************************************* 

'-------------------------------------------------------------------------------
' Use this macro in the USER FUNCTIONS SECTION in StartStateMachine1.
'    $define Function7Code() '


'*******************************************************************************
'* Name    : USER FUNCTION 8 (Meta Macro And Subroutine)                       *
'* Purpose : Not used                                                          *
'* Input   : None                                                              *
'* Output  :                                                                   *
'* Notes   :                                                                   *
'******************************************************************************* 

'-------------------------------------------------------------------------------
' Use this macro in the USER FUNCTIONS SECTION in StartStateMachine1.
'    $define Function8Code() '


'*******************************************************************************
' NO MORE USER FUNCTION
'*******************************************************************************
'* Name    : SetBackLightTime                                                  *
'* Purpose : Set the BackLight in main code                                    *
'* Input   : None                                                              *
'* Output  : None                                                              *
'* Notes   : SetBackLightTime(Max) => NO delay                                 *
'******************************************************************************* 
' Continuous setting at maximum value.
$ifdef _EnableBackLightSystem_
    Dim BackLightTime As Byte

$define SetBackLightTime(pTime) '
    $if pTime = Max             '
    PWMBLCounterValue = 11      '
    $else                       '
    BackLightTime = pTime + 1   '
    PWMBLCounterValue = 11      '
    $endif

$endif
'*******************************************************************************
'* Name    : OpenADCAN4                                                        *
'* Purpose : Open the ADC channel 4                                            *
'* Input   : None                                                              *
'* Output  : None                                                              *
'* Notes   : Write only once in all program                                    *
'******************************************************************************* 
$define OpenADCAN4() GoSub ConfigureADC_Sub
        
'*******************************************************************************
'* Name    : Enable/Disable the ADC                                            *
'* Purpose :                                                                   *
'* Input   : None                                                              *
'* Output  : None                                                              *
'* Notes   : ADC channel 4                                                     *
'******************************************************************************* 
$define SM_EnableADCInt() F_DisableADCInt = 0

$define SM_DisableADCInt() F_DisableADCInt = 1

'*******************************************************************************
'* Name    : Save Clock parameters to eeprom memory                            *
'* Purpose :                                                                   *
'* Input   : None                                                              *
'* Output  : None                                                              *
'* Notes   : Write data a the end of the eeprom memory.                        *
'******************************************************************************* 
    Dim MyAlarmDS1307_Array[5] As Byte
    Dim zMinute    As MyAlarmDS1307_Array#0     
    Dim zHour      As MyAlarmDS1307_Array#1     
    Dim zDay       As MyAlarmDS1307_Array#2     
    Dim zMonth     As MyAlarmDS1307_Array#3     
    Dim zYear      As MyAlarmDS1307_Array#4 

' Address of clock parameters in eeprom at the end of memory.
    Symbol EEADR2_Alarm2Avaible = _eeprom - 12 
    Symbol EEADR2_Minute = _eeprom - 11     ' Minute (0..59)
    Symbol EEADR2_Hour = _eeprom - 10       ' Hour    (0..11 or 0..23)

    Symbol EEADR1_Alarm1Avaible = _eeprom - 9 
    Symbol EEADR1_Minute = _eeprom - 8     ' Minute (0..59)
    Symbol EEADR1_Hour = _eeprom - 7       ' Hour    (0..11 or 0..23)
    Symbol EEADR1_Day = _eeprom - 6        ' Date  (0..31)
    Symbol EEADR1_Month = _eeprom - 5      ' Month (1..12)
    Symbol EEADR1_Year = _eeprom - 4       ' Year  (0..99)
    ' The _eeprom -1, -2, -3 locations are reserved for the LCD parameters.
'-------------------------------------------------------------------------------    
$define SaveE2PTimeDate1() GoSub SaveE2PTimeDate1_Sub

SaveE2PTimeDate1_Sub:
    EWrite EEADR1_Alarm1Avaible,[$5A,VMinute,VHour,VDay,VMonth,VYear]
    F_Alarm1Avaible = 1
    F_EnableAlarm1 = 1
    Return
'------------------------------------------------------------------------------- 
$define SaveE2PTimeDate2() GoSub SaveE2PTimeDate2_Sub

SaveE2PTimeDate2_Sub:
    EWrite EEADR2_Alarm2Avaible,[$5A,VMinute,VHour]
    F_Alarm2Avaible = 1
    F_EnableAlarm2 = 1
    Return
'*******************************************************************************
'* Name    : Save/Restore Alarm parameters to/from eeprom memory               *
'* Purpose :                                                                   *
'* Input   : None                                                              *
'* Output  : None                                                              *
'* Notes   :                                                                   *
'*******************************************************************************
' Declare local variables.
    Dim ALAvaible As SMLVSByte1

$define CheckAlarm1Avaible() GoSub CheckAlarm1Avaible_Sub

CheckAlarm1Avaible_Sub:
    ALAvaible  = ERead EEADR1_Alarm1Avaible
    If ALAvaible = $5A Then
        F_Alarm1Avaible = 1
    Else
        F_Alarm1Avaible = 0    
    EndIf
    Return

'-------------------------------------------------------------------------------
$define CheckAlarm2Avaible() GoSub CheckAlarm2Avaible_Sub

CheckAlarm2Avaible_Sub:
    ALAvaible  = ERead EEADR2_Alarm2Avaible
    If ALAvaible = $5A Then
        F_Alarm2Avaible = 1
    Else
        F_Alarm2Avaible = 0    
    EndIf
    Return
'-------------------------------------------------------------------------------
' Declare local variables.
$ifndef _SMLVWS_Block1_
    $define _SMLVWS_Block1_
    Dim SMLVSWord1 As Word     ' Local Variable Word1
    Dim SMLVSWord2 As Word     ' Local Variable Word2
$endif

    $if _eeprom > 256
    Dim CLK_Address As SMLVSWord1
    $else
    Dim CLK_Address As SMLVSByte2
    $endif
    Dim CLKLoop   As SMLVSByte3

$define RestoreE2PTimeDate1() GoSub RestoreE2PTimeDate1_Sub

RestoreE2PTimeDate1_Sub:
    CLK_Address = EEADR1_Minute
    CLKLoop = 0
    Repeat
        #ifdef WatchDog_Req
        Clrwdt
        #endif 
        MyAlarmDS1307_Array[CLKLoop] = ERead CLK_Address
        Inc CLKLoop
        Inc CLK_Address
    Until CLKLoop > 4
    Return
'-------------------------------------------------------------------------------
$define RestoreE2PTimeDate2() GoSub RestoreE2PTimeDate2_Sub

RestoreE2PTimeDate2_Sub:
    CLK_Address = EEADR2_Minute
    CLKLoop = 0
    Repeat
        #ifdef WatchDog_Req
        Clrwdt
        #endif 
        MyAlarmDS1307_Array[CLKLoop] = ERead CLK_Address
        Inc CLKLoop
        Inc CLK_Address
    Until CLKLoop > 1
    Return
'===============================================================================

'===============================================================================
UserFunctionsExit:
' END USER FUNCTIONS/END OF USER MACROS:
'*******************************************************************************
'* Name    : Amicus_ADC.Inc                                                    *
'* Purpose : ADC library For PIC K20 28 pins series                            *
'* Notes   : An error in this library has been fixed.                          *
'******************************************************************************* 
'    Include "D:\MyIncFiles\Amicus_ADCbeta.inc"
    Include "INC\Amicus_ADCbeta.inc"
'*******************************************************************************
    GoTo STM_START  ' Goto over the Interrupt Handler.
''*******************************************************************************
''* Name    : High Interrupt_Handler                                            *
''* Purpose : Generate a 1mS interrupt                                          *
''* Notes   : Using the Timer2 & PR2 it is more easy.                           *
''*******************************************************************************
'    Include "D:\MyIncFiles\SM_IntHandler01.Inc"
    Include "INC\SM_IntHandler01.Inc" 
'*******************************************************************************
'* Name    : EEPROM Data                                                       *
'* Purpose : Write some strings in the EEPROM memory                           *
'* Notes   : For testing the HRSE2PStrg(Address Eeprom) command                *
'*******************************************************************************
E2p_AddressString1 EData "Testing the STATE MACHINE STM07 scheduler",0

E2p_AddressString2 EData "using the Amicus18 Board (PIC18F25K20)",0

E2p_AddressString3 EData "and the AMI18-LCD.",0
'*******************************************************************************
STM_START:
'*******************************************************************************
' PLACE to insert the library files:
'*******************************************************************************
'* Name    : Declare the I2C pins                                              *
'* Purpose : For DS1307 Clock Calendar and TC74 Temperature sensor libraries   *
'* Notes   :                                                                   *
'*******************************************************************************  
' There is an issue with the PDS version 3.5.9.3.
' The I2C CLock for Xta = 80 Mhz is not correct.
' For 64 Mhz the SSPADD value is 39 => FClock = 100 Khz
' For 80 Mhz the SSPADD value is 249 => FClock = 25 Khz
' The workaround is X 4 the Hbus_Bitrate.
' Formula: FClock = 1/(((SSPADD + 1) *4)/FOSC)   SSPADD = 49 for Xtal = 80Mhz
'
'   Hardware/Software I2C pins
$if _xtal = 80
    Declare SDA_Pin = PORTC.4   ' SDA connected to this pin
    Declare SCL_Pin = PORTC.3   ' SCL connected to this pin
    Declare Hbus_Bitrate 400    ' Problem with declaration: 400 => 100Khz max For these circuits.
$else
    Declare SDA_Pin = PORTC.4   ' SDA connected to this pin
    Declare SCL_Pin = PORTC.3   ' SCL connected to this pin
    Declare Hbus_Bitrate 100    ' Bus frequency 100kHz max for these circuits.
$endif
'*******************************************************************************
'* Name    : DS1307-H.Inc                                                      *
'* Purpose : DS1307 Clock Calendar library                                     *
'* Notes   :                                                                   *
'******************************************************************************* 
    ' Library for the DS1307 Clock Calendar with ACK control.  
'    $define _EnableGIE_Save_
'    Include "D:\MyIncFiles\DS1307-H.Inc"    
    Include "INC\DS1307-H.Inc"
    '---------------------------------------------------------------------------
'    Include "D:\MyIncFiles\DS1307-S.Inc"    
'    Include "INC\DS1307-S.Inc"    
'*******************************************************************************
'* Name    : LCD_ST7036.Inc                                                    *
'* Purpose : LCD ST7036 driver                                                 *
'* Notes   : Take care, the contrast datas are in the end of eeprom memory.    *
'*******************************************************************************
    $ifndef _ProteusDebug_
'    Include "D:\MyIncFiles\LCD_ST7036.Inc"
    Include "INC\LCD_ST7036.Inc"    
    $endif
'*******************************************************************************
'* Name    : HRsout_K40.Inc                                                    *
'* Purpose : HRSout1,2,3,4,5 command for 1 or 2 stop bits                      *
'* Notes   :                                                                   *
'*******************************************************************************
    $define _DisableAllinterrupt_
'    Include "D:\MyIncFiles\HRsout_K40.Inc"
    Include "INC\HRsout_K40.Inc"    
'*******************************************************************************
'* Name    : ORG_SyncBlock_K20.Inc                                             *
'* Purpose : Generate an ORG command syncronized with Erase Blocks, K20 series *
'* Notes   : For Cdata Tables                                                  *
'******************************************************************************* 
'    Include "D:\MyIncFiles\ORG_SyncBlock_K20.Inc"
    Include "INC\ORG_SyncBlock_K20.Inc"
' Syntax: ORG_EBlock(pAddress)  
'*******************************************************************************
'* Name    : TC74-H.Inc                                                        *
'* Purpose : TC74 Temperature sensor library.                                  *
'* Notes   :                                                                   *
'******************************************************************************* 
'   The TC74 temperature sensor needs a correction because the LCD BackLight and self heating.
    $define TemperatureCorrection 2
    $define TempPolarityCorrection 1    ' 1 => negative, 0 => positive.    
'    Include "D:\MyIncFiles\TC74-H_beta.inc"
    Include "INC\TC74-H_Beta.inc"
    '---------------------------------------------------------------------------
'    Include "D:\MyIncFiles\TC74-S_beta.inc"
'    Include "INC\TC74-S_beta.inc"
'===============================================================================
' More library here...

'###MAIN CODE###################################################################
STMAIN:
'*******************************************************************************
'* Name    : Init some variables                                               *
'* Purpose :                                                                   *
'* Notes   :                                                                   *
'*******************************************************************************
    PORTA = 0       ' Clear all ports
    LATA = 0
    PORTB = 0
    LATB = 0    
    PORTC = 0
    LATC = 0
    Input PORTC.7
    Output PORTC.6
    IOCB = 0    ' Disable the interrupt function for all pins of PORTB.
'===============================================================================
    DelayMS 100     ' Delay for LCD to stabilize.
    Clrwdt
    DelayMS 100     ' Delay for LCD to stabilize.
    Clrwdt
    $ifdef _ST7036_INC_
    LCD_Reset()     ' Sure the reset is done.
    $else
    Cls
    $endif
    DelayMS 50
    Clrwdt
'-------------------------------------------------------------------------------
' Init the PORTS
    Low bBUZZER
    Low bLCD_BackLight      ' Clear the bLCD_BackLight.
    PWMBLCounterValue = 9   ' Start Value of PWM BackLight.
    Input bButton1
    Input bButton2
    Input bButton3
    Input bButton4
    $ifdef _PinsTest_
    Low bTest1
    Low bTest2
    $endif
    Low bAlarm1Out
'-------------------------------------------------------------------------------
' Dummy calculations to reduce ADC calculations time. (see SM2_ADCaverage)
' Because I need the Maths "__MULTIPLY_U3232_" and the "__DIVIDE_U3232_" routines!
DUMMYCALC:
    $ifdef NumberADCaverages
    ADC_DecimalResult = 1023 * NumberADCaverages
    ADCResult = (ADCToPrint * 5000) / ADC_DecimalResult 
    ADC_IntegerResult = ADCResult / 1000
    ADC_DecimalResult = ADCResult // 1000
    $else
    ADCResult = (ADC_ResultW * 5000) / 1023
    ADC_IntegerResult = ADCResult / 1000
    ADC_DecimalResult = ADCResult // 1000  
    $endif
'-------------------------------------------------------------------------------

'*******************************************************************************
'* Name    : InitStateMachine()                                                *
'* Purpose : Initialize some variables and the State Machine                   *
'* Input   : None                                                              *
'* Output  : None                                                              *
'* Notes   : The SWDTEN bit is set then the Wachtdog is ON.                    *
'*******************************************************************************
    ' Clear All flags to begin.
    SM_UserSystemFlags1 = 0
    SM_UserSystemFlags2 = 0
    SM_UserSystemFlags3 = 0                 
'-------------------------------------------------------------------------------
     InitStateMachine()
'###############################################################################
'#### STATE MACHINE STARTS #####################################################
' The Main code Start Here:
StateMachineInit:
'*******************************************************************************
'* Name    : NextState(0)                                                      *
'* Purpose : Initialize the State Machine.                                     *
'* Notes   : Start the Init state.                                             *
'*******************************************************************************
    NextState(0)
'*******************************************************************************
'* Name    : StartStateMachine1                                                *
'* Purpose : Clear the Watchdog, it is always enabled.                         *
'* Notes   : The SM_Return() goes Here:                                        *
'*******************************************************************************
StartStateMachine1:
    Clrwdt  ' Only one Clrwdt for the state machine!!!!
            ' Config Fuses: WDTPS = 128 mS.
'###############################################################################
' USER AND SPECIAL FUNCTIONS SECTION
'*******************************************************************************
'* Name    : USER Functions                                                    *
'* Purpose : Write some Timing functions                                       *
'* Notes   : Use short code only to avoid delaying the state machine           *
'*******************************************************************************

'*******************************************************************************
'* Name    : User FUNCTION1                                                    *
'* Purpose : Toggle the BUZZER output every 1 second                           *
'* Input   : Name of the flag TimerN: F_ToggleSignal1                            *
'* Output  : An output PORT bit: bBUZZER                                       *
'* Notes   : Start, Stop, Code                                                 *
'*******************************************************************************
FUNCTION1:
'# Running.
    ToggleBuzzerCode(F_ToggleSignal1)
'*******************************************************************************
'* Name    : User FUNCTION2                                                    *
'* Purpose : Not used                                                          *
'* Notes   :                                                                   *
'*******************************************************************************
FUNCTION2:
'# Running.
    ToggleConfigCode(F_ToggleSignal1)
'*******************************************************************************
'* Name    : User FUNCTION3                                                    *
'* Purpose : Not used                                                          *
'* Notes   :                                                                   *
'*******************************************************************************
'    Function3Code()

'*******************************************************************************
'* Name    : User FUNCTION4                                                    *
'* Purpose : Not used                                                          *
'* Notes   :                                                                   *
'*******************************************************************************
'    Function4Code()

'*******************************************************************************
'* Name    : User FUNCTION5                                                    *
'* Purpose : Not used                                                          *
'* Notes   :                                                                   *
'*******************************************************************************
'    Function5Code()

'*******************************************************************************
'* Name    : User FUNCTION6                                                    *
'* Purpose : Not used                                                          *
'* Notes   :                                                                   *
'*******************************************************************************
'    Function6Code()
 
'*******************************************************************************
'* Name    : User FUNCTION7                                                    *
'* Purpose : Not used                                                          *
'* Notes   :                                                                   *
'*******************************************************************************
'    Function7Code()

'*******************************************************************************
'* Name    : User FUNCTION8                                                    *
'* Purpose : Not used                                                          *
'* Notes   :                                                                   *
'*******************************************************************************
'    Function8Code()

'===============================================================================
' END USER FUNCTIONS SECTION
'###############################################################################

'### STATE MACHINE FUNCTIONS IN ################################################
'*******************************************************************************
'* Name    : RunPulseOutCode()                                                 *
'* Purpose : Set a pulse to PORT pin                                           *
'* Notes   : Start the pulse with SMPulseOut(pPin,pValue) command              *
'*******************************************************************************
'# Running.
    RunPulseOutCode()  ' Generic Pulse Pin Code.
'###############################################################################

'####-SM1 SCHEDULER-############################################################
'    Include "D:\MyIncFiles\SM1_Scheduler01.Inc" 
    Include "INC\SM1_Scheduler01.Inc"
'####-SM1 SCHEDULER-############################################################

'### USER CODE MODULES #########################################################
'*******************************************************************************
'* Name    : STATE00                                                          
'* Purpose : Initialize the project in the terminal window and LCD.                   
'* Input   : None                                                              
'* Output  : None                                                              
'* Notes   : Start the first state.                                            
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE00:
'===============================================================================
'    StateInit() 
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################

    '##-END INIT-###############################################################
'    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    HRSOut CR,LF    
    '---------------------------------------------------------------------------
'    StartToggleBuzzer() ' Toggle the BUZZER every second (code from Reset)
    '---------------------------------------------------------------------------
    ' Open an AN4 ADC channel in PORTA.5.
    $ifdef _EnableADCSystem_
    SM_DisableADCInt()
    OpenADCAN4()
    $endif
    '---------------------------------------------------------------------------
    Temperature = 99    ' Don't print the temperature before reading the T sensor.
    '---------------------------------------------------------------------------
    $ifdef NumberADCaverages
    TrueAverageHead = 0
    ADC_Average = 0
    ADCToPrint = 0
    AverageLoop = 0
    $endif
    CountStrings = 0
    '---------------------------------------------------------------------------    
    ' Terminal
    HRSOut "STATE MACHINE STM07",CR,LF
    HRSOut "By A. Freixanet",CR,LF
    HRSOut CR,LF
    '---------------------------------------------------------------------------
    ' LCD
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear()
    ' Print all the errors to the LCD Display at start up.
    Cursor 1,1
    If SM_Errors > 0 Then
        If F_Reset = 1 Then
            Print "RESET ERROR"
        ElseIf F_Watchdog = 1 Then
            Print "WATCHDOG RESET"    
        ElseIf F_BrownOut = 1 Then
            Print "BROWN-OUT RESET"
        ElseIf F_StackFull = 1 Then
            Print "STACK FULL"            
        ElseIf F_StackUnderflow = 1 Then
            Print "STACK UNDERFLOW"    
        EndIf
    Else
        Print "STATE MACHINE"
        Print At 2,1, "By A. Freixanet"
    EndIf
    '---------------------------------------------------------------------------
    NextStateDelay(3000,IncState)  ' Goto State 01 with 3Sec delay to see the LCD. 

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'    StartDelayMS(0) ' Delay between execution of MAIN module (adjust the delay)
'===============================================================================
'    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
'    StateOutEnd()    
'===============================================================================    
    SM_Return()
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE01 [SM_ProjectInfo]   
'* Purpose : Initialize the project in the terminal window                     
'* Input   : None                                                              
'* Output  : None                                                              
'* Notes   : Start the first state.                                            
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:
    Dim Loop As SMLVSByte1
    Dim DataConfig2H As SMLVSByte2
    Dim DataTemp As SMLVSByte3    
' Define the Symbols used:
    Symbol _ID_LOCATION = $200000   ' Write the IDLOCS address. Eight memory locations (200000h-200007h) are designated, see the datasheet
    Symbol _Config2h = $300003      ' Address in ROM of the WDT config, see the datasheet
'===============================================================================
STATE01:
'===============================================================================
    StateInit() 
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################

    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    $ifdef _ST7036_INC_
    HRSLStrg(TXT33,1)   ' "Using the ST7036 LCD with the LCD Driver <LCD_ST7036.inc>"
    HRSLStrg(TXT28,1)   ' "Using the library <DS1307-H.Inc>"
    SetBackLightTime(Max)           ' Set the Backlight to see the text on the LCD.
    LCD_Clear()
    PrintStrg(TXT33,1,1,16)         ' "Using the ST7036" Print 16 characters only.
    PrintStrg(TXT34,2,1,AllChars)   ' "<LCD_ST7036.Inc>"
    $endif

    HRSLStrg(TXT3,1)    ' "Testing the DS1307 Clock Calendar"
    HRSLStrg(TXT4,1)    ' "Sending Time every second & Date every minute To the terminal."    
    HRSLStrg(TXT5,0)    ' "The Hserial Baud Rate is set to "  
'===============================================================================
    HRSOut Dec _HSerialBaud_
    HRSLStrg(TXT23,1)    ' " Bauds."
'===============================================================================
    ' Testing the Xtal used.
    $if _xtal = 16
        $ifdef _ProteusDebug_
        HRSLStrg(TXT31,1)   ' "Xtal = 16 Mhz for Proteus testing"
        $else
        HRSLStrg(TXT90,1)   ' "Xtal must be 64 or 80 Mhz For real piece"
        $endif
    $elseif _xtal = 64
    HRSLStrg(TXT32,1)   ' "Xtal = 64 Mhz For real piece testing"    
    $elseif _xtal = 80
    HRSLStrg(TXT89,1)   ' "Xtal = 80 Mhz For real piece testing"    
    $else
    HRSLStrg(TXT90,1)   ' "Xtal must be 64 or 80 Mhz For real piece"
    $endif
'===============================================================================
    ' Sending more information from the eeprom to the terminal.
    HRSOut CR,LF
    ' Send 1st String.
    HRSE2PStrg(E2p_AddressString1,1)
    ' Send 2nd String.
    HRSE2PStrg(E2p_AddressString2,1)
    ' Send 3rd String.
    $ifdef _ST7036_INC_    
    HRSE2PStrg(E2p_AddressString3,1)
    $endif
'===============================================================================
    ' READING THE IDLOCS...
    HRSOut CR,LF
    HRSLStrg(TXT35,0)               ' "Project Version (Read PIC ID):"
    EECON1 = %11000000              ' Access Configuration registers
    TBLPTRL = _ID_LOCATION & 255
    TBLPTRH = (_ID_LOCATION >> 8) & 255
    TBLPTRU = (_ID_LOCATION >> 16) & 255
    For Loop = 0 To 7       ' Start the loop
        Tblrd*+             ' Read one memory position.
        HRSOut TABLAT       ' Read ASCII characters
    Next  
    EECON1 = 0  ' Disable read the Configuration registers
    TBLPTRU = 0 ' The HRSOut command of the compiler does not clear the "TBLPTRU", then take care of that.
                ' The HRSOut command can crash if the TBLPTRU SFR is not cleared.
    HRSOut CR,LF
'===============================================================================
' ADC informations
    $if TaskADC_Interval > 0
    HRSOut "Read the ADC every ", Dec TaskADC_Interval, " mS.",CR,LF
    $endif
    $ifdef NumberADCaverages
    HRSOut "Number of Averages: ", Dec NumberADCaverages,CR,LF
    $else
    HRSOut "NO AVERAGE",CR,LF   
    $endif
'===============================================================================
' Read config Fuses.
'    bit 7-5 Unimplemented: Read as 0
'    bit 4-3 BORV<1:0>: Brown-out Reset Voltage bits(1)
'    11 = VBOR set to 1.8V nominal
'    10 = VBOR set to 2.2V nominal
'    01 = VBOR set to 2.7V nominal
'    00 = VBOR set to 3.0V nominal
'    bit 2-1 BOREN<1:0>: Brown-out Reset Enable bits(2)
'    11 = Brown-out Reset enabled in hardware only (SBOREN is disabled)
'    10 = Brown-out Reset enabled in hardware only and disabled in Sleep mode
'    (SBOREN is disabled)
'    01 = Brown-out Reset enabled and controlled by software (SBOREN is enabled)
'    00 = Brown-out Reset disabled in hardware and software
'    bit 0 PWRTEN: Power-up Timer Enable bit(2)
'    1 = PWRT disabled
'    0 = PWRT enabled

' Read the WDT config in memory map.
    EECON1 = %11000000              ' Access Configuration registers
    TBLPTRL = _Config2h & $0000FF
    TBLPTRH = (_Config2h >> 8) & 255
    TBLPTRU = (_Config2h >> 16) & 255
    Tblrd*                          ' Read one memory position.  
    DataConfig2H = TABLAT           ' Copy the TABLAT SFR immediatly because it is cleared out of this routine??
    EECON1 = 0                      ' Disable Access the Configuration registers
    TBLPTRU = 0                     ' The HRSOut command of the compiler does not clear the "TBLPTRU = 0", then take care of that.

    HRSOut CR,LF
    HRSOut "CONFIG FUSES: ",CR,LF
    $if (_device = _18F23K20) Or (_device = _18F24K20) Or (_device = _18F25K20) Or (_device = _18F26K20) Or (_device = _18F43K20) Or (_device = _18F44K20) Or (_device = _18F45K20) Or (_device = _18F46K20)
    DataTemp = DataConfig2H & 1
    If DataTemp = 1 Then
        HRSOut "WDT enabled in hardware; SWDTEN bit disabled."    
    Else
        HRSOut "WDT controlled by the SWDTEN bit."   
    EndIf
    $endif
    HRSOut CR,LF

' Check the SWDTEN Bit.
    HRSOut "Be carrefull the WDT is always active!",CR,LF  
    If DataTemp = 0 Then
        HRSOut "After Reset the "
        If WDTCONBits_SWDTEN = 1 Then
            HRSOut "SWDTEN Bit = 1",CR,LF
        Else
            HRSOut "SWDTEN Bit = 0",CR,LF
            If WDTCONBits_SWDTEN = 1 Then
                HRSOut "Now the SWDTEN Bit is set => Watchdog Enabled.",CR,LF
            Else
                HRSOut "Now the SWDTEN Bit is Cleared => Watchdog Disabled.",CR,LF    
            EndIf
        EndIf
    EndIf
'===============================================================================
' Config Fuses:   BOREN = On    ;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    Clrwdt
    DataTemp = (DataConfig2H >> 1) & %00000011
    If DataTemp = 0 Then
        HRSOut "Brown-out Reset disabled in hardware And software",CR,LF
    ElseIf DataTemp = 1 Then
        HRSOut "Brown-out Reset enabled And controlled by software (SBOREN is enabled)",CR,LF    
    ElseIf DataTemp = 2 Then    
        HRSOut "Brown-out Reset enabled in hardware only And disabled in Sleep mode (SBOREN is disabled)",CR,LF
    ElseIf DataTemp = 3 Then  
        HRSOut "Brown-out Reset enabled in hardware only (SBOREN is disabled)",CR,LF
    EndIf
    If RCONBits_SBOREN = 1 Then
        HRSOut "Brown-out Reset (SBOREN bit) enabled!",CR,LF 
    Else
        HRSOut "Brown-out Reset (SBOREN Bit) disabled!",CR,LF     
    EndIf
'===============================================================================
    ' Check if some Alarm is available in eeprom memory and restore the alarm status.
    CheckAlarm1Avaible()
    CheckAlarm2Avaible()
    '---------------------------------------------------------------------------
    F_MenuContrastFromReset = 1
    '---------------------------------------------------------------------------
    NextStateDelay(2500,IncState)   ' With delay because there are some strings 
                                    ' printed to the LCD.
':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0) ' Delay between execution of MAIN module (adjust the delay)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    HRSOut CR,LF
    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================    
    SM_Return()
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE02  (SM_DS1307CheckACK)                                                        
'* Purpose : Check if the DS1307 is present on the I2C bus.                               
'* Input   : None                                                             
'* Output  : None                                                            
'* Notes   : The ReturnACK must to be = 0.                                                                 
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE02:
'===============================================================================
    StateInit() 
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    HRSLStrg(TXT11,1)       ' "Check the DS1307 ackknowledge."
    HRSLStrg(TXT12,1)       ' Check if the DS1307 Clock Calendar is connected?
    '##-END INIT-###############################################################
    StateInitEnd(SyncOn)    ' Module synchronized with timing 1mS.
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    ' Only check value = 0, other values could be = 1 to 255.
    DS1307_Present(ReturnACK)
    If ReturnACK = 1 Then
        HRSLStrg(TXT14,1)           ' "NACK DS1307 RTC device!", CR
        NextState(SM_DS1307Error_NACK)    ' The program demo cannot run.
    Else
        HRSLStrg(TXT13,1)           ' "The DS1307 Device is ready!", CR
        F_ErrorNACK_DS1307 = 0      ' Reset the DS1307 error
        NextState(SM_TC74CheckACK)  ' Now check the temperature sensor.
    EndIf  

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)    ' Delay between execution of the MAIN module (adjust the delay)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    HRSOut CR,LF
    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================    
    SM_Return()
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE03  (SM_Enable1Hz)                                                          
'* Purpose : Config the DS1307 to output the 1Hz signal.                                  
'* Input   : None                                                               
'* Output  : None                                                               
'* Notes   : Goto the menu 1.                                                                   
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE03:
'===============================================================================
    StateInit() 
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    HRSLStrg(TXT15,1)       ' "Enable signal Output with frequency = 1 Hz."
    '##-END INIT-###############################################################
    StateInitEnd(SyncOn)    ' Module synchronized with timing 1mS.
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    ' Write to the Config register: Enable signal output with frequency = 1 Hz.
    DS1307_WriteControl(%10010000,ReturnACK)
    If ReturnACK = 1 Then
        HRSLStrg(TXT16,1)                   ' "NACK DS1307 #00 Write Control", CR
        NextState(SM_DS1307Error_NACK)            ' Go to SM_DS1307Error_NACK State.
    Else
        AllButtons = 0
        StartTimeOut(SM_AskForConfig,10)    ' Start a Time Out for 10 seconds.
        NextState(SM_ContrastSetup)         ' Menu Configuration of the LCD contrast.
    EndIf

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)  ' Delay between execution of MAIN module (adjust the delay)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    RTC_MinuteOld = 255     ' Force to Print the date for reading at first time.
    HRSOut CR,LF
    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================    
    SM_Return()
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE04  (SM_WriteConfig)                                         *
'* Purpose : Configure a valor to the DS1307.                                  *
'* Input   : None                                                              *
'* Output  : None                                                              *
'* Notes   : Write All data in the DS1307 calendar                             *
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE04:
'===============================================================================
    StateInit() 
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    HRSLStrg(TXT61,1)       ' "Update the Time & Date"
    '##-END INIT-###############################################################
    StateInitEnd(SyncOn)    ' Module synchronized with timing 1mS.
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

' Code for configuring the DS1307. Only write it once to PIC. 
' If the RTC have a Battery only, compile again without this lines
'  & write to the PIC again.
    ' Config Time and Date to the clock calendar.

    DS1307_WriteDate(VDayOfWeek, VDay, VMonth, VYear, ReturnACK)    ' Write a new date
    If ReturnACK = 1 Then
        HRSLStrg(TXT29,1)               ' "NACK DS1307 #01 Write Date"
        NextState(SM_DS1307Error_NACK)        ' The program demo cannot run. 
        GoTo STATE04EXIT
    EndIf
    '---------------------------------------------------------------------------
    Sync1mS()   ' DS1307_WriteTime command synchronized with timing 1mS.
    '---------------------------------------------------------------------------
    DS1307_WriteTime(VHour, VMinute, 0, ReturnACK) ' Write a new time   
    If ReturnACK = 1 Then
        HRSLStrg(TXT30,1)               ' "NACK DS1307 Write Time"
        NextState(SM_DS1307Error_NACK)        ' The program demo cannot run. 
        GoTo STATE04EXIT
    EndIf
    '---------------------------------------------------------------------------
    NextState(SM_ReadTimeDate)          ' Write with success.

STATE04EXIT:
':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0) ' Delay between execution of MAIN module (adjust the delay)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================    
    SM_Return()
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE05 (SM_ReadTimeDate)                                                         
'* Purpose : READING Time & Date                                          
'* Input   : None                                                              
'* Output  : None                                                              
'* Notes   : MAIN program: Read the Time every second & Read the date every minute.
'*           Set the Backlight to 5 Sec every minute to see the Time & Date on the LCD.
'*           INFINITE LOOP                                                                 
'===============================================================================
' Define the Global variables used:
    Dim ButtonsVirtualPORT As Byte
    Dim F_SW1 As ButtonsVirtualPORT.0
    Dim F_SW2 As ButtonsVirtualPORT.1    
    Dim F_SW3 As ButtonsVirtualPORT.2    
    Dim F_SW4 As ButtonsVirtualPORT.3
    Dim RTC_DayName As String * 9         ' Holds the name of the day
    Dim RTC_MonthName As String * 10      ' Holds the name of the month
' Define the Local variables used in this state:

' Define the Symbols used :

'===============================================================================
STATE05:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    LoadTask(SM2_PrintLCDLine1) ' Print something To avoid a blank LCD line1.
    StopToggleConfig()          ' Stop Blinking LED while configuration is running.
'    StopToggleBuzzer()          ' Stop Toggle the BUZZER
    SetBackLightTime(5) ' Set the Backlight 5 seconds to see the text on the LCD.
    '---------------------------------------------------------------------------
    ' Initialise some values.
    F_MenuContrastFromReset = 0 ' Reset finished.  
    DisableTimeOut()            ' Only for security.
    '---------------------------------------------------------------------------
    $ifdef _EnableADCSystem_
    $ifdef _CheckADCSystem_
    ADCCounter = 0
    ADCCounterOld = 0
    $endif
    SM_EnableADCInt()   ' Enable the ADC interrupt.
    $endif
    '---------------------------------------------------------------------------
    '##-END INIT-###############################################################
    StateInitEnd()   
    ' Static State.
    '---------------------------------------------------------------------------
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    ' Check comunications errors from the Tasks
    If F_ErrorNACK_DS1307 = 1 Then
        F_ErrorNACK_DS1307 = 0
        NextState(SM_DS1307Error_NACK)
        GoTo ERROR05EXIT
    EndIf
    If F_ErrorNACK_TC74 = 1 Then
        F_ErrorNACK_TC74 = 0
        NextState(SM_TC74Error_NACK)
        GoTo ERROR05EXIT
    EndIf
    '===========================================================================
    ' DO every second.
    If F_TaskTimer1S = 1 Then
'    If F_DS1307_1S = 1 Then
    '---------------------------------------------------------------------------
        F_TaskTimer1S = 0
'        F_DS1307_1S = 0
        '-----------------------------------------------------------------------
        LoadTask(SM2_ReadTime)
        '-----------------------------------------------------------------------
        ' DO JOB every second.
        ' Decrement the Delay to stop the Backlight.
        $ifdef _EnableBackLightSystem_
        If BackLightTime > 0 Then                
            Dec BackLightTime           ' Decrement the Backlight counter every second.
            If BackLightTime = 0 Then
                PWMBLCounterValue = 3   ' Minimum value (30%) of the PWM BackLight.
            EndIf
        EndIf    
        $endif
    '---------------------------------------------------------------------------
    Else    
    '---------------------------------------------------------------------------
        '=======================================================================
        ' DO a job Every Minute.
        If RTC_Minute <> RTC_MinuteOld Then
            '-------------------------------------------------------------------
            ' Start a code (every minute) in BackGround with the States Machine2.
            ' This code is executed always after this module.
            LoadTask(SM2_ReadDate)
            LoadTask(SM2_CheckAlarmState)
            LoadTask(SM2_Data1ToTerminal)
            '-------------------------------------------------------------------
            ' Delayed Task: 1 to 65535 mS (65,535 Seconds max).
            ' Reading the TC74 Temperature sensor every minute but time shifted of 30 seconds.
            LoadDelayedTask(30000,SM2_TC74SetNormalMode)
           '--------------------------------------------------------------------
           ' Clear the Buttons FAULT. ' Try checking again if it has been fixed.
            AllButtonsBitsDefects = %11110000  
            ButtonFail = 0            
        EndIf                        
        '=======================================================================
        ' Press a Button if any function is needed again.
        GoSub ReadButtons2PORT_Sub  ' There is not a debounce code.
        
        ' Press the Button1 to go to the Contrast Setup (menu).
        If F_SW1 = 0 Then
            StartTimeOut(SM_ReadTimeDate,10) ' Start a Time Out For 10 seconds.
            NextIndState(SM_ButtonsOFF2,SM_ContrastSetup)
        EndIf
            
        ' Press the Button2 to go to the DS1307 Configuration setup (menu).
        If F_SW2 = 0 Then
            StartTimeOut(SM_ReadTimeDate,10) ' Start a Time Out For 10 seconds.
            NextIndState(SM_ButtonsOFF2,SM_AskForConfig)            
        EndIf

        ' Press the Button3 to enable the Day Alarm.
        If F_SW3 = 0 Then       ' If F_SW3 pressed...
            NextIndState(SM_ButtonsOFF2,SM_SeeAlarm2)
        EndIf

        ' Press the Button4 To disable the Alarms.
        If F_SW4 = 0 Then       ' If F_SW4 pressed...
            NextIndState(SM_ButtonsOFF2,SM_DisableAlarms)
        EndIf
        AllButtons = 0
        '=======================================================================
        ' Check if an alarm is on, detected by the SM2_CheckAlarmState.
        If F_AlarmX = 1 Then
            F_AlarmX = 0
            AllButtons = 0
            NextState(SM_Alarm)
        EndIf
    '---------------------------------------------------------------------------
    EndIf
'-------------------------------------------------------------------------------
ERROR05EXIT:
'    $ifdef _PinsTest_
'    Clear bTest2
'    $endif
':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0) ' Delay between execution of MAIN module (adjust the delay)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    SM_DisableADCInt()             ' Disable the ADC interrupt.
    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()
'===============================================================================
    SM_Return() 
'===============================================================================
$ifdef _EnableADCSystem_
ConfigureADC_Sub:
    ' Configure the ADC channel every time to use is better againts high noise in input pins.
    ' Open an AN4 ADC channel in PORTA.5 for PIC18F25K20.
    ' Commands from the ADC library
    $ifdef _ProteusDebug_
    OpenADC(ADC_FOSC_16 & ADC_RIGHT_JUST & ADC_16_TAD, ADC_CH4, ADC_REF_VDD_VSS, ADC_4ANA_ONLY)
'    OpenADC(ADC_FOSC_FRC1 & ADC_RIGHT_JUST & ADC_16_TAD, ADC_CH4, ADC_REF_VDD_VSS, ADC_4ANA_ONLY) 
    $else
        $if _xtal = 64
        OpenADC(ADC_FOSC_64 & ADC_RIGHT_JUST & ADC_16_TAD, ADC_CH4, ADC_REF_VDD_VSS, ADC_4ANA_ONLY)    
        $else
        OpenADC(ADC_FOSC_FRC1 & ADC_RIGHT_JUST & ADC_16_TAD, ADC_CH4, ADC_REF_VDD_VSS, ADC_4ANA_ONLY)    
        $endif    
    $endif
    ' ADC working with interrupt for best timing.
    ADC_IntEnable()     
    Return
$endif
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE06 (SM_DS1307Error_NACK)                                                          
'* Purpose : Press any Button To try the communication again.  
'* Input   : None                                                              
'* Output  : None                                                              
'* Notes   :                                                                  
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used :

'===============================================================================
STATE06:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    HRSLStrg(TXT22,1)       ' "Warning: Communication error! ACK no valid."
    HRSLStrg(TXT25,1)       ' "Press the Button 2 to ReStart the programm."
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear() 
    PrintStrg(TXT26,1,1)    ' "NACK DS1307"
    PrintStrg(TXT27,2,1)    ' "RESTART any SW"
    SMPulseOut(RB0,250)       ' Start a pulse of the bBUZZER
    '##-END INIT-###############################################################
    StateInitEnd()    
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    GoSub ReadButtons2PORT_Sub                      ' Read All Buttons.
    '---------------------------------------------------------------------------
    ' If any Button pressed... in case of a Button fails...
    If F_SW1 = 0 Or F_SW2 = 0 Or F_SW3 = 0 Or F_SW4 = 0 Then
        CLL(2)
        PrintStrg(TXT39,2,1)    ' "TRY DS1307 AGAIN"
        NextIndStateDelay(1500,SM_ButtonsOFF2,SM_DS1307CheckACK) ' Indirect addressing
    EndIf

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(30)     ' Read the Button every 30 mS.
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd() 
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE07 [SM_AskForConfig]                                                      
'* Purpose : Fist window of the menu. Make choice: Config Time & Date or not.                                               
'* Input   : None                                                          
'* Output  : None                                                            
'* Notes   : There is a Time Out of 10 seconds by Delay1                                                        
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE07:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    StartToggleConfig()     ' Blinking LED while configuration is running.
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear()
    PrintStrg(TXT41,1,1)    ' "SET TIME/DATE S3"
    PrintStrg(TXT72,2,1)    ' "SET ALARMS    S1"
    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    Select AllButtons
        Case 2,4                        ' Answer NO => Read Time & Date 
            AllButtons = 0                     
            NextState(SM_ReadTimeDate)  ' Goto Read Time & Date.

        Case 1                               ' Answer S1 => Set Alarms.
            AllButtons = 0                    
            StartTimeOut(SM_ReadTimeDate,10) ' Start a Time Out 10 seconds 
            NextState(SM_AlarmMenu)          ' Start configuration.

        Case 3                              ' Answer S3 => Go to Get Week.
            AllButtons = 3                  ' Like the Button 3 pressed (+)
            VDayOfWeek = 0                  ' Initialise parameter.
            StartTimeOut(SM_ReadTimeDate,10)    ' Start a Time Out for 10 seconds 
            NextState(SM_GetWeek)           ' Start configuration.

        Case Else
            ' Make a loop with the SM_ButtonsOFF Module.
            StartTimeOut(SM_ReadTimeDate,10)          ' Start a Time Out for 10 seconds  
            NextIndState(SM_ButtonsOFF,CallerState)   ' Load the present state to return here.
    EndSelect 

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)             ' Delay between execution of the MAIN module (adjust the delay)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE08 [SM_ButtonsOFF]                                                       
'* Purpose : Check All Buttons are OFF                      
'* Input   : None                                                              
'* Output  : None                                                             
'* Notes   : While/Wend loop by state machine. Waiting the Buttons until OFF.       
'*           In Delay used to do these functions, the PDS programmer could write
'*           some applications running in the BackGroung using the State Machine 2.
'===============================================================================
' Define the Global variables used in this state:
'    Dim ButtonsOffCounter As Byte
' Define the Local variables used in this state:
 
' Define the Symbols used:

'===============================================================================
STATE08:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    SetBackLightTime(Max)     ' Set the Backlight to see the text on the LCD.
    ButtonsOffCounter = 0
    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    GoSub ReadButtonsPORT_Sub           ' Init & Read All Buttons.

    ' Waiting All Buttons OFF (All Buttons have a Pull-up resistor).
    If ButtonsVirtualPORT = 255 Then
        NextStateDelay(30,IncState)     ' Delay 30mS to exit the state.
    EndIf

    ' Check if the Buttons fail. Just for joke.
    GoSub CheckButtonsFail_Sub          ' If 5 Sec pull down => Buttons Fails!

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(30)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()
'===============================================================================
    SM_Return() 
'===============================================================================
'* Name    : ReadButtonsPORT_Sub                                              
'* Called From: SM_ButtonsOff, SM_ReadButtons                                     
'* Purpose : Read All Buttons                                                 
'* Input   : None                                                              
'* Output  : None                                                              
'* Notes   : 4 Buttons in PORTA & disable the Button that failed.                         
'===============================================================================  
ReadButtonsPORT_Sub:
    ANSEL = ANSEL & %11110000       ' PORTA digital for Push Buttons
    TRISA = TRISA | %00011111       ' SW0 to SW3 for input + 1Hz Timer0 for input.
ReadButtons2PORT_Sub:
    If ButtonFail > 0 Then
        SetBit AllButtonsBitsDefects,ButtonFail - 1   ' Disable the PushButton.
    EndIf
    ' All Buttons failed are saved in the AllButtonsBitsDefects buffer.
    ButtonsVirtualPORT = PORTA | AllButtonsBitsDefects ' Read SW0 to F_SW3
    Return
'===============================================================================
'* Name    : CheckButtonsFail_Sub                                              
'* Called From: SM_ButtonsOff, SM_ButtonsOFF2                                     
'* Purpose : Check If one Button fails.                                                 
'* Input   : None                                                              
'* Output  : None                                                              
'* Notes   : This routine works with the command: StartDelayMS(30) within loop delay.                         
'=============================================================================== 
CheckButtonsFail_Sub:
    Inc ButtonsOffCounter
    If ButtonsOffCounter = 167 Then             ' 167 x 30mS = 5 Seconds
        ButtonsOffCounter = 0
        If ButtonsVirtualPORT.0 = 0 Then
            ButtonFail = 1
        ElseIf ButtonsVirtualPORT.1 = 0 Then
            ButtonFail = 2
        ElseIf ButtonsVirtualPORT.2 = 0 Then
            ButtonFail = 3
        ElseIf ButtonsVirtualPORT.3 = 0 Then
            ButtonFail = 4
        EndIf
        NextState(SM_ReadTimeDate)              ' Come back to the Main Module.
    EndIf
    Return
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE09 [SM_ReadButtons]
'* Purpose : Read All Buttons in PORTA
'* Input   : None
'* Output  : None                                                             
'* Notes   :                                                                  
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE09:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
'    HRSLStrg(TXT40,1)       ' "Waiting a Button pressed!"
    KEYnOld = 0             ' Initialize parameters for debouncing.
    KEYCounter = 0
    AllButtons = 0
    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    GoSub ReadButtons2PORT_Sub                  ' Read All Buttons.

    If ButtonsVirtualPORT < 255 Then            ' If one Button pressed.
        If KEYnOld = ButtonsVirtualPORT Then    ' Some debounce system
            Inc KEYCounter
            If KEYCounter = 4 Then              ' 4 + 1 readings necessary.
                KEYCounter = 0
                '---------------------------------------------------------------
                If F_SW1 = 0 Then       ' Check Button 1.
                    AllButtons = 1
                    IncState()
                ElseIf F_SW2 = 0 Then   ' Check Button 2.
                    AllButtons = 2
                    IncState()
                ElseIf F_SW3 = 0 Then   ' Check Button 3.
                    AllButtons = 3
                    IncState()
                ElseIf F_SW4 = 0 Then   ' Check Button 4.
                    AllButtons = 4
                    IncState()
                EndIf            
                '---------------------------------------------------------------
            EndIf
        Else
            KEYCounter = 0
        EndIf
        KEYnOld = ButtonsVirtualPORT
    Else
        KEYCounter = 0
    EndIf
    '---------------------------------------------------------------------------
    ' Check the Time Out & goto the correspondent state.
    CheckTimeOut()

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(30)    ' Adjust some sample time 30mS to debounce the Buttons.
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    If F_TimeOutSystemEnable = 0 Then
        StopToggleBuzzer() ' Clear the BUZZER when alarm Timeout is finished.
        Low bAlarm1Out     ' Clear the Alarm output
    EndIf
    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 
    StateOutEnd()
'===============================================================================
    SM_Return()
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE10 [SM_ButtonsOFF2]                                                       
'* Purpose : Waiting the Buttons Until OFF.                     
'* Input   : None                                                              
'* Output  : None                                                             
'* Notes   : It is a While 1=1/Wend loop made by the State Machine. 
'*           Using the Indirect addressing [ReturnIndState()]     
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:
 
' Define the Symbols used:

'===============================================================================
STATE10:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    SMPulseOut(RB0,250) ' Start a pulse of the bBUZZER with every Button pressed.
    ButtonsOffCounter = 0
    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    GoSub ReadButtons2PORT_Sub  ' Read All Buttons.

    ' Waiting All Buttons OFF (All Buttons have a Pull-up resistor).
    If ButtonsVirtualPORT = 255 Then
        ReturnIndState(0)       ' Load the Return State defined by the caller.
    EndIf

    ' Check if the Buttons fail. Just for joke.
    GoSub CheckButtonsFail_Sub  ' If 5 Sec pull down => Buttons Fails! 

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(30)    ' Read the buttons every 30 mS.
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    
    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE11 [SM_GetWeek]                                                      
'* Purpose : Module to configurate the Day of Week.                                                
'* Input   : None                                                          
'* Output  : None                                                            
'* Notes   : (DayOfWeek, Day, Month, Year)                                                       
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE11:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    StartToggleConfig()     ' Blinking LED while configuration is running.
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear()
    PrintStrg(TXT43,1,1)    ' "Day of Week? S3+"
    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    Select AllButtons
        Case 1,3
            AllButtons = 0
            Inc VDayOfWeek
            If VDayOfWeek > 7 Then
                VDayOfWeek = 1
            EndIf
            Select VDayOfWeek        
                CLL(2)                      ' Erase the Line 2 of the LCD.
                Case 1
                    PrintStrg(TXT44,2,1)    ' "Sunday" 
                Case 2
                    PrintStrg(TXT45,2,1)    ' "Monday"
                Case 3
                    PrintStrg(TXT46,2,1)    ' "Tuesday"
                Case 4
                    PrintStrg(TXT47,2,1)    ' "Wednesday"
                Case 5
                    PrintStrg(TXT48,2,1)    ' "Thursday"
                Case 6
                    PrintStrg(TXT49,2,1)    ' "Friday"
                Case 7
                    PrintStrg(TXT50,2,1)    ' "Saturday"
            EndSelect
            NextIndState(SM_ButtonsOFF,CallerState)   ' Load the present state to return here.

        Case 2
            AllButtons = 3                  ' Like the Button 3 pressed (+)                     
            VDay = 14
            NextState(SM_GetDay)

        Case 4
            AllButtons = 0
            NextState(SM_ReadTimeDate)      ' Escape.

        Case Else
            ' Make a loop with the SM_ButtonsOFF Module.
            NextIndState(SM_ButtonsOFF,CallerState)    ' Load the present state to return here.
    EndSelect

    StartTimeOut(SM_ReadTimeDate,10)      ' Start a Time Out for 10 seconds

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE12 [SM_GetDay]                                                        
'* Purpose : Module To configurate the Day (1 to 31).                                                 
'* Input   : None                                                              
'* Output  : None                                                             
'* Notes   : (DayOfWeek, Day, Month, Year)                                                                 
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE12:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear()
    PrintStrg(TXT51,1,1)    ' "S1-   Day?   S3+"
    PrintStrg(TXT56,2,1)    ' "Day: "
    '##-END INIT-###############################################################
    StateInitEnd() 
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    If AllButtons = 1 Then          ' Button (-)
         AllButtons = 0
        Dec VDay
        If VDay = 0 Then
            VDay = 31
        EndIf
        Print Dec2 VDay
        NextIndState(SM_ButtonsOFF,CallerState) ' Load the present state to return here.

    ElseIf AllButtons = 2 Then
        AllButtons = 3              ' Like the Button 3 pressed (+)                     
        VMonth = 5
        NextState(SM_GetMonth)

    ElseIf AllButtons = 3 Then      ' Button (+)
        AllButtons = 0
        Inc VDay
        If VDay > 31 Then
            VDay = 1
        EndIf
        Print Dec2 VDay
        NextIndState(SM_ButtonsOFF,CallerState) ' Load the present state to return here.

    ElseIf AllButtons = 4 Then
        AllButtons = 0
        NextState(SM_ReadTimeDate)  ' Escape.

    Else
        ' Make a loop with the SM_ButtonsOFF Module.
        NextIndState(SM_ButtonsOFF,CallerState)    ' Load the present state to return here.  
    EndIf

    StartTimeOut(SM_ReadTimeDate,10)      ' Start a Time Out for 10 seconds

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE13 [SM_GetMonth]                                                      
'* Purpose : Module To configurate the Month (1 to 12).                                               
'* Input   : None                                                             
'* Output  : None                                                             
'* Notes   : (DayOfWeek, Day, Month, Year)                                                                   
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE13:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD
    LCD_Clear()
    PrintStrg(TXT52,1,1)    ' "S1-  Month?  S3+"
    PrintStrg(TXT57,2,1)    ' "Month: "
    '##-END INIT-###############################################################
    StateInitEnd() 
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    If AllButtons = 1 Then              ' Button (-)
        AllButtons = 0
        Dec VMonth
        If VMonth = 0 Then
            VMonth = 12
        EndIf
        Print Dec2 VMonth
        NextIndState(SM_ButtonsOFF,CallerState)    ' Load the present state to return here.

    ElseIf AllButtons = 2 Then
        AllButtons = 3                  ' Like the Button 3 pressed (+)                     
        VYear = ThisYear - 1
        NextState(SM_GetYear)

    ElseIf AllButtons = 3 Then          ' Button (+)
        AllButtons = 0
        Inc VMonth
        If VMonth > 12 Then
            VMonth = 1
        EndIf
        Print Dec2 VMonth
        NextIndState(SM_ButtonsOFF,CallerState)    ' Load the present state to return here.

    ElseIf AllButtons = 4 Then          ' Escape.
        AllButtons = 0
        NextState(SM_ReadTimeDate)

    Else
        ' Make a loop with the SM_ButtonsOFF Module.
        NextIndState(SM_ButtonsOFF,CallerState)    ' Load the present state to return here.
    EndIf

    StartTimeOut(SM_ReadTimeDate,10)      ' Start a Time Out for 10 seconds

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE14  [SM_GetYear]                                                       
'* Purpose : Module To configurate the Year.                                                
'* Input   : None                                                              
'* Output  : None                                                              
'* Notes   : (DayOfWeek, Day, Month, Year)                                                                  
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE14:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear()
    PrintStrg(TXT53,1,1)    ' "S1-  Year?   S3+"
    PrintStrg(TXT58,2,1)    ' "Year: "
    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    If AllButtons = 1 Then          ' Button (-)
        AllButtons = 0
        Dec VYear
        If VYear < ThisYear Then
            VYear = MaxYear
        EndIf
        Print Dec2 VYear
        NextIndState(SM_ButtonsOFF,CallerState) ' Load the present state to return here.

    ElseIf AllButtons = 2 Then
        AllButtons = 3              ' Like the Button 3 pressed (+)                     
        VHour = 11
        NextState(SM_GetHour)

    ElseIf AllButtons = 3 Then      ' Button (+)
        AllButtons = 0
        Inc VYear
        If VYear > MaxYear Then
            VYear = ThisYear
        EndIf
        Print Dec2 VYear
        NextIndState(SM_ButtonsOFF,CallerState) ' Load the present state to return here.

    ElseIf AllButtons = 4 Then
        AllButtons = 0
        NextState(SM_ReadTimeDate) ' Escape.

    Else
        ' Make a loop with the SM_ButtonsOFF Module.
        NextIndState(SM_ButtonsOFF,CallerState)    ' Load the present state to return here. 
    EndIf

    StartTimeOut(SM_ReadTimeDate,10)      ' Start a Time Out for 10 seconds

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    
    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE15 [SM_GetHour]                                                       
'* Purpose : Module To configurate the Hour.                                                
'* Input   : None                                                             
'* Output  : None                                                              
'* Notes   : DS1307_WriteTime(VHour, VMinute, VSecond)                                                                   
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE15:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    StartToggleConfig()     ' Blinking LED while configuration is running.
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear()
    PrintStrg(TXT54,1,1)    ' "S1-  Hour?   S3+"
    PrintStrg(TXT59,2,1)    ' "Hour: "
    '##-END INIT-###############################################################
    StateInitEnd()    
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    If AllButtons = 1 Then              ' Button (-)
        AllButtons = 0
        Dec VHour
        If VHour = 255 Then
            VHour = 23
        EndIf
        Print Dec2 VHour
        NextIndState(SM_ButtonsOFF,CallerState)  ' Load the present state to return here.

    ElseIf AllButtons = 2 Then
        AllButtons = 0
        If F_SettingAlarm2 = 1 Then     ' Programming Alarm2 now?
            VMinute = zMinute - 1       ' Value programmed yet.
        Else
            VMinute = 29                ' Standard value.
        EndIf
        AllButtons = 3                  ' Like the Button 3 pressed (+)
        NextState(SM_GetMinute)

    ElseIf AllButtons = 3 Then          ' Button (+)
        AllButtons = 0
        Inc VHour
        If VHour > 23 Then
            VHour = 0
        EndIf
        Print Dec2 VHour
        NextIndState(SM_ButtonsOFF,CallerState)    ' Load the present state to return here.

    ElseIf AllButtons = 4 Then
        AllButtons = 0          
        NextState(SM_ReadTimeDate)      ' Escape.

    Else
        ' Make a loop with the SM_ButtonsOFF Module.
        NextIndState(SM_ButtonsOFF,CallerState)    ' Load the present state to return here.
    EndIf

    StartTimeOut(SM_ReadTimeDate,10)      ' Start a Time Out for 10 seconds

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE16 [SM_GetMinute]                                                       
'* Purpose : Module To configurate the Minutes.                                                 
'* Input   : None                                                              
'* Output  : None                                                              
'* Notes   : DS1307_WriteTime(VHour, VMinute, VSecond)                                                                    
'===============================================================================
' Define the Global variables used in this state:
 
' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE16:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear()
    PrintStrg(TXT55,1,1)    ' "S1- Minute?  S3+"
    PrintStrg(TXT60,2,1)    ' "Minute: "
    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    If AllButtons = 1 Then               ' Button (-)
        Dec VMinute
        If VMinute = 255 Then
            VMinute = 59
        EndIf
        Print Dec2 VMinute
        NextIndState(SM_ButtonsOFF,CallerState)   ' Load the present state to return here.

    ElseIf AllButtons = 2 Then
        If F_SettingAlarmX = 0 Then
            NextState(SM_WriteConfig)   ' Goto Write the configuration into the DS1307 calendar.
        Else
            NextState(SM_SaveAlarm)  
            GoTo STATE16EXIT            ' Exit priority, No TimeOut.
        EndIf

    ElseIf AllButtons = 3 Then          ' Button (+)
        Inc VMinute
        If VMinute > 59 Then
            VMinute = 0
        EndIf
        Print Dec2 VMinute
        NextIndState(SM_ButtonsOFF,CallerState)   ' Load the present state to return here.

    ElseIf AllButtons = 4 Then
        NextState(SM_ReadTimeDate) ' Escape.

    Else
        ' Make a loop with the SM_ButtonsOFF Module.
        NextIndState(SM_ButtonsOFF,CallerState)    ' Load the present state to return here.
    EndIf

    StartTimeOut(SM_ReadTimeDate,10)      ' Start a Time Out for 10 seconds 
    AllButtons = 0

STATE16EXIT:
':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE17 [SM_ContrastSetup]                                                           
'* Purpose : Asking to set the contrast or not.                                                 
'* Input   : None                                                               
'* Output  : None                                                               
'* Notes   :                                                                  
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE17:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    StartToggleConfig()     ' Blinking LED while configuration is running.
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear()
    PrintStrg(TXT62,1,1)    ' "CONTRAST SETUP?"
    PrintStrg(TXT42,2,1)    ' "S2=YES  OTHER=NO"
    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    Select AllButtons
        Case 1,3,4          ' Buttons (NO)
            AllButtons = 0
            If F_MenuContrastFromReset = 1 Then
                NextState(SM_AskForConfig)  ' Exits from Reset 
            Else
                NextState(SM_ReadTimeDate)
            EndIf
            
        Case 2              ' Button 2 (YES)         
            AllButtons = 3                      ' Like Button (+) pressed.
            $ifndef _ProteusDebug_
            LCDContrast = LCD_ReadContrast()    ' Read the present contrast of the LCD driver.
            LCDContrast = LCDContrast - 1
            $else
            LCDContrast = 7
            $endif
            NextState(SM_SetContrast)

        Case Else
            ' Make a loop with the SM_ButtonsOFF Module.
            NextIndState(SM_ButtonsOFF,CallerState)    ' Load the present state to return here.
    EndSelect

    If F_MenuContrastFromReset = 1 Then
        StartTimeOut(SM_AskForConfig,10)      ' Start a Time Out for 10 seconds            
    Else
        StartTimeOut(SM_ReadTimeDate,10)      ' Start a Time Out for 10 seconds
    EndIf

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)    ' Delay between execution of the MAIN module (adjust the delay)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE18  [SM_SetContrast]                                                       
'* Purpose : Adjust the contrast value of the LCD (by software)                                                 
'* Input   : None                                                               
'* Output  : None                                                               
'* Notes   :                                                                    
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE18:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear()
    PrintStrg(TXT65,2,1)    ' "S1-  S2=> OK S3+"
    PrintStrg(TXT64,1,1,AllChars) : Print Dec2 LCDContrast   ' "Contrast = "
    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    If AllButtons = 1 Then           ' Button (-)
        If LCDContrast > 0 Then
            Dec LCDContrast
        EndIf
        $ifndef _ProteusDebug_
        LCD_WriteContrast(LCDContrast)                      ' Preset Contrast (0-15)
        $endif
        PrintStrg(TXT64,1,1)
        Print Dec2 LCDContrast   ' "CONTRAST: "
        NextIndState(SM_ButtonsOFF,CallerState)               ' Load the present state to return here.

    ElseIf AllButtons = 2 Then
        NextState(SM_SaveContrast)  ' Goto Write the contrast into the LCD.

    ElseIf AllButtons = 3 Then      ' Button (+)
        Inc LCDContrast
        If LCDContrast > 15 Then
            LCDContrast = 15
        EndIf
        $ifndef _ProteusDebug_
        LCD_WriteContrast(LCDContrast)             ' Preset Contrast (0-15)
        $endif
        PrintStrg(TXT64,1,1)
        Print Dec2 LCDContrast   ' "Contrast = "
        NextIndState(SM_ButtonsOFF,CallerState)    ' Load the present state to return here.

    ElseIf AllButtons = 4 Then                     ' Escape.
        If F_MenuContrastFromReset = 1 Then
            NextState(SM_AskForConfig)   ' Exits from Reset 
        Else
            NextState(SM_ReadTimeDate)         
        EndIf
    Else
        ' Make a loop with the SM_ButtonsOFF Module.
        NextIndState(SM_ButtonsOFF,CallerState)    ' Load the present state to return here.
    EndIf

    If F_MenuContrastFromReset = 1 Then
        StartTimeOut(SM_AskForConfig,10)      ' Start a Time Out
    Else
        StartTimeOut(SM_ReadTimeDate,10)      ' Start a Time Out         
    EndIf
    AllButtons = 0

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)    ' Delay between execution of the MAIN module (adjust the delay)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE19  [SM_SaveContrast]                                                      
'* Purpose : Save the contrast value to the eeprom memory.                                                    
'* Input   : None                                                            
'* Output  : None                                                            
'* Notes   :                                                               
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE19:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear()
    PrintStrg(TXT66,1,1)    ' " Save CONTRAST?"
    PrintStrg(TXT42,2,1)    ' "S2=YES  OTHER=NO"
    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::MAIN CODE:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    Select AllButtons
        Case 1,3,4
            CLL(2)
            PrintStrg(TXT68,2,1,AllChars)               ' "---NOT SAVED----" 
            If F_MenuContrastFromReset = 1 Then
                NextStateDelay(1500,SM_AskForConfig)    ' Exits with Delay from Reset
            Else
                NextStateDelay(1500,SM_ReadTimeDate)    ' Exits with Delay
            EndIf
   
        Case 2
            $ifndef _ProteusDebug_
            LCD_SaveContrast()
            $endif
            CLL(2)
            PrintStrg(TXT67,2,1,AllChars)               ' "Contrast Saved"        
            If F_MenuContrastFromReset = 1 Then
                NextStateDelay(1500,SM_AskForConfig)    ' Exits with Delay from Reset
            Else
                NextStateDelay(1500,SM_ReadTimeDate)    ' Exits with Delay
            EndIf

        Case Else
            ' Make a loop with the SM_ButtonsOFF Module.
            NextIndState(SM_ButtonsOFF,CallerState)    ' Load the present state to return here.
    EndSelect
               
    If F_MenuContrastFromReset = 1 Then
        StartTimeOut(SM_AskForConfig,10)      ' Start a Time Out
    Else
        StartTimeOut(SM_ReadTimeDate,10)      ' Start a Time Out         
    EndIf
    AllButtons = 0

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)
'===EXIT SECTION=================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE20  [SM_AlarmMenu]                                  
'* Purpose : Choose to configure between Alarm1 or Alarm2.                                                              
'* Input   : None                                                             
'* Output  : None                                                              
'* Notes   :                                                                  
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:
'===============================================================================
STATE20:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    StartToggleConfig()     ' Blinking LED while configuration is running.
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear()
    PrintStrg(TXT73,1,1)    ' "SETTING ALARM"
    PrintStrg(TXT74,2,1)    ' "AL1=S1    AL2=S3"
    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    Select AllButtons
        Case 1
            AllButtons = 0
            F_SettingAlarm1 = 1
            F_SettingAlarmX = 1
            LCD_Clear()
            PrintStrg(TXT73,1,1,AllChars)   ' "SETTING ALARM" 
            Print "1"
            AllButtons = 3                  ' Like the Button 3 pressed (+)
            VDayOfWeek = 0                  ' Initialise parameter.
            NextStateDelay(1500,SM_GetWeek)

        Case 3
            F_SettingAlarm2 = 1
            F_SettingAlarmX = 1
            LCD_Clear()
            PrintStrg(TXT73,1,1,AllChars)   ' "SETTING ALARM" 
            Print "2"
            If F_Alarm2Avaible = 1 Then
                RestoreE2PTimeDate2()
                VHour = zHour - 1
            Else
                VHour = 11
            EndIf
            AllButtons = 3                  ' Like the Button 3 pressed (+)
            NextStateDelay(1500,SM_GetHour) ' Goto Write the configuration into the DS1307 calendar.

        Case 2,4
            AllButtons = 0          
            NextState(SM_ReadTimeDate)  ' Escape. Goto Read Time & Date.

        Case Else
            ' Make a loop with the SM_ButtonsOFF Module.
            NextIndState(SM_ButtonsOFF,CallerState) ' Load the present state to return here.
    EndSelect
               
    StartTimeOut(SM_ReadTimeDate,10) ' Start a Time Out for 10 seconds

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)    ' Delay between execution of the MAIN module (adjust the delay)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE21  [SM_SaveAlarm]
'* Purpose : Save Time/Date parameters for alarm.                                                         
'* Input   : None                                                      
'* Output  : None                                                      
'* Notes   :                                                           
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:
'===============================================================================
STATE21:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################

    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    If F_SettingAlarm1 = 1 Then
        SaveE2PTimeDate1()
        F_SettingAlarm1 = 0
        F_SettingAlarmX = 0
        ' Only for testing.
        HRSLStrg(TXT72,9,0)   ' "SET ALARM"
        HRSOut " 1", CR,LF
        HRSLStrg(TXT20,0)   ' "Date = "
        HRSOut Dec2 VDay, "/", Dec2 VMonth, "/20", Dec2 VYear,CR,LF
        HRSLStrg(TXT21,0)   ' "Time = "            
        HRSOut Dec2 VHour, ":", Dec2 VMinute,CR,LF 

    ElseIf F_SettingAlarm2 = 1 Then
        SaveE2PTimeDate2()
        F_SettingAlarm2 = 0
        F_SettingAlarmX = 0
        ' Only for testing.
        HRSLStrg(TXT72,9,0)   ' "SET ALARM"
        HRSOut " 2", CR,LF
        HRSLStrg(TXT21,0)   ' "Time = "            
        HRSOut Dec2 VHour, ":", Dec2 VMinute,CR,LF 

    EndIf

    NextState(SM_ReadTimeDate)
       
':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)    ' Delay between execution of the MAIN module (adjust the delay)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    HRSOut CR,LF
    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================
    SM_Return() 
'*******************************************************************************
                          
'*******************************************************************************
'* Name    : STATE22  [SM_Alarm]
'* Purpose : Clear the Alarm 1 or 2.                                                         
'* Input   : None                                                      
'* Output  : None                                                      
'* Notes   : Timeout 30 seconds.                                                          
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:
'===============================================================================
STATE22:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear()
    PrintStrg(TXT69,1,1)    ' "---A L A R M----"
    PrintStrg(TXT71,2,1)    ' "  PRESS BUTTON  "
    StartToggleBuzzer()     ' Toggle the BUZZER every second
    High bAlarm1Out         ' Set an Alarm output (for relay for example).
    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    Select AllButtons
        Case 1,2,3,4
            GoSub ClearAlarms_Sub
            NextState(SM_ReadTimeDate)

        Case Else
            ' Make a loop with the SM_ButtonsOFF Module.
            StartTimeOut(SM_ReadTimeDate,30) ' Start a Time Out for 30 seconds
            NextIndState(SM_ButtonsOFF,CallerState) ' Load the present state to return here.
    EndSelect
          
':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)    ' Delay between execution of the MAIN module (adjust the delay)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
        
    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================
    SM_Return() 
'===============================================================================
'* Name    : ClearAlarms_Sub                                              
'* Called From: SM_Alarm, CheckTimeOut()                                      
'* Purpose : Reset the Alarms                                                
'* Input   : None                                                              
'* Output  : None                                                              
'* Notes   :                           
'=============================================================================== 
ClearAlarms_Sub:
    AllButtons = 0
    If F_Alarm1 = 1 Then
        F_Alarm1 = 0
        EWrite EEADR1_Alarm1Avaible,[$FF]   ' Erase the Alarm1 (Not avaible)
        F_Alarm1Avaible = 0
        F_EnableAlarm1 = 0
    EndIf
    If F_Alarm2 = 1 Then
        F_Alarm2 = 0
        F_EnableAlarm2 = 0
    EndIf
    StopToggleBuzzer()
    Low bAlarm1Out         ' Clear the Alarm output
    Return
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE23  [SM_SeeAlarm2]
'* Purpose : See the alarm2 values in the LCD.                                                         
'* Input   : None                                                      
'* Output  : None                                                      
'* Notes   :                                                           
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:
'===============================================================================
STATE23:
'===============================================================================
'    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################

    '##-END INIT-###############################################################
'    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    StartToggleConfig()     ' Blinking LED while configuration is running.
    SetBackLightTime(Max)                ' Set the Backlight 
    CheckAlarm2Avaible()
    LCD_Clear()
    If F_Alarm2Avaible = 1 Then
        RestoreE2PTimeDate2()
        PrintStrg(TXT75,1,1)            ' "DAYLY ALARM 2"
        Print At 2,1, "     ", Dec2 zHour, ":", Dec2 zMinute, "      " 
        F_EnableAlarm2 = 1
    Else
        PrintStrg(TXT75,1,1)            ' "DAYLY ALARM"
        PrintStrg(TXT76,2,1)            ' "NO ALARM"
    EndIf

    NextStateDelay(2000,SM_SeeAlarm1)
        
':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'    StartDelayMS(0)    ' Delay between execution of the MAIN module (adjust the delay)
'===============================================================================
'    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
'    StateOutEnd()    
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE24  [SM_SeeAlarm1]
'* Purpose : See the alarm1 values in the LCD.                                                       
'* Input   : None                                                      
'* Output  : None                                                      
'* Notes   :                                                           
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:
'===============================================================================
STATE24:
'===============================================================================
'    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################

    '##-END INIT-###############################################################
'    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    CheckAlarm1Avaible()
    LCD_Clear()
    If F_Alarm1Avaible = 1 Then
        RestoreE2PTimeDate1()
        Print At 1,1, "AL1 "
        Print Dec2 zDay, "/", Dec2 zMonth, "/20", Dec2 zYear
        Print At 2,2, "     ", Dec2 zHour, ":", Dec2 zMinute, "      "
    Else
        PrintStrg(TXT76,1,1,12)        ' "NO ALARM"
        Print "1"
    EndIf

    NextStateDelay(2000,SM_ReadTimeDate)
          
':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'    StartDelayMS(0)    ' Delay between execution of the MAIN module (adjust the delay)
'===============================================================================
'    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
'    StateOutEnd()    
'===============================================================================
    SM_Return() 
'*******************************************************************************
                                 
'*******************************************************************************
'* Name    : STATE25  [SM_DisableAlarms]
'* Purpose : Disable the Alarms.                                                         
'* Input   : None                                                      
'* Output  : None                                                      
'* Notes   :                                                           
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:
'===============================================================================
STATE25:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    StartToggleConfig()     ' Blinking LED while configuration is running.
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear()
    PrintStrg(TXT36,1,1)    ' "DISABLE   ALARMS"
    PrintStrg(TXT74,2,1)    ' "AL1=S1    AL2=S3"
    '##-END INIT-###############################################################
    StateInitEnd()
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    Select AllButtons
        Case 1
            EWrite EEADR1_Alarm1Avaible,[$FF]   ' Disable the Alarm1 (Not avaible)
            F_Alarm1Avaible = 0
            F_EnableAlarm1 = 0
            F_Alarm1 = 0
            PrintStrg(TXT37,2,1)                ' "ALARM1  DISABLED"
            NextStateDelay(1500,SM_ReadTimeDate)

        Case 3
            F_EnableAlarm2 = 0
            F_Alarm2 = 0
            PrintStrg(TXT38,2,1)                ' "ALARM2  DISABLED"
            NextStateDelay(1500,SM_ReadTimeDate)

        Case 2,4
            NextState(SM_ReadTimeDate)          ' Escape. Goto Read Time & Date.

        Case Else
            ' Make a loop with the SM_ButtonsOFF Module.
            NextIndState(SM_ButtonsOFF,CallerState) ' Load the present state to return here.
    EndSelect
               
    StartTimeOut(SM_ReadTimeDate,10) ' Start a Time Out for 10 seconds
    AllButtons = 0
          
':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)    ' Delay between execution of the MAIN module (adjust the delay)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE26  (SM_TC74CheckACK)                                                        
'* Purpose : Check if the TC74 is present on the I2C bus.                               
'* Input   : None                                                             
'* Output  : None                                                            
'* Notes   : The ReturnACK must to be = 0.                                                                 
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
STATE26:
'===============================================================================
    StateInit() 
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    HRSLStrg(TXT81,1)       ' "Check the TC74 ackknowledge."
    '##-END INIT-###############################################################
    StateInitEnd(SyncOn)    ' Module synchronized with timing 1mS.
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    ReturnACK = TC74_ReadStatus(TC74A4) ' TC74 sensor of the LCD shield.
    If ReturnACK = 1 Then
        HRSLStrg(TXT83,1)               ' "NACK TC74 #03 temperature Device!"
        NextState(SM_TC74Error_NACK)
    Else
        HRSLStrg(TXT82,1)               ' "The TC74 Device is ready!"
        F_ErrorNACK_TC74 = 0            ' Reset the DS1307 error.
        NextState(SM_Enable1Hz)
    EndIf  

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(0)    ' Delay between execution of the MAIN module (adjust the delay)
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    HRSOut CR,LF
    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd()    
'===============================================================================    
    SM_Return()
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE27 (SM_TC74Error_NACK)                                                          
'* Purpose : Press any Button To try the communication again.  
'* Input   : None                                                              
'* Output  : None                                                              
'* Notes   :                                                                  
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used :

'===============================================================================
STATE27:
'===============================================================================
    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################
    HRSLStrg(TXT22,1)       ' "Warning: Communication error! ACK no valid."
    HRSLStrg(TXT25,1)       ' "Press the Button 2 to ReStart the programm."
    SetBackLightTime(Max)   ' Set the Backlight to see the text on the LCD.
    LCD_Clear() 
    PrintStrg(TXT83,1,1,9)  ' "NACK TC74"
    PrintStrg(TXT27,2,1)    ' "RESTART any SW"
    SMPulseOut(RB0,250)       ' Start a pulse of the bBUZZER
    '##-END INIT-###############################################################
    StateInitEnd()    
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    GoSub ReadButtons2PORT_Sub                      ' Read All Buttons.

    ' If any Button pressed... in case of a Button fails...
    If F_SW1 = 0 Or F_SW2 = 0 Or F_SW3 = 0 Or F_SW4 = 0 Then
        CLL(2)
        PrintStrg(TXT18,2,1)    ' "TRY DSTC74 AGAIN"
        NextIndStateDelay(1500,SM_ButtonsOFF2,SM_TC74CheckACK) ' Indirect addressing
    EndIf

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    StartDelayMS(30)     ' Read the Button every 30 mS.
'===============================================================================
    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    StateOutEnd() 
'===============================================================================
    SM_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : STATE28 []                                                          
'* Purpose : Run code every minute  
'* Input   : None                                                              
'* Output  : None                                                              
'* Notes   : Separate the module 5 in two parts to run the priority task better.                                                                 
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used :

'===============================================================================
'STATE28:
'===============================================================================
'    StateInit()
    ' Transitional Input State.
    '##-INIT SECTION-###########################################################

    '##-END INIT-###############################################################
'    StateInitEnd()   ' (SyncOn) Module synchronized with timing 1mS.    
    ' Static State.
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'    StartDelayMS(30)     ' Read the Button every 30 mS.
'===============================================================================
'    StateOut()
    'Transitional Output State.
    '$$$-EXIT SECTION-$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
'    StateOutEnd() 
'===============================================================================
'    SM_Return() 
'*******************************************************************************




'###############################################################################
'#### STATE MACHINE 2 IN #######################################################
$ifdef _StatesMachine2_
'    Include "D:\MyIncFiles\SM2_Scheduler01.Inc."
    Include "INC\SM2_Scheduler01.Inc"
'===============================================================================
    ' PDS user SM2 code.
    Include "STM07_SM2Code.bas"
$endif
'#### STATE MACHINE 2 END ######################################################
'###############################################################################
' Some Subroutines:
'*******************************************************************************
'* Name    : RunPulsePinCode_Sub                                                         
'* Purpose : Clear a PORT Pin after a delay.  
'* Input   : none                                                              
'* Output  : Write the available Output pins of the project only.                                                               
'* Notes   : The pins to clear must be written for every project.
'*           It is not a generic code.                                                                 
'===============================================================================
$ifdef _EnablePulsePin_
RunPulsePinCode_Sub:
    Select PortPin      ' Byte variable with the number of the Pin.
        Case _RB0
            Low LATB.0  ' BUZZER
        Case _RB1
            Low LATB.1  ' BACKLIGHT           
        Case _RC0
            Low LATC.0  ' bTest1
        Case _RC1
            Low LATC.1  ' bTest2
        Case _RC2
            Low LATC.2  ' bTest3
        Case _RC5
            Low LATC.5  ' ALARM OUTPUT  
    EndSelect
    F_StartPortPinPulse = 0
    Return
$endif
                                                                
'===============================================================================
  
'###############################################################################

'*******************************************************************************
'* Name    : STM_Strings07.Inc                                             
'* Purpose : CData Table for Strings or Datas                             
'* Notes   : Write the Tables at top of memory syncronized with Erase Block   
'*         : The Cdata table could have 1600 Bytyes more or less.                          
'*         : Table of Strings of the program                                 
'*******************************************************************************
'    ORG_EBlock(_MAXMEM - 1600)
    Include "D:\MyIncFiles\STM_Strings07.Inc"
'    Include "INC\STM_Strings07.Inc"    
'*******************************************************************************
' Write some static data to the ID Locations.
' Version of the programm: STM07V10
Asm
__IDLOCS   __IDLOC0,0x53
__IDLOCS   __IDLOC1,0x54
__IDLOCS   __IDLOC2,0x4D
__IDLOCS   __IDLOC3,0x30
__IDLOCS   __IDLOC4,0x37
__IDLOCS   __IDLOC5,0x56
__IDLOCS   __IDLOC6,0x31
__IDLOCS   __IDLOC7,0x30
EndAsm
'###############################################################################
'*** END OF FILE ***************************************************************
'*******************************************************************************

                
