'****************************************************************
'*  Name    : STM07_SM2Code.bas                                 *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2017 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 05/06/2017                                        *
'*  Version : 1.0                                               *
'*  Notes   :                                                   *
'*          :                                                   *
'****************************************************************

$ifndef _SM2Codebas_
$define _SM2Codebas_
'*******************************************************************************
'* Name    : SM2_STATE00 [SM2_ReadTime]        
'* Purpose : Read the Time from the DS1307 every second.                                                     
'* Input   : None                                                      
'* Output  : None                                                        
'* Notes   : DO job every second.                                                       
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
SM2_STATE00:
'===============================================================================
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    ' Do every Second.
    F_ErrorNACK_DS1307 = 0
    '---------------------------------------------------------------------------
    DS1307_ReadTime(ReturnACK)      ' Read the time from the DS1307
    '---------------------------------------------------------------------------
    If ReturnACK = 1 Then
        HRSOut "#04",CR,LF
        F_ErrorNACK_DS1307 = 1    ' Send an error flag to the main code.
    Else
        SM2_LoadTask(SM2_PrintLCDLine2)
    EndIf
    '---------------------------------------------------------------------------

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'===============================================================================
    SM2_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : SM2_STATE01 [SM2_PrintLCDLine1]        
'* Purpose : Print strings to the LCD in Background.                                                     
'* Input   : None                                                      
'* Output  : None                                                        
'* Notes   : DO job every second.                                                            
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
SM2_STATE01:
'===============================================================================
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    ' PRINT the Results to the LCD by the State Machine2 in BackGround.
    '===========================================================================
    ' DO every Second.    
    ' Print the Date to the LCD.
    If ButtonFail > 0 Then                      ' Printing the last button failed.
        If ButtonFail = 1 Then
            PrintStrg(TXT77,1,1)                ' "Button 1 FAILS"
        ElseIf ButtonFail = 2 Then
            PrintStrg(TXT78,1,1)                ' "Button 2 FAILS"
        ElseIf ButtonFail = 3 Then
            PrintStrg(TXT79,1,1)                ' "Button 3 FAILS"
        ElseIf ButtonFail = 4 Then
            PrintStrg(TXT80,1,1)                ' "Button 4 FAILS"
        EndIf
    Else
        ' DO every Second.
        Print At 1,1, Dec2 RTC_Day, "/", Dec2 RTC_Month, "/20", Dec2 RTC_Year, "      "
        '---------------------------------------------------------------------------
        ' DO every Second.       
        ' Check Alarm enable to print "ALARM" in the right side of LCD.
        Cursor 1,12
        If F_EnableAlarm1 = 1 Or F_EnableAlarm2 = 1 Then
            PrintStrg(TXT70,1,12,5)    ' "ALARM"
        Else
            '-------------------------------------------------------------------
            If Temperature = 99 Then    ' temperature value not valid.
                ' The fisrt 30 seconds after reset 7 characters of the RTC_DayName are printed. 
                Print Str RTC_MonthName\5  ' Print the Month Name To the LCD.
            Else
                ' When the temperature is valid then print it.
                If ReturnPolarity = 1 Then
                    Print "-", Dec2 Temperature,223,"C"
                Else
                    Print "+", Dec2 Temperature,223,"C"    
                EndIf
            EndIf                 
            '-------------------------------------------------------------------
        EndIf
    EndIf
    '---------------------------------------------------------------------------

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'===============================================================================
    SM2_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : SM2_STATE02 [SM2_PrintLCDLine2]        
'* Purpose : Task to print Time and ADC result.                                                      
'* Input   : None                                                      
'* Output  : None                                                        
'* Notes   : DO job every second.
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
SM2_STATE02:
'===============================================================================
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    ' DO every Second.
    ' Print the Time to the LCD.
    Cursor 2,1
    'Print Dec2 RTC_Hour,":", Dec2 RTC_Minute, ":", Dec2 RTC_Second, " ", Str RTC_DayName\7
    Print Dec2 RTC_Hour,"H", Dec2 RTC_Minute, ":", Dec2 RTC_Second, "        "
    '---------------------------------------------------------------------------
    ' Print the ADC results to the LCD.
    Cursor 2,10
    If F_PrintADCResult = 1 Then    ' print the ADC result if ADC is ready.
        F_PrintADCResult = 0
        Print "+", Dec ADC_IntegerResult,".",Dec3 ADC_DecimalResult, "V" 
    Else
        Print Str RTC_DayName\7  ' Print the Month Name To the LCD.
    EndIf    
    '---------------------------------------------------------------------------

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'===============================================================================
    SM2_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : SM2_STATE03 [SM2_CheckAlarmState]        
'* Purpose : Check the Alarms every Minute.                                                      
'* Input   : None                                                      
'* Output  : None                                                        
'* Notes   : DO job every minute.                                                          
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
SM2_STATE03:
'===============================================================================
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    ' Do every Minute.  Check the alarms clock
    ' Checking Alarm1
    If F_Alarm1Avaible = 1 Then
        If F_EnableAlarm1 = 1 Then
            RestoreE2PTimeDate1()
            If RTC_Minute = zMinute Then
                If RTC_Hour = zHour Then
                    If RTC_Day = zDay Then
                        If RTC_Month = zMonth Then
                            If RTC_Year = zYear Then
                                F_Alarm1 = 1    ' Set an Alarm1.
                                F_AlarmX = 1    ' Set an general Alarm.
                            EndIf
                        EndIf
                    EndIf
                EndIf
            EndIf
        EndIf
    EndIf
    ' Checking Alarm2 (Dayly)
    If F_Alarm2Avaible = 1 Then
        If F_EnableAlarm2 = 1 Then
            RestoreE2PTimeDate2()
            If RTC_Minute = zMinute Then
                If RTC_Hour = zHour Then
                    F_Alarm2 = 1    ' Set an Alarm2.
                    F_AlarmX = 1    ' Set an general Alarm.
                EndIf
            EndIf
        EndIf
    EndIf

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'===============================================================================
    SM2_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : SM2_STATE04 [SM2_ADCaverage]        
'* Purpose : Calculations of the result of the default ADC channel.                                                      
'* Notes   : The ADC is not closed.
'*         : Using integer variables only.
'*         : The AN4 ADC channel is open in PORTA.5.
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
SM2_STATE04:
'===============================================================================
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    '---------------------------------------------------------------------------
    ' Reconfigure the PORTA.     (not necessary with the modifications in the Amicus_ADC.inc)
'    ANSEL = %11110000           ' PORTA digital for Push Buttons
'    TRISA = TRISA | %00111111   ' Push Buttons for input + Digital 1Hz input.
    '---------------------------------------------------------------------------
    ' Check the validity of the ADC result.
    $ifdef _CheckADCSystem_
    Inc ADCCounterOld
    If ADCCounter <> ADCCounterOld Then
        ' In the 1st error printed, you find the state that made the error.
        $ifdef _EnableTerminal_
        HRSLStrg(TXT2,1)    ' "ADC ERROR!"
        HRSLStrg(TXT91,0)   ' "SM2_STATE"
        HRSOut Dec2 SM2StateIndexOld,CR,LF
        $endif
'        F_ADC_Error = 1
        ADCCounter = 0
        ADCCounterOld = 0
        SM2_Return()            ' Don't calculate/print the erroneous value. (more faster) 
        'GoTo SM2_STATE04EXIT    ' Don't calculate/print the erroneous value.
    EndIf
    ADCCounterOld = ADCCounter
    $endif    
'-------------------------------------------------------------------------------
' WITH AVERAGES
    $ifdef NumberADCaverages
    '---------------------------------------------------------------------------
    ' Average for N samples.
    ' Must be a power of 2 number like 2,4,8,16,32,64 max.
    ' It is faster because the circular buffer works more eficient. ADC_ResultW variable from ADC library.
    ' Loading the every sample in the Word Array
    ADC_TrueAverage[TrueAverageHead] = ADC_ResultW  ' Load the ADC value in the circular buffer.
    Inc TrueAverageHead
    TrueAverageHead = TrueAverageHead & (NumberADCaverages - 1) ' Circular pointer. 
    '---------------------------------------------------------------------------    
    ' Check if the Buffer is full, otherwize do not calculate averages.
    If F_AverageReady = 0 Then
        If TrueAverageHead = 0 Then
            F_AverageReady = 1
        Else
            F_PrintADCResult = 0
            SM2_Return()    ' Don't calculate/print because there not enough values. 
        EndIf
    EndIf
    '---------------------------------------------------------------------------
    ' Calculate the average for every sample.
    AverageLoop = 0
    ADCToPrint = 0
    Repeat
        ADC_Average = ADC_TrueAverage[AverageLoop]
        ADCToPrint = ADCToPrint + ADC_Average
        Inc AverageLoop
    Until AverageLoop = NumberADCaverages   
'===============================================================================
'    ADCToPrint = ADCToPrint / NumberADCaverages
'    ADCResult = (ADCToPrint * 5000) / 1023
'    ADC_IntegerResult = ADCResult / 1000
'    ADC_DecimalResult = ADCResult // 1000
'-------------------------------------------------------------------------------
' Calculations for +5V power supply.
' More eficient doing this calculation. NumberADCaverages = 64 max
'    ADC_DecimalResult = 1023 * NumberADCaverages    ' Result = Word variable
''    ADCResult = (ADCToPrint * 5000) / ADC_DecimalResult 
''    ADC_IntegerResult = ADCResult / 1000
''    ADC_DecimalResult = ADCResult // 1000
'-------------------------------------------------------------------------------
' Calculations for +3,3V power supply for PIC +3,3V.
    ADC_DecimalResult = 1023 * NumberADCaverages    ' Result = Word variable
''    ADCResult = (ADCToPrint * 3300) / ADC_DecimalResult 
''    ADC_IntegerResult = ADCResult / 1000
''    ADC_DecimalResult = ADCResult // 1000
'-------------------------------------------------------------------------------

' Using the asm instead of basic to delete some lines of code.
' Some lines could be deleted because repetition.
' Maths32.
Asm
;F13_000788 equ $ ; IN [STM07_SM2CODE.BAS] ADCRESULT = (ADCTOPRINT * 5000) / ADC_DECIMALRESULT
;F13_000788 equ $ ; IN [STM07_SM2CODE.BAS] ADCRESULT = (ADCTOPRINT * 3300) / ADC_DECIMALRESULT
    Movff ADCToPrint,PP0
    Movff ADCToPrintH,PP0H
    Clrf PP0HH,0
    Clrf PP0HHH,0
    Clrf PP2HHH,0
    Clrf PP2HH,0
    ;---------------------------------------------------------------------------
    ; For 3.3V = +VDD
    $ifdef _VDD33V_
    Movlw 12
    Movwf PP2H,0
    Movlw 228
    Movwf PP2,0
    $else
    ; For 5.0V = +VDD
    Movlw 19
    Movwf PP2H,0
    Movlw 136
    Movwf PP2,0
    $endif
    ;---------------------------------------------------------------------------
    Call __MULTIPLY_U3232_
;    movff PP0HHH,PP7HHH
;    movff PP0HH,PP7HH
;    movff PP0H,PP7H
;    movff PP0,PP7
;    movff PP7HHH,PP0HHH
;    movff PP7HH,PP0HH
;    movff PP7H,PP0H
;    movff PP7,PP0
    Movff ADC_DecimalResult,PP2
    Movff ADC_DecimalResultH,PP2H
    Clrf PP2HH,0
    Clrf PP2HHH,0
    Call __DIVIDE_U3232_
;    movff PP0HHH,ADCResultHHH
;    movff PP0HH,ADCResultHH
;    movff PP0H,ADCResultH
;    movff PP0,ADCResult
;-------------------------------------------------------------------------------
;F13_000789 equ $ ; IN [STM07_SM2CODE.BAS] ADC_INTEGERRESULT = ADCRESULT / 1000
;    movff ADCResultHHH,PP0HHH
;    movff ADCResultHH,PP0HH
;    movff ADCResultH,PP0H
;    movff ADCResult,PP0
    Clrf PP2HHH,0
    Clrf PP2HH,0
    Movlw 3
    Movwf PP2H,0
    Movlw 232
    Movwf PP2,0
    Call __DIVIDE_U3232_
    Movff PP0,ADC_IntegerResult
;-------------------------------------------------------------------------------
;F13_000790 equ $ ; IN [STM07_SM2CODE.BAS] ADC_DECIMALRESULT = ADCRESULT // 1000
;    movff ADCResultHHH,PP0HHH
;    movff ADCResultHH,PP0HH
;    movff ADCResultH,PP0H
;    movff ADCResult,PP0
;    clrf PP2HHH,0
;    clrf PP2HH,0
;    movlw 3
;    movwf PP2H,0
;    movlw 232
;    movwf PP2,0
;    call __DIVIDE_U3232_
    Movff PPZH,ADC_DecimalResultH
    Movff PPZ,ADC_DecimalResult
EndAsm
'-------------------------------------------------------------------------------
$else
' NO AVERAGE
'-------------------------------------------------------------------------------
' Calculations for +5,0V power supply for PIC +5,0V.
'    ADCResult = (ADC_ResultW * 5000) / 1023
'    ADC_IntegerResult = ADCResult / 1000
'    ADC_DecimalResult = ADCResult // 1000
'-------------------------------------------------------------------------------
' Calculations for +3,3V power supply for PIC +3,3V.
''    ADCResult = (ADCToPrint * 3300) / ADC_DecimalResult 
''    ADC_IntegerResult = ADCResult / 1000
''    ADC_DecimalResult = ADCResult // 1000
'-------------------------------------------------------------------------------
' Using the asm instead of basic to delete some lines of code.
' Some lines could be deleted because repetition.
' Maths32.
Asm
;F1_001215 equ $ ; IN [STM07.BAS] ADCRESULT = (ADC_RESULTW * 5000) / 1023
;F13_000788 equ $ ; IN [STM07_SM2CODE.BAS] ADCRESULT = (ADCTOPRINT * 3300) / ADC_DECIMALRESULT
    Movff ADRESL,PP0
    Movff ADRESH,PP0H
    Clrf PP0HH,0
    Clrf PP0HHH,0
    Clrf PP2HHH,0
    Clrf PP2HH,0
    ;---------------------------------------------------------------------------
    ; For 3.3V = +VDD
    $ifdef _VDD33V_
    Movlw 12
    Movwf PP2H,0
    Movlw 228
    Movwf PP2,0
    $else
    ; For 5.0V = +VDD
    Movlw 19
    Movwf PP2H,0
    Movlw 136
    Movwf PP2,0
    $endif
    ;---------------------------------------------------------------------------
    Call __MULTIPLY_U3232_
;    movff PP0HHH,PP7HHH
;    movff PP0HH,PP7HH
;    movff PP0H,PP7H
;    movff PP0,PP7
;    movff PP7HHH,PP0HHH
;    movff PP7HH,PP0HH
;    movff PP7H,PP0H
;    movff PP7,PP0
    Clrf PP2HHH,0
    Clrf PP2HH,0
    Movlw 3
    Movwf PP2H,0
    Setf PP2,0
    Call __DIVIDE_U3232_
;    movff PP0HHH,ADCResultHHH
;    movff PP0HH,ADCResultHH
;    movff PP0H,ADCResultH
;    movff PP0,ADCResult
;F1_001216 equ $ ; IN [STM07.BAS] ADC_INTEGERRESULT = ADCRESULT / 1000
;    movff ADCResultHHH,PP0HHH
;    movff ADCResultHH,PP0HH
;    movff ADCResultH,PP0H
;    movff ADCResult,PP0
    Clrf PP2HHH,0
    Clrf PP2HH,0
    Movlw 3
    Movwf PP2H,0
    Movlw 232
    Movwf PP2,0
    Call __DIVIDE_U3232_
    Movff PP0,ADC_IntegerResult
;F1_001217 equ $ ; IN [STM07.BAS] ADC_DECIMALRESULT = ADCRESULT // 1000
;    movff ADCResultHHH,PP0HHH
;    movff ADCResultHH,PP0HH
;    movff ADCResultH,PP0H
;    movff ADCResult,PP0
;    clrf PP2HHH,0
;    clrf PP2HH,0
;    movlw 3
;    movwf PP2H,0
;    movlw 232
;    movwf PP2,0
;    call __DIVIDE_U3232_
    Movff PPZH,ADC_DecimalResultH
    Movff PPZ,ADC_DecimalResult
EndAsm
'-------------------------------------------------------------------------------
$endif
    '---------------------------------------------------------------------------
    F_PrintADCResult = 1    ' The ADC Results are ready to print.
    '---------------------------------------------------------------------------
    GoSub ConfigureADC_Sub
    '---------------------------------------------------------------------------
SM2_STATE04EXIT:
    $ifdef _PinsTest_
    Clear bTest2
    $endif

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'===============================================================================
    SM2_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : SM2_STATE05 [SM2_TC74SetNormalMode]        
'* Purpose : Set the TC74 to Normal Mode.                                                      
'* Input   : None                                                      
'* Output  : None                                                        
'* Notes   : The delay to read the sensor is 240 mS.                                                          
'*           The TC74 library has been modified. The delay is written here.
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
SM2_STATE05:
'===============================================================================
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    F_ErrorNACK_TC74 = 0
    TC74_NormalMode(TC74A4, ReturnACK)
    '---------------------------------------------------------------------------
    If ReturnACK = 1 Then
        HRSOut "#05",CR,LF
        F_ErrorNACK_TC74 = 1    ' Send an error flag to the main code.
    Else
        ' The sensor need a 250 mS virtual delay to run after a Normal Mode command.
        SM2_NextStateDelay(250,SM2_TC74Read)
    EndIf  

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'===============================================================================
    SM2_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : SM2_STATE06 [SM2_TC74Read]        
'* Purpose : Configure the sensor to normal mode.                                                      
'* Input   : None                                                      
'* Output  : None                                                        
'* Notes   : TC74 sensor of the LCD shield.                                                         
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
SM2_STATE06:
'===============================================================================
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    ' Read the temperature sensor.
    F_ErrorNACK_TC74 = 0
    Temperature = TC74_ReadSRTemp(TC74A4, ReturnPolarity, ReturnACK) 
    '---------------------------------------------------------------------------
    ' Check the NACK
    If ReturnACK.0 = 1 Then
        HRSOut "#06",CR,LF
        F_ErrorNACK_TC74 = 1    ' Send an error flag to the main code.
    Else
        ' Check the Ready Mode.
        If ReturnACK.6 = 0 Then
            HRSLStrg(TXT84,1)       ' "#07 The sensor is NOT ready"
            Temperature = 99
            F_ErrorNACK_TC74 = 1    ' Send an error flag to the main code.
            GoTo SM2_STATE06EXIT
        Else
            SM2_LoadTask(SM2_TC74Close)   ' Goto Close the sensor.
        EndIf
    EndIf
    '---------------------------------------------------------------------------
    'Temperature Correction
    $ifdef TemperatureCorrection
    If ReturnPolarity = 1 Then
        If TempPolarityCorrection = 1 Then
            Temperature = Temperature + TemperatureCorrection
        Else
            If Temperature > TemperatureCorrection Then
                Temperature = Temperature - TemperatureCorrection
            Else
                Temperature = TemperatureCorrection - Temperature
                ReturnPolarity = 0
            EndIf
        EndIf
    Else
        If TempPolarityCorrection = 1 Then
            If Temperature > TemperatureCorrection Then
                Temperature = Temperature - TemperatureCorrection
            Else
                Temperature = TemperatureCorrection - Temperature
                ReturnPolarity = 1
            EndIf
        Else
            Temperature = Temperature + TemperatureCorrection
        EndIf    
    EndIf        
    If Temperature = 0 Then
        ReturnPolarity = 0
    EndIf 
    $endif    

SM2_STATE06EXIT:
':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'===============================================================================
    SM2_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : SM2_STATE07 [SM2_TC74Close]        
'* Purpose : Close the TC74 temperature sensor to avoid self heating, because SMD chip.                                                      
'* Input   : None                                                      
'* Output  : None                                                        
'* Notes   :                                                          
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
SM2_STATE07:
'===============================================================================
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    ' Because self heating, close the sensor.
    F_ErrorNACK_TC74 = 0
    TC74_StandbyMode(TC74A4, ReturnACK)
    '---------------------------------------------------------------------------
    If ReturnACK = 1 Then
        HRSOut "#08",CR,LF
        F_ErrorNACK_TC74 = 1    ' Send an error flag to the main code.
    Else
        SM2_LoadTask(SM2_TC74TempToTerminal)  ' Print the result to the Terminal.
        SM2_LoadTask(SM2_PrintLCDLine1)       ' Print the result to the LCD.
    EndIf  

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'===============================================================================
    SM2_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : SM2_STATE08 [SM2_ReadDate]        
'* Purpose : Read the Date of the DS1307 clock calendar.                                                      
'* Input   : None                                                      
'* Output  : None                                                        
'* Notes   : Read informations for the DS1307 library. Do Job every minute.                                                        
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
SM2_STATE08:
'===============================================================================
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    F_ErrorNACK_DS1307 = 0
    DS1307_ReadDate(ReturnACK)    ' Read the date from the DS1307
    '---------------------------------------------------------------------------
    If ReturnACK = 1 Then
        HRSOut "#09",CR,LF
        F_ErrorNACK_DS1307 = 1    ' Send an error flag to the main code.
    Else
        ' Get the day's name into string RTC_DayName from DS1307 library.
        DS1307_Get_DayName(RTC_DayOfWeek, RTC_DayName) 
        ' Get the months's name into string RTC_MonthName from DS1307 library.
        DS1307_Get_MonthName(RTC_Month, RTC_MonthName) 
        RTC_MinuteOld = RTC_Minute      ' Save the new minute value.
        '-----------------------------------------------------------------------
        SM2_LoadTask(SM2_PrintLCDLine1)
        '-----------------------------------------------------------------------
    EndIf

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'===============================================================================
    SM2_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : SM2_STATE09 [SM2_Data1ToTerminal]        
'* Purpose : Send the 1st mensage To the Terminal every minute.                                                      
'* Input   : None                                                      
'* Output  : None                                                        
'* Notes   : This string is divided in 3 parts to solve ADC timing of 4mS (250Hz)                                                          
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
SM2_STATE09:
'===============================================================================
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    $if _xtal = 80
    '---------------------------------------------------------------------------
    ' Do every Minute.
    ' FOSC = 80 Mhz the UART is configured to 230400 Bauds and runs 4 times more faster than the Proteus.
    '---------------------------------------------------------------------------
    ' Send Date to the terminal.
    HRSOut RTC_DayName, " ", Dec2 RTC_Day, "/", Dec2 RTC_Month, "/20", Dec2 RTC_Year,_
    " ",Dec2 RTC_Hour, ":", Dec2 RTC_Minute, ":", Dec2 RTC_Second,CR,LF
    SM2_LoadTask(SM2_Data2ToTerminal)
    '---------------------------------------------------------------------------
    $else
    '---------------------------------------------------------------------------
    ' Do every Minute.
    ' FOSC = 16 Mhz the UART is configured to 57600 Bauds for Proteus.
    ' FOSC = 64 Mhz the UART is configured to 115200 Bauds for Amicus Board.    
    If CountStrings = 0 Then
        HRSOut RTC_DayName          ' Send the 1rst string
        Inc CountStrings
        SM2_LoadTask(SM2_Data1ToTerminal)   ' Reload the same task.
    ElseIf CountStrings = 1 Then
        HRSOut " ", Dec2 RTC_Day, "/", Dec2 RTC_Month, "/20", Dec2 RTC_Year, " " ' Send the 2nd string
        Inc CountStrings
        SM2_LoadTask(SM2_Data1ToTerminal)   ' Reload the same task.
    ElseIf CountStrings = 2 Then
        HRSOut Dec2 RTC_Hour, ":", Dec2 RTC_Minute, ":", Dec2 RTC_Second,CR,LF ' Send the 3th string
        CountStrings = 0                    ' Task closed.
        SM2_LoadTask(SM2_Data2ToTerminal)
    EndIf
    '---------------------------------------------------------------------------
    $endif

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'===============================================================================
    SM2_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : SM2_STATE10 [SM2_Data2ToTerminal]        
'* Purpose : Send a message To the Terminal every minute.                                                     
'* Input   : None                                                      
'* Output  : None                                                        
'* Notes   : With Proteus the UART Baudrate is too slow.                                                        
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
SM2_STATE10:
'===============================================================================
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    ' Do every Minute.
    ' Send ADC value to the terminal.
    '---------------------------------------------------------------------------
    $if _xtal = 80
    ' FOSC = 80 Mhz the UART is configured to 230400 Bauds and runs 4 times more faster than the Proteus.
    '---------------------------------------------------------------------------
    HRSOut "ADC Result: +", Dec ADC_IntegerResult,".", Dec3 ADC_DecimalResult, "V",CR,LF 
    '---------------------------------------------------------------------------
    $else
    '---------------------------------------------------------------------------
    ' FOSC = 16 Mhz the UART is configured to 57600 Bauds for Proteus.
    ' FOSC = 64 Mhz the UART is configured to 115200 Bauds for Amicus Board. 
    If CountStrings = 0 Then
        HRSOut "ADC Result: +"     
        Inc CountStrings
        SM2_LoadTask(SM2_Data2ToTerminal)
    ElseIf CountStrings = 1 Then
        HRSOut Dec ADC_IntegerResult, ".", Dec3 ADC_DecimalResult, "V",CR,LF
        CountStrings = 0        ' End of the Task.
    EndIf                   
    '---------------------------------------------------------------------------
    $endif

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'===============================================================================
    SM2_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : SM2_STATE11[SM2_TC74TempToTerminal]        
'* Purpose : Send the Temperature message To the Terminal every minute.                                                     
'* Input   : None                                                      
'* Output  : None                                                        
'* Notes   : This String is divided in 3 parts To solve ADC timing of 4mS (250Hz)                                                        
'*         : Reload the task with the command: SM2_LoadTask(SM2_TC74TempToTerminal
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
SM2_STATE11:
'===============================================================================
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    ' Do every Minute + 30 seconds.
    ' Send the Polarity and the Temperature result to the terminal.
    '---------------------------------------------------------------------------
    $if _xtal = 80
    ' FOSC = 80 Mhz the UART is configured to 230400 Bauds and runs 4 times more faster than the Proteus.
    If ReturnPolarity = 1 Then          
        HRSLStrg(TXT85,0)       ' "Temperature: "
        HRSOut "-", Dec Temperature
        HRSLStrg(TXT86,2)       ' "Deg C"
    Else                                    
        HRSLStrg(TXT85,0)       ' "Temperature: "
        HRSOut "+", Dec Temperature
        HRSLStrg(TXT86,2)       ' "Deg C"
    EndIf
    $else
    ' FOSC = 16 Mhz the UART is configured to 57600 Bauds for Proteus.
    ' FOSC = 64 Mhz the UART is configured to 115200 Bauds for Amicus Board. 
    If ReturnPolarity = 1 Then          
        If CountStrings = 0 Then
            HRSLStrg(TXT85,0)       ' "Temperature:"
            Inc CountStrings
            SM2_LoadTask(SM2_TC74TempToTerminal)
        ElseIf CountStrings = 1 Then
            HRSOut "-", Dec Temperature
            HRSLStrg(TXT86,2)       ' "Deg C"
            CountStrings = 0        ' End of the Task. 
        EndIf
    Else                                    
        If CountStrings = 0 Then
            HRSLStrg(TXT85,0)       ' "Temperature:"
            Inc CountStrings
            SM2_LoadTask(SM2_TC74TempToTerminal) 
        ElseIf CountStrings = 1 Then
            HRSOut "+", Dec Temperature
            HRSLStrg(TXT86,2)       ' "Deg C"
            CountStrings = 0        ' End of the Task.
        EndIf
    EndIf
    $endif

':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'===============================================================================
    SM2_Return() 
'*******************************************************************************

'*******************************************************************************
'* Name    : SM2_STATE12 []        
'* Purpose :                                                      
'* Input   : None                                                      
'* Output  : None                                                        
'* Notes   :                                                       
'===============================================================================
' Define the Global variables used in this state:

' Define the Local variables used in this state:

' Define the Symbols used:

'===============================================================================
'SM2_STATE12:
'===============================================================================
':::-MAIN CODE-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


':::-EXIT MAIN-:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
'===============================================================================
'    SM2_Return() 
'*******************************************************************************



'*******************************************************************************
'###############################################################################
$endif
' End of SM2 Code.
'##### STATE MACHINE 2 END #####################################################
'###############################################################################


